/*--------------------------------------------------------------------------
**      WLists.h                Twin           From:  Twin/dlls/commctrl
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc. 
**------------------------ < File Content Description > --------------------
**
**  Module:	dlls/commctrl/WLists.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/dlls/commctrl/WLists.h,v 1.2 1999/07/12 15:38:21 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/

#ifndef __WLIST_H__
#define __WLIST_H__



typedef struct _WLIST
{
	HGLOBAL                   hData;
	LPVOID                    lpData;
	UINT                      ElementSize;
	UINT                      ElementCount;
	UINT                      AllocatedElements;
	UINT                      AllocationSize;
	UINT                      LockCount;
} WLIST;

#define WBASE_LIST                2000
#define WLERR_NULLHANDLE          ( -WBASE_LIST - 1 )
#define WLERR_NULLELEMENTSIZE     ( -WBASE_LIST - 2 )
#define WLERR_GLOBALALLOC         ( -WBASE_LIST - 3 )
#define WLERR_LOCKED              ( -WBASE_LIST - 4 )
#define WLERR_OUTOFBOUNDS         ( -WBASE_LIST - 5 )
#define WLERR_GLOBALREALLOC       ( -WBASE_LIST - 6 )
#define WLERR_GLOBALLOCK          ( -WBASE_LIST - 7 )
#define WLERR_NOTLOCKED           ( -WBASE_LIST - 8 )

#define WLWARN_NULLHANDLE         ( WBASE_LIST + 1 )
#define WLWARN_LOCKED             ( WBASE_LIST + 2 )


/*----- Create a List --------------------------------------------------------*/
int WINAPI
WLCreate
(
	WLIST*                    lpList,
	UINT                      ElementSize
);
/*----- Insert and element into a list ---------------------------------------*/
int WINAPI
WLInsertEx
(
	WLIST*                    lpList,
	UINT                      IndexBefore,
	LPVOID                    lpElementData
);
/*----- Insert and element into a list ---------------------------------------*/
int WINAPI
WLInsert
(
	WLIST*                    lpList,
	UINT                      IndexBefore,
	LPVOID                    lpElementData
);
/*----- Delete an element from a list ----------------------------------------*/
int WINAPI
WLDelete
(
	WLIST*                    lpList,
	UINT                      Index
);
/*----- Lock the list and get a pointer to the data --------------------------*/
int WINAPI
WLLock
(
	WLIST*                    lpList,
	LPVOID*                   lplpData
);
/*----- UnLock the list and set a pointer to the data to NULL ----------------*/
int WINAPI
WLUnlock
(
	WLIST*                    lpList,
	LPVOID*                   lplpData
);
/*----- Destory the list even if it's locked ---------------------------------*/
int WINAPI
WLDestroy
(
	WLIST*                    lpList
);

/*----- Get the number of elements in the list -------------------------------*/
int WINAPI
WLCount
(	
	WLIST                     *lpList,
	UINT                      *lpCount
);

/*----- Allocate space large enough to house certain amount of elements -----*/
int WINAPI
WLSetSize
(
	WLIST*                    lpList,
	int                       iElements
);
#endif /* __WLIST_H__ */

/*------------------------- < Full Revision History > ----------------------
** $Log: WLists.h,v $
** Revision 1.2  1999/07/12 15:38:21  mwalsh
** Updated comments to standard header and footer style
**
**-------------------------------------------------------------------------*/