/*--------------------------------------------------------------------------
**      About.c                 Twin           From:  Twin/dlls/shell
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	dlls/shell/About.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/dlls/shell/About.c,v 1.2 1999/07/12 19:35:38 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <ctype.h>
#include <stdio.h>
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/

#include "windows.h"
#include "windowsx.h"
#include "commdlg.h"
#include "Log.h"
#include "Dialog.h"
#include "About.h"
#include "Strings32.h"
#include "version.h"
#include "Release.h"

static BOOL ShellAboutHandler(HWND, unsigned, WPARAM, LPARAM);

void
ShellAbout(HWND hWnd, LPCSTR lpszCaption, LPCSTR lpszAboutText,
		HICON hIcon)
{
	FARPROC	lpProc;
	SHELLABOUTDATA sad;
	HINSTANCE  hInst;
	int     rc;
	

	hInst = LoadLibrary("SHELL");

	lpProc= MakeProcInstance((FARPROC)ShellAboutHandler,hInst);

	sad.lpszText    = (LPSTR)lpszAboutText;
	sad.lpszCaption = (LPSTR) lpszCaption;
	sad.hIcon = hIcon;

	rc = DialogBoxParam(
	        hInst, 
		"ShellAbout",
		hWnd,
		lpProc,
		(LPARAM)&sad		
		);

	FreeProcInstance(lpProc); 
	FreeLibrary(hInst);
}

#ifdef TWIN_BIN32
void
ShellAboutA(HWND hWnd, LPCSTR lpszCaption, LPCSTR lpszAboutText, HICON hIcon)
{
	ShellAbout(hWnd, lpszCaption, lpszAboutText, hIcon);
}

void
ShellAboutW(HWND hWnd, LPWSTR lpszCaption, LPWSTR lpszAboutText, HICON hIcon)
{
	char *lpcap;
	char *lptxt;

	lpcap = WinStrdupW(lpszCaption);
	lptxt = WinStrdupW(lpszAboutText);

	ShellAbout(hWnd, lpcap, lptxt, hIcon);
	WinFree(lpcap);
	WinFree(lptxt);
}
#endif

#ifdef DEBUG
#define  TWINDBG "debug"
#else
#define  TWINDBG "optimized"
#endif

static BOOL FAR PASCAL
ShellAboutHandler(HWND hDlg, unsigned msg, WPARAM wParam, LPARAM lParam)
{
	SHELLABOUTDATA *sad;
	char		abouttext[256];
   	DWORD version;
   	char  verstr[80];
	char  *ver;

	switch(msg) {
	case WM_INITDIALOG:
		sad = (SHELLABOUTDATA *) lParam;
		
		/************************************************/
		/*	Caption first				*/
		/************************************************/

		SendDlgItemMessage (hDlg, IDC_ABOUT, WM_SETTEXT, 0,
		                ( LONG )sad->lpszCaption);

		ver = sad->lpszText;
		if(ver == 0 || strlen(ver) == 0) {
			version = GetVersion();

			wsprintf(verstr,"Version %d.%d (build %d: %s)",
				LOBYTE(LOWORD(version)), 
				HIBYTE(LOWORD(version)), 
				HIWORD(version),VER_PATCH);
			ver = verstr;
		}
					
		SendDlgItemMessage (hDlg, IDC_VERSION, WM_SETTEXT, 0, 
			(LONG) ver);
		SendDlgItemMessage (hDlg, IDC_COPYRITE, WM_SETTEXT, 0,
			(LONG)"Copyright (C) 1999 CodeWeavers, Inc.");

		/************************************************/
		/*	Twin specific 4 lines			*/
		/************************************************/

		gethostname(abouttext,256);
		/* add any shell about specific string */
		SendDlgItemMessage(hDlg, IDC_HOST, WM_SETTEXT, 0,
			(LONG) abouttext);
		sprintf(abouttext,"%s",getenv("DISPLAY"));
		SendDlgItemMessage(hDlg, IDC_DISPLAY, WM_SETTEXT, 0,
			(LONG) abouttext);

		/************************************************/
		/*	Icon passed in?				*/
		/************************************************/
		if (sad->hIcon) 
	    		SendDlgItemMessage(hDlg, (int) IDC_ICON, STM_SETICON,
				(WPARAM) sad->hIcon, 0L);

		return TRUE;

	case WM_COMMAND:
		switch ( GET_WM_COMMAND_ID(wParam, lParam) )
		{
			case IDOK:
				EndDialog(hDlg, IDOK);
				break;
			default:
				return ( FALSE );
		}
		return ( TRUE );
	}
	return FALSE;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: About.c,v $
** Revision 1.2  1999/07/12 19:35:38  mwalsh
** Updated to utilize standard comment headers and footers.
**
**-------------------------------------------------------------------------*/