/*--------------------------------------------------------------------------
**      DragDrop.c                 Twin           From:  Twin/dlls/shell
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	dlls/shell/DragDrop.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/dlls/shell/DragDrop.c,v 1.2 1999/07/12 19:35:38 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "windowsx.h"

#include "Log.h"
#include "DragDrop.h"

void WINAPI
DragAcceptFiles(HWND hWnd, BOOL fAccept)
{
    DWORD dwExStyle;

    APISTR((LF_APICALL,"DragAcceptFiles(HWND=%x,BOOL=%d)\n",
	hWnd,fAccept));

    if (!IsWindow(hWnd)) {
        APISTR((LF_APIRET,"DragAcceptFiles: returns void\n"));
	return;
    }

    dwExStyle = GetWindowExStyle(hWnd);

    if (fAccept)
	dwExStyle |= WS_EX_ACCEPTFILES;
    else
	dwExStyle &= ~WS_EX_ACCEPTFILES;

    SetWindowLong(hWnd,GWL_EXSTYLE,dwExStyle);
    APISTR((LF_APIRET,"DragAcceptFiles: returns void\n"));
}

UINT WINAPI
DragQueryFile(HDROP hDrop, UINT iFile, LPSTR lpszFile, UINT cb)
{
    APISTR((LF_APISTUB,"DragQueryFile(HDROP=%x,UINT=%x,LPSTR=%s,UINT=%d)\n",
	hDrop,iFile,lpszFile?lpszFile:"NULL",cb));

    return 0;
}

UINT WINAPI
DragQueryFileA(HDROP hDrop, UINT iFile, LPSTR lpszFile, UINT cb)
{
    APISTR((LF_APISTUB,"DragQueryFileA(HDROP=%x,UINT=%x,LPSTR=%s,UINT=%d)\n",
	hDrop,iFile,lpszFile?lpszFile:"NULL",cb));

    return 0;
}

BOOL WINAPI
DragQueryPoint(HDROP hDrop, POINT *lppt)
{
#ifdef	LATER
    HGLOBAL hGlobal = (HGLOBAL)hDrop;
    LPDRAGINFO lpDragInfo;

    APISTR((LF_APICALL,"DragQueryPoint(HDROP=%x,POINT=%x)\n",
	hDrop,lppt));

    if (NULL == (lpDragInfo = (LPDRAGINFO)GlobalLock(hGlobal))) {
        APISTR((LF_APICALL,"DragQueryPoint: returns BOOL FALSE\n"));
	return FALSE;
    }

    lppt->x = lpDragInfo->x;
    lppt->y = lpDragInfo->y;

    GlobalUnlock(hGlobal);

    APISTR((LF_APICALL,"DragQueryPoint: returns BOOL TRUE\n"));
    return TRUE;
#else
    APISTR((LF_APISTUB,"DragQueryPoint(HDROP=%x,POINT=%x)\n",
	hDrop,lppt));
    return FALSE;
#endif
}

void WINAPI
DragFinish(HDROP hDrop)
{
    HGLOBAL hGlobal = (HGLOBAL)hDrop;

    APISTR((LF_APICALL,"DragFinish(HDROP=%x)\n",hDrop));

    GlobalFree(hGlobal);

    APISTR((LF_APIRET,"DragFinish: returns void\n"));
}

/*------------------------- < Full Revision History > ----------------------
** $Log: DragDrop.c,v $
** Revision 1.2  1999/07/12 19:35:38  mwalsh
** Updated to utilize standard comment headers and footers.
**
**-------------------------------------------------------------------------*/