/*--------------------------------------------------------------------------
**      Registry.c                 Twin           From:  Twin/dlls/shell
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	dlls/shell/Registry.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/dlls/shell/Registry.c,v 1.9 2000/01/28 20:21:33 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "windowsx.h"
#include "Log.h"
#include "kerndef.h"
#include "KrnAtoms.h"

static ATOMTABLE AtomTable;

/* one value of a key */
typedef struct tagKEYVALUE
{
    LPSTR   name;          /* name of value (UNICODE) or NULL for win31 */
    DWORD    type;          /* type of value */
    DWORD    len;           /* length of data in BYTEs */
    DWORD    lastmodified;  /* time of seconds since 1.1.1970 */
    LPBYTE   data;          /* content, may be strings, binaries, etc. */
} KEYVALUE,*LPKEYVALUE;

typedef struct HKEY__ {
	HKEY hParentKey;	/* our parent */
	ATOM atomKey;		/* what is this key string */
	BOOL fOpen;		/* are we open? */
	HKEY hSubKey;		/* next subkey */
	HKEY hNext;		/* next Key */
	LPSTR lpszValue;	/* the value...*/
	int   nValues;		/* how many values contained int values...*/
	int   len;		/* size of data */
	int   type;		/* type of data */
	KEYVALUE *values;	/* win32 registry values...*/
} KEYSTRUCT;

typedef KEYSTRUCT *LPKEYSTRUCT;

static KEYSTRUCT RootKey;
static KEYSTRUCT LocalKey;
static KEYSTRUCT UsersKey;
static KEYSTRUCT CurrentUsersKey;
static KEYSTRUCT DynamicKey;
static KEYSTRUCT ConfigKey;

static LPKEYSTRUCT InternalFindKey(LPKEYSTRUCT,LPCSTR,WORD);
static LPKEYSTRUCT InternalCreateKey(LPKEYSTRUCT,ATOM);

static void ReadSetupReg();

#define IFK_FIND	0
#define IFK_CREATE	1

#define DEBUGBREAK(s)	{ logstr(LF_CONSOLE,"RegistryBreak %s",s);debuggerbreak();}

static void
InitKey(LPKEYSTRUCT lpKeyStruct,char *atomstring)
{
    	if(lpKeyStruct->fOpen)
		return;

 	lpKeyStruct->hParentKey = (HKEY)0L;
    	if (!lpKeyStruct->atomKey) {
		lpKeyStruct->atomKey = AddAtomEx(&AtomTable,atomstring);
	}
    	lpKeyStruct->fOpen = TRUE;
}

LPKEYSTRUCT 
GetHKEY(HKEY hKey)
{
    LPKEYSTRUCT lpKeyStruct;
    static int  InitRegistry;

    if(InitRegistry == 0) {
	InitRegistry++;
    	ReadSetupReg();
    }

    switch((int) hKey) {
        case HKEY_CLASSES_ROOT:
		lpKeyStruct = &RootKey;
		InitKey(lpKeyStruct,"HKEY_CLASSES_ROOT");
		break;

	case HKEY_LOCAL_MACHINE:
		lpKeyStruct = &LocalKey;
		InitKey(lpKeyStruct,"HKEY_LOCAL_MACHINE");
		break;
	case HKEY_USERS:
		lpKeyStruct = &UsersKey;
		InitKey(lpKeyStruct,"HKEY_USERS");
		break;
	case HKEY_CURRENT_USER:
		lpKeyStruct = &CurrentUsersKey;
		InitKey(lpKeyStruct,"HKEY_CURRENT_USER");
		break;
	case HKEY_DYN_DATA:
		lpKeyStruct = &DynamicKey;
		InitKey(lpKeyStruct,"HKEY_DYN_DATA");
		break;
	case HKEY_CURRENT_CONFIG:
		lpKeyStruct = &ConfigKey;
		InitKey(lpKeyStruct,"HKEY_CURRENT_CONFIG");
		break;
	default:
		lpKeyStruct = (KEYSTRUCT *)hKey;
		if (!lpKeyStruct || !lpKeyStruct->fOpen) {
	    		return (LPKEYSTRUCT) 0;

		}
    }
    if (!lpKeyStruct) {
	return (LPKEYSTRUCT) 0;
    }
    return lpKeyStruct;
}

LONG RegCreateKeyEx(
    HKEY hKey,	// handle of an open key 
    LPCTSTR lpszSubKey,	// address of subkey name 
    DWORD Reserved,	// reserved 
    LPTSTR lpClass,	// address of class string 
    DWORD dwOptions,	// special options flag 
    REGSAM samDesired,	// desired security access 
    LPSECURITY_ATTRIBUTES lpSecurityAttributes,	// address of key security structure 
    PHKEY phkResult,	// address of buffer for opened handle  
    LPDWORD lpdwDisposition 	// address of disposition value buffer 
   )
{
    LPKEYSTRUCT lpKeyStruct;
    LPKEYSTRUCT lpSubKey;
    DWORD	bFound = REG_OPENED_EXISTING_KEY;
    LONG 	dwReturn = ERROR_BADKEY;

    APISTR((LF_APICALL,"RegCreateKeyEx(HKEY=%x,LPCSTR=%s,DWORD=%x,CLASS=%p,DWORD=%x,REGSAM=%x,LPSECURITY=%p,PHKEY=%x,LPDWORD=%p)\n",
		hKey,HIWORD(lpszSubKey)?lpszSubKey:"NULL",
		Reserved,lpClass,dwOptions,samDesired,
		lpSecurityAttributes,
		phkResult,lpdwDisposition));

    if((lpKeyStruct = GetHKEY(hKey)) == 0) {
    	APISTR((LF_APIRET,"RegCreateKey: returns LONG %d\n",dwReturn));
	return dwReturn;
    }

    lpSubKey = InternalFindKey(lpKeyStruct,lpszSubKey,IFK_FIND);
    if(lpSubKey == 0) {
    	lpSubKey = InternalFindKey(lpKeyStruct,lpszSubKey,IFK_CREATE);
	bFound   = REG_CREATED_NEW_KEY;
    }

    if ((*phkResult = (HKEY)lpSubKey)) {
	dwReturn = ERROR_SUCCESS;
    }
    if(lpdwDisposition) {
	*lpdwDisposition = bFound;
    }

    APISTR((LF_APIRET,"RegCreateKeyEx: returns LONG %d (%p)\n",
	dwReturn,lpSubKey));
    return dwReturn;
}

LONG WINAPI
RegCreateKey(HKEY hKey, LPCSTR lpszSubKey, PHKEY phkResult)
{
    LONG dwReturn;
    LONG dwDisposition;
    APISTR((LF_APICALL,"RegCreateKey(HKEY=%x,LPCSTR=%s,PHKEY=%x)\n",
		hKey,HIWORD(lpszSubKey)?lpszSubKey:"NULL",phkResult));

    dwReturn = RegCreateKeyEx(hKey,
		lpszSubKey,
		0,
		"",
		REG_OPTION_NON_VOLATILE,
		KEY_ALL_ACCESS,
		NULL,
		phkResult,
		&dwDisposition);

    APISTR((LF_APIRET,"RegCreateKey: returns LONG %d\n",dwReturn));
    return dwReturn;
}

LONG WINAPI
RegOpenKeyEx
(
  HKEY  hKey,           /* handle of open key */
  LPCTSTR  lpszSubKey,    /* address of name of subkey to open */
  DWORD  ulOptions,     /* reserved */
  REGSAM  samDesired,   /* security access mask */
  PHKEY  phkResult        /* address of handle of open key */
)
{
    KEYSTRUCT *lpKeyStruct;
    LPKEYSTRUCT lpSubKey;
    LONG rc = ERROR_BADKEY;

    APISTR((LF_APICALL,
	"RegOpenKeyEx(HKEY=%x,LPCSTR=%s,DWORD=%x,REGSAM=%p,PHKEY=%x)\n",
		hKey,HIWORD(lpszSubKey)?lpszSubKey:"NULL",ulOptions,samDesired,
		phkResult));

    if((lpKeyStruct = GetHKEY(hKey)) == 0) {
    	APISTR((LF_APIRET,"RegOpenKeyEx: returns LONG %d (error_badkey)\n",
		rc));
	return rc;
    }

    lpSubKey = InternalFindKey(lpKeyStruct,lpszSubKey,IFK_FIND);
    if(lpSubKey == 0)
            rc = ERROR_FILE_NOT_FOUND;

    if (phkResult) { 
	if((*phkResult = (HKEY)lpSubKey)) {
		rc = ERROR_SUCCESS;
    	}
    } 

    APISTR((LF_APIRET,"RegOpenKeyEx: returns LONG %d (%p)\n",rc,lpSubKey));
    return rc;
}

LONG WINAPI
RegOpenKey(HKEY hKey, LPCSTR lpszSubKey, PHKEY phkResult)
{
	LONG rc;
    	APISTR((LF_APICALL,"RegOpenKey(HKEY=%x,LPCSTR=%s,PHKEY=%x)\n",
		hKey,HIWORD(lpszSubKey)?lpszSubKey:"NULL",phkResult));

  	rc = RegOpenKeyEx(hKey,lpszSubKey,0,0,phkResult);

    	APISTR((LF_APIRET,"RegOpenKey: returns LONG %d\n",rc));
	return rc;
}

void
DEBUG_EnumRegistry(HKEY hKey)
{
    KEYSTRUCT *lpKeyStruct;
 
    if(hKey == 0)
	hKey = HKEY_CLASSES_ROOT;

    if((lpKeyStruct = GetHKEY(hKey)) == 0) {
	LOGSTR((LF_CONSOLE,"EnumRegistry: bad hKey=%x\n",hKey));
	return;
    }

    LOGSTR((LF_CONSOLE,"EnumRegistry: hKey=%x %p\n",hKey,lpKeyStruct));

    for (lpKeyStruct = (KEYSTRUCT *)lpKeyStruct->hSubKey; 
		lpKeyStruct;
		lpKeyStruct = (KEYSTRUCT *)lpKeyStruct->hNext) {
    	char buf[256];
	
    	if (GetAtomNameEx(&AtomTable,lpKeyStruct->atomKey,buf,sizeof(buf)))
		LOGSTR((LF_CONSOLE,"%p -> atom(%x)=%s value=%s\n",
			lpKeyStruct,
			lpKeyStruct->atomKey,buf,
			lpKeyStruct->lpszValue));
    }
}

LONG WINAPI
RegEnumKey(HKEY hKey, DWORD iSubKey, LPSTR lpszBuffer, DWORD cbBuffer)
{
    KEYSTRUCT *lpKeyStruct;
    char buf[256];
    int i;

    APISTR((LF_APICALL,"RegEnumKey(HKEY=%x,DWORD=%x,LPSTR=%x,DWORD=%x)\n",
	hKey,iSubKey,lpszBuffer,cbBuffer));

    if((lpKeyStruct = GetHKEY(hKey)) == 0) {
    	APISTR((LF_APIRET,"RegEnumKey: returns LONG %d\n",ERROR_BADKEY));
	return (LONG) ERROR_BADKEY;
    }

    for (i=0,lpKeyStruct = (KEYSTRUCT *)lpKeyStruct->hSubKey; 
		i< iSubKey && lpKeyStruct;
		i++,lpKeyStruct = (KEYSTRUCT *)lpKeyStruct->hNext);

    if ((i != iSubKey) || (lpKeyStruct == NULL)) {
    	APISTR((LF_APIRET,"RegEnumKey: returns LONG %d\n",ERROR_BADKEY));
	return (LONG) ERROR_BADKEY;
    }

    if (GetAtomNameEx(&AtomTable,lpKeyStruct->atomKey,buf,sizeof(buf)))
	strncpy(lpszBuffer,buf,min(cbBuffer,strlen(buf)+1));
    else
	lpszBuffer[0] = '\0';

    APISTR((LF_APIRET,"RegEnumKey: returns LONG %d\n",ERROR_SUCCESS));
    return ERROR_SUCCESS;
}

LONG WINAPI
RegQueryValue(HKEY hKey, LPCSTR lpszSubKey, LPSTR lpszValue, LONG *lpcb)
{
    KEYSTRUCT *lpKeyStruct;
    KEYSTRUCT *lpSubKeyStruct;

    APISTR((LF_APICALL,"RegQueryValue(HKEY=%x,LPCSTR=%s,LPSTR=%x,LONG *%x)\n",
		hKey,(lpszSubKey)?lpszSubKey:"NULL",lpszValue,lpcb));

    if((lpKeyStruct = GetHKEY(hKey)) == 0) {
    	APISTR((LF_APIRET,"RegQueryValue: returns LONG %d\n",ERROR_BADKEY));
	return (LONG) ERROR_BADKEY;
    }

    lpSubKeyStruct = InternalFindKey(lpKeyStruct,lpszSubKey,IFK_FIND);

    if (!lpSubKeyStruct) {
    	APISTR((LF_APIRET,"RegQueryValue: returns LONG %d\n",ERROR_BADKEY));
	return (LONG) ERROR_BADKEY;
    }

    if (lpSubKeyStruct->lpszValue && (*lpSubKeyStruct->lpszValue != 0)) {
	strncpy(lpszValue,lpSubKeyStruct->lpszValue,min(*lpcb,
		(int)strlen(lpSubKeyStruct->lpszValue)+1));
	*lpcb = strlen(lpszValue) + 1;
    }
    else {
	lpszValue[0] = '\0';
	*lpcb = 0;
    }
    
    APISTR((LF_APIRET,"RegQueryValue: returns LONG %d\n",ERROR_SUCCESS));
    return ERROR_SUCCESS;
}

LONG WINAPI
RegQueryValueEx
(
  HKEY hKey,
  LPCSTR lpszValue,
  LPDWORD lpdwReserved,
  LPDWORD lpType,
  LPBYTE lpbData,
  LPDWORD lpcbData
)
{
    	KEYSTRUCT 	*lpKeyStruct;
    	KEYSTRUCT 	*lpSubKeyStruct;

    	APISTR((LF_APICALL,"RegQueryValueEx(%p,%s,%d,%p,%p,%p)\n",
		hKey,lpszValue?lpszValue:"NULL",lpdwReserved,
		lpType,lpbData,lpcbData));

        if((lpKeyStruct = GetHKEY(hKey)) == 0) {
    		APISTR((LF_APIRET,"RegQueryValueEx: returns LONG %d\n",
			ERROR_BADKEY));
		return (LONG) ERROR_BADKEY;
	}

	if ((lpbData && ! lpcbData) || lpdwReserved) {
    		APISTR((LF_APIRET,"RegQueryValueEx: returns LONG %d\n",
			ERROR_INVALID_PARAMETER));
	  	return ERROR_INVALID_PARAMETER;
	}
    
	if(lpszValue) {
		lpSubKeyStruct = InternalFindKey(lpKeyStruct,lpszValue,IFK_FIND);

    		if (!lpSubKeyStruct) {
    			APISTR((LF_APIRET,"RegQueryValueEx: returns LONG %d\n",
				ERROR_BADKEY));
			return (LONG) ERROR_BADKEY;
		}

    		if (lpSubKeyStruct->lpszValue && 
			(*lpSubKeyStruct->lpszValue != 0)) {
			strncpy(lpbData,lpSubKeyStruct->lpszValue,
				min(*lpcbData, (int)strlen(lpSubKeyStruct->lpszValue)+1));
			if(lpcbData)
				*lpcbData = strlen(lpszValue) + 1;
  			if(lpType)
				*lpType = REG_SZ;
    		}
    		else {
			lpbData[0] = '\0';
			if(lpcbData)
				*lpcbData = 0;
  			if(lpType)
				*lpType = REG_SZ;
    		}

   	} 
#if 0
	/* An empty name string is equivalent to NULL */
	if (lpszValue&& !*lpszValue)
	  lpszValue = NULL;

	if (lpszValue==NULL) { 
	  /* Use key's unnamed or default value, if any */
	  for (i=0;i<lpkey->nrofvalues;i++)
	    if (lpkey->values[i].name==NULL)
	      break;
	} 
	else 
	{ /* Search for the key name */
	  for (i=0;i<lpkey->nrofvalues;i++)
	    if ( lpkey->values[i].name && !lstrcmpiW(lpszValue,lpkey->values[i].name))
	      break;
	}
	if (i==lpkey->nrofvalues) 
	{ 
	  if (lpValueName==NULL) 
	  { /* Empty keyname not found */
	    if (lpbData) 
	    { *(WCHAR*)lpbData = 0;
	      *lpcbData	= 2;
	    }
	    if (lpdwType)
	      *lpdwType	= REG_SZ;
	    return ERROR_SUCCESS;
	  }
	  
	  return ERROR_FILE_NOT_FOUND;
	}
	ret = ERROR_SUCCESS;

	if (lpdwType) 					/* type required ?*/
	  *lpdwType = lpkey->values[i].type;

	if (lpbData)					/* data required ?*/
	{ if (*lpcbData >= lpkey->values[i].len)	/* buffer large enought ?*/
	    memcpy(lpbData,lpkey->values[i].data,lpkey->values[i].len);
	  else {
	    *lpcbData = lpkey->values[i].len;
	    ret = ERROR_MORE_DATA;
	  }
	}

	if (lpcbData) 					/* size required ?*/
	{ *lpcbData = lpkey->values[i].len;
	}
#endif

    APISTR((LF_APIRET,"RegQueryValueEx returns %d\n",ERROR_SUCCESS));
    return (LONG) ERROR_SUCCESS;
}


LONG WINAPI
RegSetValue(HKEY hKey, LPCSTR lpszSubKey,
		DWORD fdwType, LPCSTR lpszValue, DWORD cb)
{
    KEYSTRUCT *lpKeyStruct;
    KEYSTRUCT *lpSubKeyStruct;

    APISTR((LF_APICALL,
	"RegSetValue(HKEY=%x,LPCSTR=%s,DWORD=%x,LPCSTR=%s,DWORD=%x)\n",
		hKey,lpszSubKey?lpszSubKey:"NULL",fdwType,
		HIWORD(lpszValue)?lpszValue:"NULL",cb));

    if((lpKeyStruct = GetHKEY(hKey)) == 0) {
    	APISTR((LF_APIRET,"RegSetValue: returns LONG %d\n",ERROR_BADKEY));
	return (LONG) ERROR_BADKEY;
    }

    lpSubKeyStruct = InternalFindKey(lpKeyStruct,lpszSubKey,IFK_FIND);

    if (!lpSubKeyStruct) {
        APISTR((LF_APIRET,"RegSetValue: returns LONG %d\n",ERROR_BADKEY));
	return (LONG) ERROR_BADKEY;
    }

    if (lpSubKeyStruct->lpszValue)
	WinFree(lpSubKeyStruct->lpszValue);

    if (lpszValue && (*lpszValue != 0)) {
	lpSubKeyStruct->lpszValue = (LPSTR) WinMalloc(strlen(lpszValue)+1);
	strcpy(lpSubKeyStruct->lpszValue,lpszValue);
    }
    
    APISTR((LF_APIRET,"RegSetValue: returns LONG %d\n",ERROR_SUCCESS));
    return ERROR_SUCCESS;
}

LONG RegSetValueEx(
    HKEY hKey,	// handle of key to set value for  
    LPCTSTR lpszValue,	// address of value to set 
    DWORD Reserved,	// reserved 
    DWORD dwType,	// flag for value type 
    CONST BYTE *lpData,	// address of value data 
    DWORD cbData 	// size of value data 
   )
{
	LONG rc = ERROR_BADKEY;
        KEYSTRUCT *lpKeyStruct;

    	APISTR((LF_APICALL,"RegSetValueEx(%p,%s,%d,%d,%p,%d)\n",
		hKey,lpszValue,Reserved,dwType,lpData,cbData));

    	if((lpKeyStruct = GetHKEY(hKey)) == 0) {
    		APISTR((LF_APIRET,"RegSetValueEx: returns LONG %d\n",rc));
		return rc;
    	}

	if(lpszValue == NULL || strlen(lpszValue) == 0) {
		/* default data stored in KEYSTRUCT */
    		if (lpKeyStruct->lpszValue) {
			WinFree(lpKeyStruct->lpszValue);
			lpKeyStruct->lpszValue = 0;
		}
		/* allocate space, and copy in */
		lpKeyStruct->lpszValue = (LPSTR) WinMalloc(cbData);
		memcpy(lpKeyStruct->lpszValue,lpData,cbData);
		lpKeyStruct->len       = cbData;
		lpKeyStruct->type      = dwType;
		rc = ERROR_SUCCESS;
	} else {
		/* go through named values for key... */
		/* if we found the value, clear its current data */
		int i;
		LPKEYVALUE lpkv;
		for(i = 0;i<lpKeyStruct->nValues;i++) {
			lpkv = &lpKeyStruct->values[i];
			if(strcasecmp(lpszValue,lpkv->name) == 0) {
				WinFree(lpkv->data);
				lpkv->data = 0;
				break;
			}
		}
		/* we went through the loop, not finding the value */
		if(i == lpKeyStruct->nValues) {
			/* need to allocate a NEW KEYSTRUCT */
			lpKeyStruct->nValues++;

			/* either a new block, or extend the current one */
			if(lpKeyStruct->values == 0)
				lpKeyStruct->values = WinMalloc(lpKeyStruct->nValues * sizeof(KEYVALUE));
			else
				lpKeyStruct->values = WinRealloc(lpKeyStruct->values,lpKeyStruct->nValues * sizeof(KEYVALUE));

			/* new ones need to have their names saved */	
			lpKeyStruct->values[i].name = WinStrdup(lpszValue);
		}

		/* overwrite/update the current KEYVALUE */
		lpKeyStruct->values[i].type = dwType;
		lpKeyStruct->values[i].len  = cbData;
		lpKeyStruct->values[i].data =  WinMalloc(cbData);

		/* copy the data in */
		memcpy(lpKeyStruct->values[i].data,lpData,cbData);
		rc = ERROR_SUCCESS;
	}

    	APISTR((LF_APIRET,"RegSetValueEx: returns LONG %d\n",rc));
    	return rc;
}


LONG RegDeleteValue(
    HKEY hKey,	// handle of key 
    LPCTSTR lpszValue // address of value name 
   )
{
	APISTR((LF_APISTUB,"RegDeleteValue(%x,%p)\n",hKey,lpszValue));
    	DEBUGBREAK("RegDeleteValue");
	return 0;
}

LONG WINAPI
RegCloseKey(HKEY hKey)
{
    KEYSTRUCT *lpKeyStruct;

    APISTR((LF_APICALL,"RegCloseKey(HKEY=%x)\n",hKey));

    if((lpKeyStruct = GetHKEY(hKey)) == 0) {
    	APISTR((LF_APIRET,"RegOpenKey: returns LONG %d\n",ERROR_BADKEY));
	return (LONG) ERROR_BADKEY;
    }

    lpKeyStruct->fOpen = 0;

    APISTR((LF_APIRET,"RegCloseKey: returns LONG %d\n",ERROR_SUCCESS));
    return (LONG)ERROR_SUCCESS;
}

LONG WINAPI
RegDeleteKey(HKEY hKey, LPCSTR lpszSubKey)
{
    LPKEYSTRUCT lpKeyStruct,lpKeyParent,lpKeyTmp;

    APISTR((LF_APICALL,"RegDeleteKey(HKEY=%x,LPCSTR=%x)\n",
	hKey,lpszSubKey));

    lpKeyStruct = InternalFindKey((LPKEYSTRUCT)hKey,lpszSubKey,IFK_FIND);

    if (!lpKeyStruct || !lpKeyStruct->fOpen) {
    	APISTR((LF_APIRET,"RegDeleteKey: returns LONG %d\n",ERROR_BADKEY));
	return ERROR_BADKEY;
    }

    if (lpKeyStruct == &RootKey) {
    	APISTR((LF_APIRET,"RegDeleteKey: returns LONG %d\n",
		ERROR_ACCESS_DENIED));
	return ERROR_ACCESS_DENIED;
    }

    if (lpKeyStruct->lpszValue)
	WinFree(lpKeyStruct->lpszValue);

    while (lpKeyStruct->hSubKey) 
	RegDeleteKey(lpKeyStruct->hSubKey,"");

    lpKeyParent = (LPKEYSTRUCT)lpKeyStruct->hParentKey;

    if (lpKeyParent->hSubKey == (HKEY)lpKeyStruct)
	lpKeyParent->hSubKey = lpKeyStruct->hNext;
    else {
	for (lpKeyTmp = (LPKEYSTRUCT)lpKeyParent->hSubKey;
			lpKeyTmp;
			lpKeyTmp = (LPKEYSTRUCT)lpKeyTmp->hNext) 
	    if (lpKeyTmp->hNext == (HKEY)lpKeyStruct)
		lpKeyTmp->hNext = lpKeyStruct->hNext;
    }

    DeleteAtomEx(&AtomTable,lpKeyStruct->atomKey);

    WinFree((LPSTR)lpKeyStruct);

    APISTR((LF_APIRET,"RegDeleteKey: returns LONG %d\n",ERROR_SUCCESS));
    return ERROR_SUCCESS;
}

LONG WINAPI
RegDeleteKeyW(HKEY hKey, LPCWSTR lpszSubKey)
{
    APISTR((LF_APISTUB,"RegOpenKeyW(...)\n"));
    DEBUGBREAK("RegDeleteKeyW");
    return 0;
}

/******************************************************************************/

static void
ReadSetupReg()
{
    char buf[_MAX_PATH];
    HFILE hf;
    LPSTR lpBuffer,ptr;
    char lpTmp[0x100];
    DWORD dwFileSize;
    int i = 0,j;
    HKEY hk;

    if (!GetWindowsDirectory(buf,_MAX_PATH)) 
	getcwd(buf,_MAX_PATH);
    lstrcat(buf,"/setup.reg");
    if ((hf = _lopen(buf,READ)) == HFILE_ERROR) {
	logstr(LF_CONSOLE,"Registration Database [%s] not loaded.\n", buf);
	return;
    }
    dwFileSize = _llseek(hf,0,2);
    lpBuffer = (LPSTR) WinMalloc(dwFileSize+2);
    _llseek(hf,0,0);
    _lread(hf,lpBuffer,dwFileSize);
    _lclose(hf);

    ptr = lpBuffer;
    while (i < dwFileSize) {
	while ((lpBuffer[i] != '\n') && (i < dwFileSize))
	    i++;
	if (lpBuffer[i] == '\n')
	    lpBuffer[i] = '\0';
	else
	    lpBuffer[i+1] ='\0';
	if (lpBuffer[i-1] == 0xd)	/* strip ^M */
	    lpBuffer[i-1] = 0;
	strncpy(lpTmp,ptr,17);
	lpTmp[17] = 0;
	if (!lstrcmpi(lpTmp,"HKEY_CLASSES_ROOT")) { /* <key> <value> pair */
	    char *key = 0;
	    lstrcpy(lpTmp,ptr);
	    while (*ptr++ != '\\');
	    j = 0;
	    while (ptr[j] != ' ') j++;
	    ptr[j] = 0;
	    logstr(LF_CONSOLE,"registry: creating %s\n",ptr);
	    if (RegCreateKey((HKEY)HKEY_CLASSES_ROOT,ptr,&hk) != ERROR_SUCCESS) {
	        logstr(LF_CONSOLE,"registry: failed\n",key,ptr);
		break;
	    }
 	    key = ptr;
	    ptr += j+3;
	    if (RegSetValue(hk,NULL,REG_SZ,ptr,strlen(ptr)) != ERROR_SUCCESS)
		break;
	    logstr(LF_CONSOLE,"registry: %s %s \n",key,ptr);
	}
	ptr = &lpBuffer[i+1];
    }
    WinFree(lpBuffer);
}



static LPKEYSTRUCT
InternalFindKey(LPKEYSTRUCT lpKeyStruct, LPCSTR lpszSubKey, WORD wFlag)
{
    LPKEYSTRUCT lpKeyTmp;
    char buf[256];
    LPSTR ptr,lpTmp;
    ATOM atomSubKey;

    if (lpszSubKey)
	strcpy(buf,lpszSubKey);
    if (lpszSubKey && (strlen(lpszSubKey) != 0)) {
	ptr = buf;
	while (1) {
	    lpTmp = strchr(ptr,'\\');
	    if (lpTmp)
		*lpTmp = '\0';
	    atomSubKey = FindAtomEx(&AtomTable,ptr);
	    if (atomSubKey == 0) {
		if (wFlag == IFK_CREATE)
		    atomSubKey = AddAtomEx(&AtomTable,ptr);
		else
		    return (HKEY)0L;
	    }
	    for (lpKeyTmp = (KEYSTRUCT *)lpKeyStruct->hSubKey;
			lpKeyTmp;
			lpKeyTmp = (KEYSTRUCT *)lpKeyTmp->hNext) {
		if (lpKeyTmp->atomKey == atomSubKey)
		    break;
	    }
	    if (lpKeyTmp)
		lpKeyStruct = lpKeyTmp;
	    else 
		if (wFlag == IFK_CREATE)
		    lpKeyStruct = InternalCreateKey(lpKeyStruct,atomSubKey);
		else
		    return (HKEY)0L;
	    lpKeyStruct->fOpen = TRUE;
	    if (lpTmp)
		ptr = lpTmp + 1;
	    else
		break;
	}
    }
    return lpKeyStruct;
}

static LPKEYSTRUCT
InternalCreateKey(LPKEYSTRUCT lpKeyStruct, ATOM atomSubKey)
{
    LPKEYSTRUCT lpSubKey;
    LPKEYSTRUCT lpKeyTmp;

    lpSubKey = (KEYSTRUCT *)WinMalloc(sizeof(KEYSTRUCT));
    memset((LPSTR)lpSubKey,'\0',sizeof(KEYSTRUCT));
    if (!lpKeyStruct->hSubKey)
	lpKeyStruct->hSubKey = (HKEY)lpSubKey;
    else {
	for (lpKeyTmp = (KEYSTRUCT *)lpKeyStruct->hSubKey;
		lpKeyTmp->hNext;
		lpKeyTmp = (KEYSTRUCT *)lpKeyTmp->hNext);
	lpKeyTmp->hNext = (HKEY)lpSubKey;
    }
    lpSubKey->hParentKey = (HKEY)lpKeyStruct;
    lpSubKey->atomKey = atomSubKey;
    return lpSubKey;
}

LONG WINAPI
RegOpenKeyW(HKEY hKey, LPCWSTR lpszSubKey, PHKEY phkResult)
{
    APISTR((LF_APISTUB,"RegOpenKeyW(...)\n"));
    DEBUGBREAK("RegOpenKeyW");
    return 0;
}

LONG WINAPI
RegQueryValueW(HKEY hKey, LPCWSTR lpszSubKey, LPWSTR lpszValue, LONG *lpcb)
{
    APISTR((LF_APISTUB,"RegQueryValueW(...)\n"));
    DEBUGBREAK("RegQueryValueW");
    return 0;
}

LONG
WINAPI
RegQueryValueExW
(
  HKEY hKey,
  LPCWSTR lpszValue,
  LPDWORD lpReserved,
  LPDWORD lpType,
  LPBYTE lpData,
  LPDWORD lpcbData
)
{
    APISTR((LF_APISTUB,"RegQueryValueExW(...)\n"));
    DEBUGBREAK("RegQueryValueExW");
    return 0;
}

LONG
RegEnumKeyEx(HKEY hKey,DWORD dwSubKey,LPTSTR lpszName,LPDWORD lpcdwName,LPDWORD lpdwReserved,LPTSTR lpszClassName,LPDWORD lpcdwClassName,PFILETIME pftLastChanged)
{
    APISTR((LF_APISTUB,"RegEnumKeyExW(...)\n"));
    DEBUGBREAK("RegEnumKeyExW");
    return 0;
}
LONG
RegEnumValue(HKEY hKey,DWORD dwValueIndex,LPTSTR lpszValue,LPDWORD lpcdwValue, LPDWORD lpdwReserved,LPDWORD lpdwType,LPBYTE lpDataBuffer,LPDWORD lpcdwDataBuffer)
{
    APISTR((LF_APISTUB,"RegEnumValue(...)\n"));
    DEBUGBREAK("RegEnumValue");
    return 0;
}

LONG
RegFlushKey(HKEY hKey)
{
    APISTR((LF_APISTUB,"RegFlushKey(HKEY=%x)\n",hKey));
    return 0;
}

LONG
RegLoadKey(HKEY hKey,LPCTSTR lpszSubKey,LPCTSTR lpszFile)
{
    APISTR((LF_APISTUB,"RegLoadKey(HKEY=%x,LPCTSTR=%p,LPCTSTR=%p)\n",
	hKey,lpszSubKey,lpszFile));
    DEBUGBREAK("RegLoadKey");
    return 0;
}

LONG
RegQueryInfoKey(HKEY hKey,LPTSTR lpClass,LPDWORD lpcdwClass,LPDWORD lpdwReserved,LPDWORD lpcdwSubKeys,LPDWORD lpcdwMaxSubKey,LPDWORD lpcdwMaxClass,LPDWORD lpcdwValues,LPDWORD lpcdwMaxValueName,LPDWORD lpcdwMaxValueData,LPDWORD lpcdwSecurity,PFILETIME lpftLastWriteTime)
{
    APISTR((LF_APISTUB,"RegQueryInfoKey(...)\n"));
    DEBUGBREAK("RegQueryInfoKey");
    return 0;
}

LONG
WINAPI
RegRestoreKey (
    HKEY hKey,
    LPCSTR lpFile,
    DWORD   dwFlags
    )
{
    APISTR((LF_APISTUB,"RegRestoreKey(...)\n"));
    return 0;
}

LONG
WINAPI
RegSaveKey (
    HKEY hKey,
    LPCSTR lpFile,
    LPSECURITY_ATTRIBUTES lpSecurityAttributes
    )
{
    APISTR((LF_APISTUB,"RegSaveKey(...)\n"));
    return 0;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: Registry.c,v $
** Revision 1.9  2000/01/28 20:21:33  jwhite
** Revised debug message to look nicer.
**
** Revision 1.8  2000/01/07 20:43:44  jwhite
** Added stubs needed for PSM.
**
** Revision 1.7  1999/11/30 01:14:22  robf
** added registry functions
**
** Revision 1.5  1999/07/12 19:35:38  mwalsh
** Updated to utilize standard comment headers and footers.
**
**-------------------------------------------------------------------------*/

