/*--------------------------------------------------------------------------
**      ToolHelp.c                 Twin           From:  Twin/dlls/toolhelp
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	dlls/toolhelp/ToolHelp.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/dlls/toolhelp/ToolHelp.c,v 1.2 1999/07/12 19:35:44 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "toolhelp.h"
#include "kerndef.h"
#include "Kernel.h"
#include "Resources.h"
#include "Module.h"
#include "Log.h"



extern MODULEINFO *GetModuleFromIndex(int);

static BOOL
FillModuleEntry(MODULEENTRY *lpme,MODULEINFO *modinfo)
{
    OBJHEAD *lpObjNext;

    if (!modinfo || !lpme)
	return FALSE;
    lpme->dwSize = sizeof(MODULEENTRY);
    if (modinfo->lpModuleName)
	strcpy(&lpme->szModule[0],modinfo->lpModuleName);
    lpme->hModule = modinfo->ObjHead.hObj;
    lpme->wcUsage = 1;	/* TODO LATER -- support reference count */
    if (modinfo->lpFileName)
	strcpy(&lpme->szExePath[0],modinfo->lpFileName);
    if ((lpObjNext = (LPOBJHEAD)modinfo->ObjHead.lpObjNext))
	lpme->wNext = (WORD)lpObjNext->hObj;
    else
	lpme->wNext = 0;
    return TRUE;
}

BOOL WINAPI
ModuleFirst(MODULEENTRY *lpme)
{
    BOOL retcode;

    retcode = FillModuleEntry(lpme,GetModuleFromIndex(0));
    LOGSTR((LF_LOG,"ModuleFirst: struct size %x, we need %x\n", 
	lpme->dwSize, (unsigned int) sizeof(MODULEENTRY)));

    return retcode;
}

BOOL WINAPI
ModuleNext(MODULEENTRY *lpme)
{
    BOOL retcode = 0;

    if (lpme && lpme->wNext) {
        LPMODULEINFO modinfo = GETMODULEINFO((lpme->wNext));
	retcode = FillModuleEntry(lpme,modinfo);
	RELEASEMODULEINFO(modinfo);
    }
    return retcode;
}

HMODULE WINAPI
ModuleFindHandle(MODULEENTRY *lpme,HMODULE hModule)
{
  return(TRUE);
}

HMODULE WINAPI
ModuleFindName(MODULEENTRY *lpme,LPCSTR lpszName)
{
  return(TRUE);
}

BOOL WINAPI
MemManInfo(MEMMANINFO *lpmmi)
{
    DWORD dwSize;

    LOGSTR((LF_APISTUB,"MemManInfo: \n"));

    if (lpmmi) {
	dwSize = 2*sizeof(DWORD);

	if (dwSize >= lpmmi->dwSize)
	    return FALSE;
	dwSize += sizeof(DWORD);
	lpmmi->dwLargestFreeBlock = 0;

	if (dwSize >= lpmmi->dwSize)
	    return TRUE;
	dwSize += sizeof(DWORD);
	lpmmi->dwMaxPagesAvailable = 0x1000;

	if (dwSize >= lpmmi->dwSize)
	    return TRUE;
	dwSize += sizeof(DWORD);
	lpmmi->dwMaxPagesLockable = 0;

	if (dwSize >= lpmmi->dwSize)
	    return TRUE;
	dwSize += sizeof(DWORD);
	lpmmi->dwTotalLinearSpace = 0;

	if (dwSize >= lpmmi->dwSize)
	    return TRUE;
	dwSize += sizeof(DWORD);
	lpmmi->dwTotalUnlockedPages = 0;

	if (dwSize >= lpmmi->dwSize)
	    return TRUE;
	dwSize += sizeof(DWORD);
	lpmmi->dwFreePages = 0;

	if (dwSize >= lpmmi->dwSize)
	    return TRUE;
	dwSize += sizeof(DWORD);
	lpmmi->dwTotalPages = 0;

	if (dwSize >= lpmmi->dwSize)
	    return TRUE;
	dwSize += sizeof(DWORD);
	lpmmi->dwFreeLinearSpace = 0;

	if (dwSize >= lpmmi->dwSize)
	    return TRUE;
	dwSize += sizeof(WORD);
	lpmmi->dwSwapFilePages = 0;

	if (dwSize >= lpmmi->dwSize)
	    return TRUE;
	lpmmi->wPageSize = 0x1000;
	return TRUE;
    }
    return FALSE;
}

BOOL WINAPI
GlobalEntryHandle(GLOBALENTRY *lpge, HGLOBAL hGlobal)
{
    if (!lpge)
	return FALSE;

    LOGSTR((LF_APISTUB,"GlobalEntryHandle: hGlobal %x\n",hGlobal));

    if (lpge->dwSize != sizeof(GLOBALENTRY))
	return FALSE;

    memset((LPSTR)lpge,'\0', sizeof(GLOBALENTRY));
    lpge->dwSize = sizeof(GLOBALENTRY);

#ifdef	LATER
    /* return all correct values */
#endif
    return TRUE;
}

BOOL    WINAPI 
GlobalEntryModule(GLOBALENTRY * lpGlobal, HMODULE hModule, WORD wSeg)
{
    LOGSTR((LF_APISTUB,"GlobalEntryModule(%x,%x,%x)\n",
	lpGlobal,hModule,wSeg));
    return FALSE;
}

BOOL    WINAPI 
GlobalFirst(GLOBALENTRY * lpGlobal, WORD wFlags)
{
    LOGSTR((LF_APISTUB,"GlobalFirst(%x,%x)\n",
	lpGlobal,wFlags));
    return FALSE;
}

WORD    WINAPI 
GlobalHandleToSel(HGLOBAL hMem)
{
    LOGSTR((LF_APISTUB,"GlobalHandleToSel(%x)\n",hMem));
    return FALSE;
}

BOOL    WINAPI 
GlobalInfo(GLOBALINFO * lpGlobalInfo)
{
    LOGSTR((LF_APISTUB,"GlobalInfo(%x)\n",lpGlobalInfo));
    return FALSE;
}

BOOL    WINAPI 
GlobalNext(GLOBALENTRY * lpGlobal, WORD wFlags)
{
    LOGSTR((LF_APISTUB,"GlobalNext(%x,%x)\n",
	lpGlobal,wFlags));
    return FALSE;
}

BOOL WINAPI
TimerCount(TIMERINFO *lpti)
{
    if (!lpti)
	return FALSE;

    LOGSTR((LF_APISTUB,"TimerCount: \n"));

    return FALSE;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: ToolHelp.c,v $
** Revision 1.2  1999/07/12 19:35:44  mwalsh
** Updated to utilize standard comment headers and footers.
**
**-------------------------------------------------------------------------*/