/*--------------------------------------------------------------------------
**      DrvConfig.c           Twin           From:  Twin/graphics/x11drv
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	graphics/x11drv/DrvConfig.c
**
**  Description: Unix configuration subsystem implementation
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/graphics/x11drv/DrvConfig.c,v 1.3 2000/01/28 07:54:39 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <string.h>
#include <Driver.h>
#include <pwd.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "WinConfig.h"

/* Local prototypes */
static char *DrvGetTwinPrefsFName(void);

static DWORD DrvConfigDoNothing(LPARAM,LPARAM,LPVOID);
static DWORD DrvConfigGetDefaults(LPARAM,LPARAM,LPVOID);
static DWORD DrvConfigGetFileName(LPARAM,LPARAM,LPVOID);

DWORD DrvConfigTab(void);

/* Driver configuration defaults */
/* Twin RC file default table format: */
/* Parameter, Opcode, Section, Keyname, Default, Env, lParam, string */
TWIN_CONFIG  wcpdata[] = {
/* [boot] */
	{WCP_DISPLAY, 6,WCS_BOOT,"display",	":0.0",		"DISPLAY", 0},
	{WCP_WINDOWS, 6,WCS_BOOT,"windows",	0,		"WINDOWS", 0},
	{WCP_TEMP,    6,WCS_BOOT,"temp",	"/tmp",		"TMP",     0},
	{WCP_OPENPATH,1,WCS_BOOT,"openpath",	"",		"OPENPATH",0},

/* [windows] */
	{WCP_FONTFACE, 1,WCS_WINDOWS,"FontFace", 	"Helv",	0,	   0},
	{WCP_FONTSIZE, 1,WCS_WINDOWS,"FontSize", 	"18", 	0,	   0},
	{WCP_FONTBOLD, 1,WCS_WINDOWS,"FontWeight",   	"700", 	0,	   0},

#ifdef TWIN32
	{WCP_MENUSIZE, 1,WCS_WINDOWS,"MenuSize", 	"14", 	0,	   0},
	{WCP_MENUBOLD, 1,WCS_WINDOWS,"MenuWeight",   	"400", 	0,	   0},
	{WCP_DLGSIZE, 1,WCS_WINDOWS,"MenuSize", 	"12", 	0,	   0},
	{WCP_DLGBOLD, 1,WCS_WINDOWS,"MenuWeight",   	"700", 	0,	   0},
#else
	{WCP_MENUSIZE, 1,WCS_WINDOWS,"MenuSize", 	"18", 	0,	   0},
	{WCP_MENUBOLD, 1,WCS_WINDOWS,"MenuWeight",   	"700", 	0,	   0},
	{WCP_DLGSIZE, 1,WCS_WINDOWS,"MenuSize", 	"12", 	0,	   0},
	{WCP_DLGBOLD, 1,WCS_WINDOWS,"MenuWeight",   	"700", 	0,	   0},
#endif

	{WCP_ICONFONTFACE, 1,WCS_WINDOWS,"IconFontFace","Helv",	0,	   0},
	{WCP_ICONFONTSIZE, 1,WCS_WINDOWS,"IconFontSize","14", 	0,	   0},

	{0,0,0,0,0,0,0},
};

static DWORD
DrvConfigGetDefaults(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
	return (DWORD)wcpdata;
}

static DWORD
DrvConfigGetFileName(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
	return (DWORD)DrvGetTwinPrefsFName();
}

static DWORD
DrvConfigDoNothing(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
	return 1L;
}


static TWINDRVSUBPROC DrvConfigEntryTab[] = {
	DrvConfigDoNothing, 
	DrvConfigDoNothing, 
	DrvConfigDoNothing,
	DrvConfigGetFileName,
	DrvConfigGetDefaults
};

DWORD
DrvConfigTab(void)
{
	return (DWORD)DrvConfigEntryTab;
}

/********************************************************************
*   DrvGetTwinPrefsFName
*
*	Get the full path name of the "Twin Preferences" file.
********************************************************************/
static char *DrvGetTwinPrefsFName(void)
{
        int length;
	static char twinrc[256];
	char *p;
   	struct   stat sbuf; 	 
	struct   passwd *pw;

	/* OPTION 1: if TWINERC set, use its value */
	if((p = (char *) getenv("TWINERC"))) {
		strcat(twinrc,p);
		return twinrc;
	}

	/* OPTION 2: check to see if a twinrc in current directory */
	getcwd(twinrc,256);

        /* Add slash/backslash? */
        length = strlen( twinrc );
	p = twinrc + (length - 1 );
        if( *p != '/' && *p != '\\' )
        {
          strcat(twinrc,TWIN_SLASHSTRING);
        }
	strcat(twinrc,"twinrc");

  	/* should be mfs_stat */
	/* also check stat value for a file vs. other type. */
	if(stat(twinrc,&sbuf) == 0) {
		if(S_ISREG(sbuf.st_mode)) {
			return twinrc;
		}
	}

	/* OPTION 3: get user home directory, use .twinrc */
	p = (char *)getenv("HOME");
	if (p == 0) {
		if((pw = getpwuid(getpid()))) {
			p = pw->pw_dir;	
		}
	}
	if (p) {
		sprintf(twinrc,"%s/%s",p,TWINERC);
	}
	else 
        {
          /* If none, use current working directory */
	  getcwd(twinrc,256);

          /* Add slash/backslash? */
          length = strlen( twinrc );
	  p = twinrc + (length - 1 );
          if( *p != '/' && *p != '\\' )
          {
            strcat(twinrc,TWIN_SLASHSTRING);
          }
	  strcat(twinrc,"twinrc");
	}

	return twinrc;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: DrvConfig.c,v $
** Revision 1.3  2000/01/28 07:54:39  robf
** unused entry
**
** Revision 1.2  1999/07/08 18:53:31  mwalsh
** Updated to Standard Header and Footer comments
**
**-------------------------------------------------------------------------*/
