/*--------------------------------------------------------------------------
**      DrvDC.c               Twin           From:  Twin/graphics/x11drv
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	graphics/x11drv/DrvDC.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/graphics/x11drv/DrvDC.c,v 1.4 1999/07/08 18:53:32 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <X11/Xlib.h>
#include <X11/Xutil.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "windowsx.h"
#include "Driver.h"
#include "Log.h"
#include "DrvDP.h"
#include "GdiDC.h"
#include "DrvDC.h"
#include "mtlock.h"

static DWORD DrvDCDoNothing(LPARAM,LPARAM,LPVOID);
static DWORD DrvCreateDC(LPARAM,LPARAM,LPVOID);
static DWORD DrvDeleteDC(LPARAM,LPARAM,LPVOID);
static DWORD DrvSaveDC(LPARAM,LPARAM,LPVOID);
static DWORD DrvRestoreDC(LPARAM,LPARAM,LPVOID);
static DWORD DrvGetDCClip(LPARAM,LPARAM,LPVOID);
static DWORD DrvSetDCClip(LPARAM,LPARAM,LPVOID);
static DWORD DrvSetClipOrg(LPARAM,LPARAM,LPVOID);
static DWORD DrvSetUpdateClip(LPARAM,LPARAM,LPVOID);
static DWORD DrvSetVisRgn(LPDRIVERDC,Region);
static DWORD DrvDCSetVisRgn(LPARAM,LPARAM,LPVOID);
static DWORD DrvCombineClipRgn(LPDRIVERDC);

DWORD DrvDCTab(void);

static TWINDRVSUBPROC DrvDCEntryTab[] = {
	DrvDCDoNothing,
	DrvDCDoNothing,
	DrvDCDoNothing,
	DrvCreateDC,
	DrvDeleteDC,
	DrvSaveDC,
	DrvRestoreDC,
	DrvGetDCClip,
	DrvSetDCClip,
	DrvSetClipOrg,
	DrvSetUpdateClip,
	DrvDCSetVisRgn};

DWORD
DrvDCTab(void)
{
	return (DWORD)DrvDCEntryTab;
}

DWORD
DrvDCDoNothing(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
	return 1L;
}

static Pixmap DefaultDCPixmap = (Pixmap)0;

static DWORD
DrvCreateDC(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
    DWORD dwDCXFlags = (DWORD)dwParam1;
    Window frame_win = (Window)lpStruct;
    LPDRIVERDC lpddc;
    PRIVATEDISPLAY *dp;
    XGCValues values;
    DWORD vmask;

    lpddc = (LPDRIVERDC)TSXAllocDDC(sizeof(DRIVERDC));

    memset((LPSTR)lpddc,'\0',sizeof(DRIVERDC));
    dp = (PRIVATEDISPLAY *)GETDP();
    lpddc->dp = dp;
    lpddc->dwDCFlags = dwDCXFlags;

    /* set the drawable */
    LOCK_SECTION( _MTLOCK_VIDEO);
    if (dwDCXFlags & DCX_COMPATIBLE_DC) {
	/* for compatible DCs we stick 1x1 stock pixmap in it */
	if (DefaultDCPixmap == (Pixmap)0)
	    DefaultDCPixmap = XCreatePixmap(dp->display,
			RootWindow(dp->display,dp->screen),
			1,1, 
			DefaultDepth(dp->display,dp->screen));
	lpddc->Drawable = DefaultDCPixmap;
    }
    else
	lpddc->Drawable = (frame_win)?
		frame_win:RootWindow(dp->display, dp->screen);

    lpddc->BkFillMode = OPAQUE;
    lpddc->RopFunction = GXcopy;	/* default for new GC */
    lpddc->FillStyle = FillSolid;

    lpddc->ForeColorPixel =
		BlackPixel(lpddc->dp->display,lpddc->dp->screen);
    lpddc->BackColorPixel =
		WhitePixel(lpddc->dp->display,lpddc->dp->screen);

    /* now create graphic context */
    vmask = GCCapStyle|GCForeground|GCBackground;
    values.cap_style = CapNotLast;
    values.foreground = lpddc->ForeColorPixel;
    values.background = lpddc->BackColorPixel;

    lpddc->gc = TSXCreateGC(dp->display, lpddc->Drawable, vmask, &values);

    /* default attributes for X11 will be set by selecting stock */
    /* objects into the DC after this call returns */

    XSetSubwindowMode(dp->display,lpddc->gc,IncludeInferiors);
    UNLOCK_SECTION( _MTLOCK_VIDEO);

    /* init palette index translation table */
    lpddc->wPaletteSize = 0;
    lpddc->lpPaletteIndexTable = (LPWORD) 0;
    lpddc->lpPaletteEntryTable = (LPPALETTEENTRY) 0;

    return (DWORD)lpddc;
}

static DWORD
DrvDeleteDC(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
    LPDRIVERDC lpddc = (LPDRIVERDC)lpStruct;

    LOCK_SECTION( _MTLOCK_VIDEO);

    if (lpddc->gc) {
	TSXFreeGC(lpddc->dp->display, lpddc->gc);
    }

    if (lpddc->ClipRegion) {
	TSXDestroyRegion(lpddc->ClipRegion);
    }
    if (lpddc->UpdateRegion) {
	TSXDestroyRegion(lpddc->UpdateRegion);
    }
    if (lpddc->VisibleRegion) {
	TSXDestroyRegion(lpddc->VisibleRegion);
    }

    /* delete palette index translation table */
    if (lpddc->lpPaletteIndexTable) {
	WinFree((LPSTR) lpddc->lpPaletteIndexTable);
    }

    if (lpddc->lpPaletteEntryTable) {
	WinFree((LPSTR) lpddc->lpPaletteEntryTable);
    }

    TSXFreeDDC((LPSTR)lpddc);

    UNLOCK_SECTION( _MTLOCK_VIDEO);
    return 1L;
}

static DWORD
DrvSaveDC(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
    LPDRIVERDC lpddc = (LPDRIVERDC)lpStruct;
    LPDRIVERDC lpNewddc;
    Region xReg;

    LOCK_SECTION( _MTLOCK_VIDEO);
    lpNewddc = (LPDRIVERDC)TSXAllocDDC(sizeof(DRIVERDC));
    memcpy((LPSTR)lpNewddc,(LPSTR)lpddc,sizeof(DRIVERDC));

    if (lpNewddc->ClipRegion)
    {
	xReg = TSXCreateRegion();
	XUnionRegion(xReg,lpNewddc->ClipRegion,xReg);
	lpNewddc->ClipRegion = xReg;
    }

    if (lpNewddc->UpdateRegion)
    {
	xReg = TSXCreateRegion();
	XUnionRegion(xReg,lpNewddc->UpdateRegion,xReg);
	lpNewddc->UpdateRegion = xReg;
    }

    if (lpNewddc->VisibleRegion)
    {
	xReg = TSXCreateRegion();
	XUnionRegion(xReg,lpNewddc->VisibleRegion,xReg);
	lpNewddc->VisibleRegion = xReg;
    }

    /* copy palette index translation table */
    if (lpNewddc->lpPaletteIndexTable)
    {
	lpNewddc->lpPaletteIndexTable =
		(LPWORD) WinMalloc(sizeof(WORD)
			* (lpNewddc->wPaletteSize + 1));

	if (lpNewddc->lpPaletteIndexTable)
		memcpy(lpNewddc->lpPaletteIndexTable,
			lpddc->lpPaletteIndexTable,
			sizeof(WORD) * (lpNewddc->wPaletteSize + 1));
    }
    if (lpNewddc->lpPaletteEntryTable)
    {
	lpNewddc->lpPaletteEntryTable =
		(LPPALETTEENTRY) WinMalloc(sizeof(PALETTEENTRY)
			* (lpNewddc->wPaletteSize + 1));

	if (lpNewddc->lpPaletteEntryTable)
		memcpy(lpNewddc->lpPaletteEntryTable,
			lpddc->lpPaletteEntryTable,
			sizeof(WORD) * (lpNewddc->wPaletteSize + 1));
    }
    UNLOCK_SECTION( _MTLOCK_VIDEO);

    return (DWORD)lpNewddc;
}

/* If called with lpddc2==NULL, DrvRestoreDC() destroys clipping region
 * in lpddc1 and deallocates the structure; called with lpddc2, it saves
 * the drawable from lpddc1 to lpddc2, invalidates the fields in lpddc2 that
 * don't make sense anymore, and also frees the memory taken by lpddc1 */

static DWORD
DrvRestoreDC(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
    LPDRIVERDC lpddc2 = (LPDRIVERDC)dwParam1;
    LPDRIVERDC lpddc1 = (LPDRIVERDC)lpStruct;

    LOCK_SECTION( _MTLOCK_VIDEO);
    if (lpddc1->ClipRegion)
	TSXDestroyRegion(lpddc1->ClipRegion);

    if (lpddc1->UpdateRegion)
	TSXDestroyRegion(lpddc1->UpdateRegion);

    if (lpddc1->VisibleRegion)
	TSXDestroyRegion(lpddc1->VisibleRegion);

    if (lpddc1->lpPaletteIndexTable)
	WinFree((LPSTR) lpddc1->lpPaletteIndexTable);

    if (lpddc1->lpPaletteEntryTable)
	WinFree((LPSTR) lpddc1->lpPaletteEntryTable);

    if (lpddc2 && lpddc1->Drawable) {
	lpddc2->Drawable = lpddc1->Drawable;
	DrvCombineClipRgn(lpddc2);
    }

    TSXFreeDDC((LPSTR)lpddc1);
    UNLOCK_SECTION( _MTLOCK_VIDEO);

    return (DWORD)PGH_SUCCESS;
}

static DWORD
DrvGetDCClip(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
    LPDRIVERDC lpddc = (LPDRIVERDC)lpStruct;
    return (DWORD)lpddc->ClipRegion;
}

static DWORD
DrvSetDCClip(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
    Region RgnClip = (Region)dwParam1;
    LPDRIVERDC lpddc = (LPDRIVERDC)lpStruct;
    DWORD	clip;;

    /* if there is an empty update region, get rid of it */
    LOCK_SECTION( _MTLOCK_VIDEO);
    if (lpddc->UpdateRegion && XEmptyRegion(lpddc->UpdateRegion)) {
	TSXDestroyRegion(lpddc->UpdateRegion);
	lpddc->UpdateRegion = (Region)0;
    }

    if (lpddc->ClipRegion) {
	TSXDestroyRegion(lpddc->ClipRegion);
	lpddc->ClipRegion = 0;
    }

    if (RgnClip && !XEmptyRegion(RgnClip)) {
	lpddc->ClipRegion = TSXCreateRegion();
	XUnionRegion(RgnClip,lpddc->ClipRegion,lpddc->ClipRegion);
    }

    clip = DrvCombineClipRgn(lpddc);

    UNLOCK_SECTION( _MTLOCK_VIDEO);

    return clip;
}

static DWORD
DrvSetClipOrg(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
    LPPOINT lppt = (LPPOINT)dwParam1;
    LPDRIVERDC lpddc = (LPDRIVERDC)lpStruct;

    LOCK_SECTION( _MTLOCK_VIDEO);
    lpddc->cpClipOrigin = *lppt;
    UNLOCK_SECTION( _MTLOCK_VIDEO);
    return 1L;
}

static DWORD
DrvSetUpdateClip(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
    Region RgnUpdate = (Region)dwParam1;
    Region RgnVis = (Region)dwParam2;
    LPDRIVERDC lpddc = (LPDRIVERDC)lpStruct;
    /* if there is a non-empty update region passed, remember it */

    LOCK_SECTION( _MTLOCK_VIDEO);
    if (RgnUpdate && !XEmptyRegion(RgnUpdate)) {
	if (lpddc->UpdateRegion)
	    TSXDestroyRegion(lpddc->UpdateRegion);
	lpddc->UpdateRegion = TSXCreateRegion();
        XUnionRegion(RgnUpdate,lpddc->UpdateRegion,lpddc->UpdateRegion);
    }
    else
	if (lpddc->UpdateRegion) {
	    TSXDestroyRegion(lpddc->UpdateRegion);
	    lpddc->UpdateRegion = 0;
	}

    /* no user-defined clipping at this point */
    if(lpddc->ClipRegion) {
	TSXDestroyRegion(lpddc->ClipRegion);
	lpddc->ClipRegion = 0;
    }

    /* this combines all three... */
    (void)DrvSetVisRgn(lpddc,RgnVis);

    UNLOCK_SECTION( _MTLOCK_VIDEO);

    return 1L;
}

static DWORD
DrvDCSetVisRgn(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
	LPDRIVERDC lpddc = (LPDRIVERDC)lpStruct;
	Region VisRgn = (Region)dwParam2;
	return DrvSetVisRgn(lpddc, VisRgn);
}
 
static DWORD
DrvSetVisRgn(LPDRIVERDC lpddc, Region VisRgn)
{
    int nRet = NULLREGION;

    if(lpddc == 0)
	return 0;;

    LOCK_SECTION( _MTLOCK_VIDEO);
    if (lpddc->VisibleRegion)
	TSXDestroyRegion(lpddc->VisibleRegion);

    if (VisRgn) {
	lpddc->VisibleRegion = TSXCreateRegion();
	XUnionRegion(VisRgn,lpddc->VisibleRegion,lpddc->VisibleRegion);
	nRet = COMPLEXREGION;
    }
    else
	lpddc->VisibleRegion = 0;

    (void)DrvCombineClipRgn(lpddc);
    UNLOCK_SECTION( _MTLOCK_VIDEO);

    return (DWORD)nRet;
}

static DWORD
DrvCombineClipRgn(LPDRIVERDC lpddc)
{
    Region Rgn;

    LOCK_SECTION( _MTLOCK_VIDEO);
    Rgn = TSXCreateRegion();

    if (lpddc->VisibleRegion) {
	XUnionRegion(lpddc->VisibleRegion,Rgn,Rgn);

	if (lpddc->ClipRegion)
	    XIntersectRegion(lpddc->ClipRegion,Rgn,Rgn);
	if (lpddc->UpdateRegion)
	    XIntersectRegion(lpddc->UpdateRegion,Rgn,Rgn);

    }
    else {
	if (lpddc->ClipRegion) {
	    XUnionRegion(lpddc->ClipRegion,Rgn,Rgn);
	    if (lpddc->UpdateRegion)
		XIntersectRegion(lpddc->UpdateRegion,Rgn,Rgn);
	}
	else {
	    if (lpddc->UpdateRegion) {
		XUnionRegion(lpddc->UpdateRegion,Rgn,Rgn);
	    } else {
		if(lpddc->dp->display && lpddc->gc)
			XSetClipMask(lpddc->dp->display,lpddc->gc,None);
		TSXDestroyRegion(Rgn);
	        UNLOCK_SECTION( _MTLOCK_VIDEO);
		return (DWORD)NULLREGION;
	    }
	}
    }

    XSetRegion(lpddc->dp->display,lpddc->gc,Rgn);

    if (lpddc->cpClipOrigin.x || lpddc->cpClipOrigin.y) {
	if(lpddc->dp->display && lpddc->gc)
		XSetClipOrigin(lpddc->dp->display,lpddc->gc,
	    	    lpddc->cpClipOrigin.x,lpddc->cpClipOrigin.y);
    }

    TSXDestroyRegion(Rgn);
    UNLOCK_SECTION( _MTLOCK_VIDEO);

    return (DWORD)COMPLEXREGION;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: DrvDC.c,v $
** Revision 1.4  1999/07/08 18:53:32  mwalsh
** Updated to Standard Header and Footer comments
**
**-------------------------------------------------------------------------*/