/*--------------------------------------------------------------------------
**      XICCCM.c             Twin           From:  Twin/graphics/x11drv
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	graphics/x11drv/XICCCM.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/graphics/x11drv/XICCCM.c,v 1.3 1999/07/08 18:53:33 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Log.h"
#include "DrvDP.h"
#include "Driver.h"
#include "mtlock.h"

extern TWINLIBCALLBACKPROC lpLibCallback;
extern DWORD InternalClipboard(PRIVATEDISPLAY *, Window , Atom , Time , int );

/* 
 * dump the contents of a property in hex
 */
void
GetICCCMData(PRIVATEDISPLAY *dp,Atom atom,int state)
{
	unsigned char      *propertydata = 0;
	Atom		   actual_type; 
	int		   actual_format;
	unsigned long	   nitems;
	unsigned long	   remainder;

    	LOCK_SECTION( _MTLOCK_VIDEO);
	XGetWindowProperty(dp->display,
			RootWindow(dp->display,dp->screen),
			atom,0,8192,
			FALSE,XA_STRING,&actual_type,
			&actual_format,&nitems,&remainder,
			(unsigned char **) &propertydata);	
    	UNLOCK_SECTION( _MTLOCK_VIDEO);

	if(propertydata) {
    		LOCK_SECTION( _MTLOCK_VIDEO);
		XFree((char *)propertydata);
    		UNLOCK_SECTION( _MTLOCK_VIDEO);
	}
}

/* 
 * 	Handler called from DriverEvents.
 *	dispatch to handler for each type of atom
 *	(can be extended to user atoms... )
 */
int
InternalICCCM(int event,PRIVATEDISPLAY *dp,XEvent *report)
{
	char *lpstr;
	DWORD dwClientWin;

	lpstr = 0;

	switch(event) {
	case PropertyNotify:
    		LOCK_SECTION( _MTLOCK_VIDEO);
		lpstr = XGetAtomName(dp->display,report->xproperty.atom);
    		UNLOCK_SECTION( _MTLOCK_VIDEO);

		/* call our first and only handler for property change... */
		InternalClipboard( dp,
			report->xproperty.window,
			report->xproperty.atom,
			report->xproperty.time,
			report->xproperty.state);
#ifdef LATER
		if(state != PropertyDelete)
			GetICCCMData(dp,
				report->xproperty.atom,
				report->xproperty.state);
#endif
		break;

	case ClientMessage:
    		LOCK_SECTION( _MTLOCK_VIDEO);
		lpstr = XGetAtomName(dp->display,report->xclient.message_type);

		/* what hWnd did this occur in... */
		if (XFindContext(dp->display,report->xclient.window,
			dp->client_hwnd,(caddr_t *)&dwClientWin)) {
    			UNLOCK_SECTION( _MTLOCK_VIDEO);
			break;
		}
    		UNLOCK_SECTION( _MTLOCK_VIDEO);

		if (report->xclient.message_type == dp->wm_protocols) {
			if (report->xclient.data.l[0] == dp->wm_delete_window) {
			    MSG msg;
			    memset((LPVOID)&msg,'\0',sizeof(MSG));
			    msg.hwnd = (HWND)LOWORD(dwClientWin);
			    msg.message = WM_CLOSE;
			    (void)lpLibCallback(TWINLIBCALLBACK_POSTMESSAGE,
					0,0,(LPVOID)&msg);
			    return 1L;
			}
		}
		break;
	}

	if(lpstr) {
    		LOCK_SECTION( _MTLOCK_VIDEO);
		XFree(lpstr);
    		UNLOCK_SECTION( _MTLOCK_VIDEO);
	}
	return TRUE;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: XICCCM.c,v $
** Revision 1.3  1999/07/08 18:53:33  mwalsh
** Updated to Standard Header and Footer comments
**
**-------------------------------------------------------------------------*/
