/*--------------------------------------------------------------------------
**      ras.h              Twine           From:  Wine/include
**                                                wine-990523
**
**              
**------------------------- < License Information > ------------------------
**
**      This file is a direct copy of the equivalent file in the
**  Wine project (www.winehq.com).  The Wine license permits
**  copying of source code, so long as all copyright and credit
**  information is preserved.  Any and all copyrights originally
**  in the Wine source code file are preserved exactly as they were
**  when this copy was made from the Wine source tree.
**  (see the 'Original Copyright(s)', below).
**  An exacty copy of the Wine license is contained in WINE-LICENSE.
**  
**      For a more general description of the Wine license, and
**  for a full list of the authors who deserve credit for this
**  code, please see www.winehq.com, or WINE-AUTHORS.
**  
**      However, because this file is released with Twine, we
**  are obliged to also release it under the LGPL license.
**  In short, you may make a copy or republish this code so long as
**  all of your changes to source code are again made publicly available.
**  Further, absolutely no warranty of any kind is given or implied.
**  Refer to the LGPL description at www.fsf.org or the file LICENSE 
**  for further details.
**  
**  
**      ==========================================================
**      IF YOU MAKE CHANGES TO THIS CODE, PLEASE REFLECT
**      THOSE CHANGES IN THE ORIGINAL WINE CODE, AT WWW.WINEHQ.COM
**      ==========================================================
**
**      
**------------------------ < Original Copyright(s) > -----------------------
**
**  None in Wine source file; Wine license applies.
**
**------------------------ < File Content Description > --------------------
**
**  Module:	include/ras.h
**
**  Description: 
**      Provides Remote Acess Service defines
**
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/include/ras.h,v 1.1 1999/07/15 17:36:48 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

#ifndef __WINE_RAS_H
#define __WINE_RAS_H

#include "windef.h"

#if ! defined(UNLEN)
    #define UNLEN 256
#endif

#if ! defined(PWLEN)
    #define PWLEN 256
#endif

#if ! defined(DNLEN)
    #define DNLEN 15
#endif


#define RASDIALEVENT    "RasDialEvent"
#define WM_RASDIALEVENT 0xCCCD

DECLARE_HANDLE(HRASCONN);
#define LPHRASCONN HRASCONN*

#define RAS_MaxDeviceType     16
#define RAS_MaxPhoneNumber    128
#define RAS_MaxIpAddress      15
#define RAS_MaxIpxAddress     21

#define RAS_MaxEntryName      256
#define RAS_MaxDeviceName     128
#define RAS_MaxCallbackNumber RAS_MaxPhoneNumber

#define RAS_MaxAreaCode       10
#define RAS_MaxPadType        32
#define RAS_MaxX25Address     200
#define RAS_MaxFacilities     200
#define RAS_MaxUserData       200


typedef struct tagRASCONNA {
	DWORD		dwSize;
	HRASCONN	hrasconn;
	CHAR		szEntryName[RAS_MaxEntryName+1];
        CHAR            szDeviceType[ RAS_MaxDeviceType + 1 ];
        CHAR            szDeviceName[ RAS_MaxDeviceName + 1 ];
        CHAR            szPhonebook [ MAX_PATH ];
        DWORD           dwSubEntry;
} RASCONNA,*LPRASCONNA;

#define RASCONN RASCONNA
#define LPRASCONN LPRASCONNA


#define RASCS_PAUSED 0x1000
#define RASCS_DONE   0x2000

#define RASCONNSTATE enum tagRASCONNSTATE
RASCONNSTATE
{
    RASCS_OpenPort = 0,
    RASCS_PortOpened,
    RASCS_ConnectDevice,
    RASCS_DeviceConnected,
    RASCS_AllDevicesConnected,
    RASCS_Authenticate,
    RASCS_AuthNotify,
    RASCS_AuthRetry,
    RASCS_AuthCallback,
    RASCS_AuthChangePassword,
    RASCS_AuthProject,
    RASCS_AuthLinkSpeed,
    RASCS_AuthAck,
    RASCS_ReAuthenticate,
    RASCS_Authenticated,
    RASCS_PrepareForCallback,
    RASCS_WaitForModemReset,
    RASCS_WaitForCallback,
    RASCS_Projected,

#if (WINVER >= 0x400)
    RASCS_StartAuthentication,
    RASCS_CallbackComplete,
    RASCS_LogonNetwork,
#endif
    RASCS_SubEntryConnected,
    RASCS_SubEntryDisconnected,

    RASCS_Interactive = RASCS_PAUSED,
    RASCS_RetryAuthentication,
    RASCS_CallbackSetByCaller,
    RASCS_PasswordExpired,

    RASCS_Connected = RASCS_DONE,
    RASCS_Disconnected
};

#define LPRASCONNSTATE RASCONNSTATE*


/* Describes the status of a RAS connection.  (See RasConnectionStatus)
*/
#define RASCONNSTATUSW struct tagRASCONNSTATUSW
RASCONNSTATUSW
{
    DWORD        dwSize;
    RASCONNSTATE rasconnstate;
    DWORD        dwError;
    WCHAR        szDeviceType[ RAS_MaxDeviceType + 1 ];
    WCHAR        szDeviceName[ RAS_MaxDeviceName + 1 ];
#if (WINVER >= 0x401)
    WCHAR        szPhoneNumber[ RAS_MaxPhoneNumber + 1 ];
#endif
};

#define RASCONNSTATUSA struct tagRASCONNSTATUSA
RASCONNSTATUSA
{
    DWORD        dwSize;
    RASCONNSTATE rasconnstate;
    DWORD        dwError;
    CHAR         szDeviceType[ RAS_MaxDeviceType + 1 ];
    CHAR         szDeviceName[ RAS_MaxDeviceName + 1 ];
#if (WINVER >= 0x401)
    CHAR         szPhoneNumber[ RAS_MaxPhoneNumber + 1 ];
#endif
};

#ifdef UNICODE
#define RASCONNSTATUS RASCONNSTATUSW
#else
#define RASCONNSTATUS RASCONNSTATUSA
#endif

#define LPRASCONNSTATUSW RASCONNSTATUSW*
#define LPRASCONNSTATUSA RASCONNSTATUSA*
#define LPRASCONNSTATUS  RASCONNSTATUS*



#define RASDIALEXTENSIONS struct tagRASDIALEXTENSIONS
RASDIALEXTENSIONS
{
    DWORD dwSize;
    DWORD dwfOptions;
    HWND  hwndParent;
    DWORD reserved;
};

#define LPRASDIALEXTENSIONS RASDIALEXTENSIONS*


#define RASDIALPARAMSW struct tagRASDIALPARAMSW
RASDIALPARAMSW
{
    DWORD dwSize;
    WCHAR szEntryName[ RAS_MaxEntryName + 1 ];
    WCHAR szPhoneNumber[ RAS_MaxPhoneNumber + 1 ];
    WCHAR szCallbackNumber[ RAS_MaxCallbackNumber + 1 ];
    WCHAR szUserName[ UNLEN + 1 ];
    WCHAR szPassword[ PWLEN + 1 ];
    WCHAR szDomain[ DNLEN + 1 ];
#if (WINVER >= 0x401)
    DWORD dwSubEntry;
    DWORD dwCallbackId;
#endif
};

#define RASDIALPARAMSA struct tagRASDIALPARAMSA
RASDIALPARAMSA
{
    DWORD dwSize;
    CHAR  szEntryName[ RAS_MaxEntryName + 1 ];
    CHAR  szPhoneNumber[ RAS_MaxPhoneNumber + 1 ];
    CHAR  szCallbackNumber[ RAS_MaxCallbackNumber + 1 ];
    CHAR  szUserName[ UNLEN + 1 ];
    CHAR  szPassword[ PWLEN + 1 ];
    CHAR  szDomain[ DNLEN + 1 ];
#if (WINVER >= 0x401)
    DWORD dwSubEntry;
    DWORD dwCallbackId;
#endif
};

#ifdef UNICODE
#define RASDIALPARAMS RASDIALPARAMSW
#else
#define RASDIALPARAMS RASDIALPARAMSA
#endif

#define LPRASDIALPARAMSW RASDIALPARAMSW*
#define LPRASDIALPARAMSA RASDIALPARAMSA*
#define LPRASDIALPARAMS  RASDIALPARAMS*

#define RASENTRYNAMEW struct tagRASENTRYNAMEW
RASENTRYNAMEW
{
    DWORD dwSize;
    WCHAR szEntryName[ RAS_MaxEntryName + 1 ];
};

#define RASENTRYNAMEA struct tagRASENTRYNAMEA
RASENTRYNAMEA
{
    DWORD dwSize;
    CHAR  szEntryName[ RAS_MaxEntryName + 1 ];
};

#ifdef UNICODE
#define RASENTRYNAME RASENTRYNAMEW
#else
#define RASENTRYNAME RASENTRYNAMEA
#endif

#define LPRASENTRYNAMEW RASENTRYNAMEW*
#define LPRASENTRYNAMEA RASENTRYNAMEA*
#define LPRASENTRYNAME  RASENTRYNAME*


DWORD WINAPI RasEnumConnectionsA( LPRASCONNA rc, LPDWORD x, LPDWORD y);
#endif

/*------------------------- < Full Revision History > ----------------------
** $Log: ras.h,v $
** Revision 1.1  1999/07/15 17:36:48  jwhite
** Added OLE/COM support headers files, mostly from Wine to enable
** builds of MFC w/COM.
**
**
**-------------------------------------------------------------------------*/
