/*--------------------------------------------------------------------------
**      twinlog.h                 Twin           From:  Twin/include
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc. 
**------------------------ < File Content Description > --------------------
**
**  Module:	include/twinlog.h
**
**  Description:This is for more fine control of logging messages
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/include/twine/twinlog.h,v 1.1 1999/07/15 17:56:56 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/




#ifndef Twinlog__h
#define Twinlog__h

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/


/*
*  These are intented mostly for source ports. For example,
*  if you've narrowed down a Windows application bug to a
*  few contiguous API calls, then simply bracket your calls as follows:
*
*	TwinLogOn();
*	MoveTo(...);
*	TextOut(...);
*	TwinLogReset();
*
*  The above translates as follows
*
*	force all debug and trace messages to be enabled
*	find out all library messages about MoveTo() and TextOut()
*	restore to initial startup debug and trace levels
*
*  To make use of these, you will have to compile with
*  the DEBUG libraries. Note that these will work in conjunction with
*  the environment variable WIN_OPENLOG, particularly the file name.
*  If WIN_OPENLOG is not set, then standard output is assumed.
*/

extern unsigned long TwinLogOn(void);	/* force ON all library messages */
extern unsigned long TwinLogOff(void);	/* force OFF all library messages */
extern unsigned long TwinLogReset(void); /* go back to initial logging state */
extern unsigned long TwinLogGet(void);	/* get current logging mask */
extern unsigned long TwinLogSet(unsigned long); /* set current logging mask */


#endif /* Twinlog__h */

/*------------------------- < Full Revision History > ----------------------
** $Log: twinlog.h,v $
** Revision 1.1  1999/07/15 17:56:56  jwhite
** Moved files that are not clearly a part of the Win32 API specification
** out of the main include directory into a 'twine' subdirectory.
** This is going to allow us to more readily integrate Wine include files.
**
** Revision 1.2  1999/07/12 21:13:44  mwalsh
** Updated comments to utilize standard header and footer
**
**-------------------------------------------------------------------------*/
