/*--------------------------------------------------------------------------
**      windef.h                 Twin           From:  Twin/include
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**       
**------------------------ < File Content Description > --------------------
**
**  Module:	include/windef.h
**
**  Description:provide common types of objects
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/include/windef.h,v 1.11 2000/01/26 19:55:27 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/



#ifndef __WINDEF_H__
#define __WINDEF_H__

/*----------------------------------------------------------------------------
**  Core Windows type definitions
**--------------------------------------------------------------------------*/
#ifndef FALSE
#define FALSE       0
#define TRUE        1
#endif

typedef unsigned char	BYTE;
typedef unsigned short	WORD;
typedef unsigned long	DWORD;

typedef	char		CHAR;
#ifndef TWIN_UCHAR_DEFINED
typedef unsigned char   UCHAR;
#endif
typedef	char		TCHAR;
typedef float		FLOAT;

typedef UINT		*PUINT;
typedef UINT NEAR	*NPUINT;
typedef UINT FAR	*LPUINT;

typedef FLOAT		*PFLOAT;
typedef FLOAT NEAR	*NPFLOAT;
typedef FLOAT FAR	*LPFLOAT;

typedef unsigned char 	boolean;
typedef unsigned char 	byte;

typedef int LONG32, *PLONG32;
typedef int INT32, *PINT32;
typedef unsigned int ULONG32, *PULONG32;
typedef unsigned int DWORD32, *PDWORD32;
typedef unsigned int UINT32, *PUINT32;
 
#ifdef  TWIN32
    #if SIZEOF_VOID_P == SIZEOF_UNSIGNED_INT
        typedef unsigned int          WLS_UINT;
    #elif SIZEOF_VOID_P == SIZEOF_UNSIGNED_LONG
        typedef unsigned long         WLS_UINT;
    #endif

#else
    typedef unsigned short      WLS_UINT;
#endif

/*----------------------------------------------------------------------------
**  In some cases, the code requires an integer type that can be used
**     interchangeably with a pointer.  
**--------------------------------------------------------------------------*/
#define PTR_COMPATIBLE_UINT     WLS_UINT

/* Handle definitions ******************************************/
/*
 *  For WIN16, we cannot use the STRICT flag to enable unique
 *  handle types, since for us the underlying pointers are 32-bit 
 *  pointers, not 16-bit pointers.
 */
#if defined(STRICT) && defined(TWIN32)
#if 0
typedef void NEAR*        HANDLE;
#define DECLARE_HANDLE(name)    struct name##__ { int unused; }; \
                                typedef struct name##__ NEAR* name
#define DECLARE_HANDLE32(name)  struct name##__ { int unused; }; \
                                typedef struct name##__ FAR* name
#else
// CANNOT USE STRICT...
typedef WLS_UINT                    HANDLE;
#define DECLARE_HANDLE(name)    typedef WLS_UINT name
#define DECLARE_HANDLE32(name)  typedef DWORD name
#endif

#else
typedef WLS_UINT                    HANDLE;
#define DECLARE_HANDLE(name)    typedef WLS_UINT name
#define DECLARE_HANDLE32(name)  typedef DWORD name
#endif

typedef HANDLE*         PHANDLE;
typedef HANDLE NEAR*    SPHANDLE;
typedef HANDLE FAR*     LPHANDLE;

typedef HANDLE          HGLOBAL;
typedef HANDLE          HLOCAL;

typedef HANDLE          GLOBALHANDLE;
typedef HANDLE          LOCALHANDLE;

typedef WORD            ATOM;

typedef void *HANDLE32;           
typedef void *HTASK32;           

typedef HANDLE HTASK16;
typedef HANDLE HQUEUE16;

#ifdef STRICT
typedef void (CALLBACK*     FARPROC)(void);
typedef void (NEAR PASCAL*  NEARPROC)(void);
#else
typedef int (CALLBACK*      FARPROC)();
typedef int (NEAR PASCAL*   NEARPROC)();
#endif
typedef int (WINAPI *PROC)();

/* Core handle declarations */
DECLARE_HANDLE(HSTR);
DECLARE_HANDLE(HTASK);
DECLARE_HANDLE(HRSRC);
DECLARE_HANDLE(HDROP);

DECLARE_HANDLE(HINSTANCE);
typedef HINSTANCE HMODULE;  /* HMODULEs can be used in place of HINSTANCEs */

DECLARE_HANDLE(HDC);

DECLARE_HANDLE(HBITMAP);
DECLARE_HANDLE(HPEN);
DECLARE_HANDLE(HBRUSH);
DECLARE_HANDLE(HRGN);
DECLARE_HANDLE(HPALETTE);
DECLARE_HANDLE(HFONT);
DECLARE_HANDLE(HWND);
DECLARE_HANDLE(HGDIOBJ);

/* GDI related typedefs */
typedef struct tagRECT
{
    int left;
    int top;
    int right;
    int bottom;
} RECT;
typedef RECT*      PRECT;
typedef RECT NEAR* NPRECT;
typedef RECT FAR*  LPRECT;
typedef const RECT FAR*  LPCRECT;

typedef struct _RECTL
{
	LONG left;
	LONG top;
	LONG right;
	LONG bottom;
} RECTL;
typedef RECTL		*PRECTL;
typedef RECTL NEAR	*NPRECTL;
typedef RECTL FAR	*LPRECTL;
typedef const RECTL     *LPCRECTL;

typedef struct tagPOINT
{
    int x;
    int y;
} POINT;
typedef POINT*       PPOINT;
typedef POINT NEAR* NPPOINT;
typedef POINT FAR*  LPPOINT;

/* #define MAKEPOINT(l)      (*((POINT FAR*)&(l))) */
POINT MAKEPOINT(DWORD);

typedef struct tagPOINTS
{
    short x;
    short y;
} POINTS;
typedef POINTS*       PPOINTS;
typedef POINTS NEAR* NPPOINTS;
typedef POINTS FAR*  LPPOINTS;

#define MAKEPOINTS(l)      (*((POINTS FAR*)&(l)))

typedef struct tagSIZE
{
    int cx;
    int cy;
} SIZE;
typedef SIZE*       PSIZE;
typedef SIZE NEAR* NPSIZE;
typedef SIZE FAR*  LPSIZE;

typedef SIZE		SIZEL;
typedef SIZE		*PSIZEL;
typedef SIZE NEAR	*NPSIZEL;
typedef SIZE FAR	*LPSIZEL;

typedef DWORD	LCTYPE;

DECLARE_HANDLE(HMETAFILEPICT);

#define WINELIB_NAME_AW(func) func##A
#define DECL_WINELIB_TYPE_AW(type)  typedef WINELIB_NAME_AW(type) type;


#endif /* __WINDEF_H__ */

/*------------------------- < Full Revision History > ----------------------
** $Log: windef.h,v $
** Revision 1.11  2000/01/26 19:55:27  jwhite
** This includes a wide range of changes required to get Twine to work
** with MFC on the Alpha platform.
**
** Revision 1.10  2000/01/07 22:47:29  jwhite
** Revisions to remimplement the new anon struct/union.
** Revised to support 64 bit pointer platforms.
** Restored wtypes.h to version lost in Rob's latest changes.
**
** Revision 1.9  2000/01/03 18:08:53  robf
** moved from windows.h for files that only use windef.h
**
** Revision 1.8  1999/12/09 06:21:43  robf
** fixed declaration of HRESULT to match MS
**
** Revision 1.6  1999/07/31 02:32:01  jwhite
** Added misc definitions and types needed for Wine headers.
**
** Revision 1.5  1999/07/15 17:41:00  jwhite
** Added LPCRECTL.
**
** Revision 1.4  1999/07/12 21:13:45  mwalsh
** Updated comments to utilize standard header and footer
**
**-------------------------------------------------------------------------*/
