/*--------------------------------------------------------------------------
**      int_2f.c                 Twin           From:  Twin/msdos
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**    Copyright 1997 Willows Software, Inc.  
**------------------------ < File Content Description > --------------------
**
**  Module:	msdos/int_2f.c
**
**  Description: Multiplex Interrupt 0x2f
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/msdos/int_2f.c,v 1.2 1999/07/08 21:30:38 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "platform.h"
#include "xdos.h"
#include "BinTypes.h"
#include "Log.h"

/*
 *	Details from Microsoft MS-DOS Programmers Reference
 *	page 147, plus individual int2f opcodes
 *
 *	For windows, the only one actually detected are
 *		function 10	get SHARE.EXE Installed state
 *				note: winword may require this?
 *		function 15	seen, but we report error
 *	Others were added for completeness, to avoid possible errors
 *	later.
 */	

void
int_2f(int nNumber, ENV *envp)
{
	int func;

	func = HIBYTE(LOWORD(envp->reg.ax));

	switch (func) {
	    case 0x01:	
		/* al = 0x00	get PRINT.EXE Installed State   */
		/* al = 0x01	Add file to queue 		*/
		/* al = 0x02	Remove file from queue 		*/
		/* al = 0x03	Cancel all files from queue     */
		/* al = 0x04	Hold Print Jobs and Get Status  */
		/* al = 0x05	Release Print Jobs		*/
		/* al = 0x06	Get Printer Device		*/

	    case 0x06:		/* Get ASSIGN.COM Installed State */	
		envp->reg.flags |= CARRY_FLAG >> 16;
		envp->reg.ax = 0x000;
		break;

            case 0x10:
                /* return that share IS installed */
		envp->reg.ax |= 0x00ff;
	        break;

	    case 0x11:		/* Get Network Installed State  */
		envp->reg.flags |= CARRY_FLAG >> 16;
                /* return that network IS NOT installed */
		envp->reg.ax = 0x000;
		break;

	    case 0x14:		/* Get NLSFUNC.EXE Installed State  */
		envp->reg.flags |= CARRY_FLAG >> 16;
                /* return that NLSFUNC IS NOT installed */
		envp->reg.ax = 0x000;
		break;

	    case 0x15:	/* god knows what (mscdex requests) */
		envp->reg.flags &= ~(CARRY_FLAG >> 16);
		envp->reg.ax = 0x001;
		break;

	    case 0x16:
		/* al = 0x80	MS-DOS Idle Call */
                /* return that program suspension IS NOT supported */
		envp->reg.ax = 0x001;
		break;

	    case 0x1a:		/* Get ANSI.SYS installed State */
                /* return that ANSI.SYS IS NOT loaded */
		envp->reg.ax = 0x000;
		break;

	    case 0x43:		
		/* al = 0x00	Get HIMEM.SYS Installed State */
		/* al = 0x01	Get HIMEM.SYS Entry Point */
                /* return that HIMEM.SYS IS NOT loaded */
		envp->reg.ax = 0x000;
		break;

	    case 0x48:		
		/* al = 0x00	Get DOSKEY Installed State */
		/* al = 0x10	Read Command Line */
                /* return that DOSKEY.COM IS NOT loaded */
		envp->reg.ax = 0x000;
		break;

	    default:
		envp->reg.flags |= CARRY_FLAG >> 16;
		envp->reg.ax = 0x001;
		break;
	}
}

/*------------------------- < Full Revision History > ----------------------
** $Log: int_2f.c,v $
** Revision 1.2  1999/07/08 21:30:38  mwalsh
** Updated to new standard comment header and footer
**
**-------------------------------------------------------------------------*/
