/*--------------------------------------------------------------------------
**      os.c                   Twine           
**              
**------------------------- < License Information > ------------------------
**
**  
**      This file is released under terms of an LGPL.  Readers of 
**  this file may modify or use this source code so long as they make all
**  modifications publically available.  Please refer to TWINE-LICENSE 
**  for further details.
**  
**
**      The original Copyright is in 1999, by CodeWeavers, Inc.
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	osdll/os.c
**	
**  Description:
**		provides system dependent threads
**
**  Functions defined:
**
**		osdll_thread_init(THDB *)
**		osdll_thread_create(TLS *)
**		osdll_thread_gettdb()
**		osdll_thread_init(THDB *)
**		osdll_thread_ops(THDB *,int)
**
**		osdll_thread_yield()		nonblock yield to system
**		osdll_thread_suspend() 		block until restart
**
**  Notes:
**      1.  This file is stubbed out for all non Linux operating
**          systems.
**              TODO:  FIXME:  Implement os.c for non linux operating systems
**
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/osdll/os.c,v 1.14 1999/05/06 20:20:38 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "kerndef.h"
#include "Resources.h"
#include "Module.h"
#include "osdll.h"
#include "Exec.h"

#if defined(linux)
#include "Log.h"
#include "errno.h"
#include "sched.h"
#include "signal.h"
#include <assert.h>
#include <errno.h>
#include <syscall.h>
#endif


/*----------------------------------------------------------------------------
**              Static variables global to os.c
**--------------------------------------------------------------------------*/

static THDB *pCurrentThread;

TEB * WINAPI NtCurrentTeb(void);

static void osdll_setfs(WORD);


    

static void osdll_thread_sighandler(int sig)
{
#if defined(linux) && defined(__i386__)
	THDB *pThread;
        struct TWIN_i386_context_s  *sc;

    	sc = (struct TWIN_i386_context_s *) ((long *) &sig + 1);

	osdll_setfs(sc->fs);
	pThread = osdll_thread_gettdb();

    	//pThread->p_signal = sig;
     	//if (pThread->p_signal_jmp != NULL) 
	//	siglongjmp(*pThread->p_signal_jmp, 1);
#endif
}

#if 0
	Need to do setmp/longjmp to cleanup...
#endif
static void osdll_thread_sigcancel(int sig)
{
#if defined(linux) && defined(__i386__)
	THDB *pThread;
        struct TWIN_i386_context_s  *sc;

    	sc = (struct TWIN_i386_context_s *) ((long *) &sig + 1);

	osdll_setfs(sc->fs);
	pThread = osdll_thread_gettdb();
#endif
	exit(0);
}

THDB *
osdll_thread_init( THDB *tdb)
{
#if defined(linux)
  	struct sigaction sa;
  	sigset_t mask;

	tdb->unix_pid       = getpid();
	osdll_setfs(tdb->teb_sel);

  	sa.sa_handler = osdll_thread_sighandler;
  	sigemptyset(&sa.sa_mask);
  	sa.sa_flags = SA_RESTART; 

  	sigaction(SIGUSR1, &sa, NULL);

  	sa.sa_handler = osdll_thread_sigcancel;
  	sa.sa_flags = 0;

  	sigaction(SIGUSR2, &sa, NULL);

  	/* Block SIGUSR1 */
  	sigemptyset(&mask);
  	sigaddset(&mask, SIGUSR1);
  	sigprocmask(SIG_BLOCK, &mask, NULL);
#endif

	if(pCurrentThread == 0) {
		pCurrentThread = tdb;
	}
	return pCurrentThread;
}

THDB *
osdll_thread_gettdb()
{
    	if (!pCurrentThread) 
		return NULL;
    	return (THDB *)((char *)NtCurrentTeb() - (int)&((THDB *)0)->teb);
}

static int
osdll_startup( void *p)
{	
#if defined(linux)
	THDB *tdb = (THDB *) p;
	TLS *tls = (TLS *) tdb->stack_base;
  	sigset_t mask;
	
	osdll_thread_init(tdb);

  	sigprocmask(SIG_BLOCK, NULL, &mask);
  	sigaddset(&mask, SIGINT);
  	sigprocmask(SIG_BLOCK, &mask, NULL);

	/* call up to the library, to run the thread */
	(tls->init)(tdb);

	/* should never get here */
	exit(0);
#else
    return 0;
#endif
}

void
osdll_thread_yield()
{
#if defined(linux)
	sched_yield();
#endif
}

void
osdll_thread_suspend()
{
#if defined(linux)
  	sigset_t mask;

  	sigprocmask(SIG_SETMASK, NULL, &mask);  /* Get current signal mask */
  	sigdelset(&mask, SIGUSR1); 		/* Unblock the restart signal */
    	sigsuspend(&mask);                   	/* Wait for signal */
#endif
}

static int 
osdll_clone( int (*fn)(void *), void *stack, int flags, void *arg )
{
#if defined(linux) && defined(__i386__)
    int ret;
    void **stack_ptr = (void **)stack;
    *--stack_ptr = arg;  /* Push argument on stack */
    *--stack_ptr = fn;   /* Push function pointer (popped into ebx) */
    __asm__ __volatile__( "pushl %%ebx\n\t"
                          "movl %2,%%ebx\n\t"
                          "int $0x80\n\t"
                          "popl %%ebx\n\t"   /* Contains fn in the child */
                          "testl %%eax,%%eax\n\t"
                          "jnz 0f\n\t"
                          "call *%%ebx\n\t"       /* Should never return */
                          "xorl %%eax,%%eax\n\t"  /* Just in case it does*/
                          "0:"
                          : "=a" (ret)
                          : "0" (SYS_clone), "g" (flags), "c" (stack_ptr) );

    if (ret > 0) 
	return ret;

    exit(0);
#else
    return 0;
#endif
}

#ifndef CLONE_VM
#define       CLONE_VM      0x00000100
#endif
#ifndef CLONE_FILES
#define       CLONE_FILES   0x00000400
#endif

int
osdll_thread_create( THDB *tdb)
{
#if defined(linux)
	int rc;
	int  flags =   		CLONE_VM|CLONE_FILES; 
	char *sp = tdb->teb.stack_top;

	rc = osdll_clone(osdll_startup,sp,flags,tdb);

	if (rc) {
		return (tdb->unix_pid = rc);
	}

    	if (rc == 0) {
		exit(0);
	}

	return 0;
#else
    return 0;
#endif
}

int
osdll_thread_ops(THDB *tdb, int opcode)
{
#if defined(linux)
	int rc = 0;

	switch(opcode) {
	case OSDLL_THREAD_SUSPEND:	
		rc = kill(tdb->unix_pid,SIGSTOP);
		break;

	case OSDLL_THREAD_RESUME:	
		rc = kill(tdb->unix_pid,SIGCONT);
		break;

	case OSDLL_THREAD_CANCEL:	
		rc = kill(tdb->unix_pid,SIGUSR2); 
		break;

	case OSDLL_THREAD_RESTART:	
		rc = kill(tdb->unix_pid,SIGUSR1); 
		break;
	default:
		break;
	}
	return rc;
#else
    return 0;
#endif
}

void
osdll_dll_exit()
{
}

/**********************************************************************
 *           NtCurrentTeb   (NTDLL.89)
 *
 * This will crash and burn if called before threading is initialized
 */
TEB * WINAPI NtCurrentTeb(void)
{
#ifdef __i386__
    TEB *teb;

    /* Get the TEB self-pointer */
    __asm__( ".byte 0x64\n\tmovl (%1),%0"
             : "=r" (teb) : "r" (&((TEB *)0)->self) );
    return teb;
#else
    return &pCurrentThread->teb;
#endif  /* __i386__ */
}

static void
osdll_setfs(WORD fs)
{
#if defined(linux) && defined(__i386__)
	__asm__ __volatile__("movw %w0,%%fs\t\n"::"r" (fs));
#else
    return;
#endif
}

/*------------------------- < Full Revision History > ----------------------
** $Log: os.c,v $
** Revision 1.14  1999/05/06 20:20:38  robf
** merge jeremys changes with mine
**
** Revision 1.13  1999/05/03 05:37:31  robf
** remove unused variables
**
** Revision 1.12  1999/05/02 01:18:03  robf
** printf? really?
**
** Revision 1.11  1999/05/01 19:21:59  jwhite
** Minor revisions to permit compilation on non-linux systems.
**
** Revision 1.10  1999/04/30 09:19:58  robf
** fix for intel linux
**
** Revision 1.9  1999/04/26 09:01:22  robf
** remove sem functions
**
** Revision 1.8  1999/04/01 22:51:58  jwhite
** Added the i386 dependency too.  Credit Stewart Nutter.
**
** Revision 1.7  1999/04/01 22:48:39  jwhite
** Corrected a return type.
**
** Revision 1.6  1999/04/01 21:04:47  jwhite
** Revised so it would compile on non-linux platforms, with credit to Stewart Nutter.
**
**-------------------------------------------------------------------------*/

