/*--------------------------------------------------------------------------
**      dltest.c                 Twin           From:  Twin/samples/dltest
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	samples/dltest/dltest.c
**
**  Description:test functionality of dlopen(), dlsym() and dladdr()
**               -  dlopen()  test, w/ and w/o .init function
**               -  dlsym()   test, can we retrieve address from symbol
**               -  dladdr()  test, given an address, can we find where its from     
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/samples/dltest/dltest.c,v 1.3 1999/12/10 05:29:57 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "stdio.h"
#define __USE_GNU
#include "dlfcn.h"

typedef void (*f)();



int
main(int argc,char **argv)
{
	void *hso;
	f     fp;	
	int  *p;
	Dl_info d;

	printf("dlopen tests\n");
	hso = dlopen("./lib1.so",RTLD_GLOBAL);	

	if(hso) {
		char *entry = "TWIN_LibEntry_lib1";
 		fp = (f) dlsym(hso,	entry);

		printf("test1: passed: dlopen(library): no .init function\n");
		
		if(fp) {
			(*fp)();
			printf("test 2: passed?: dlsym().init function called\n");
		} else
			printf("test 2 failed\n");
		
		dlclose(hso);
	} else 
		printf("test 1 failed\n");


	hso = dlopen("./lib2.so",RTLD_GLOBAL);	

	if(hso) {
		char *entry = "TWIN_LibEntry_common";
		printf("test3: passed?: verify lib2 .init function called\n");

 		fp = (f) dlsym(hso,	entry);
		
		if(fp) {
			(*fp)();
			printf("test4: passed?: verify lib2 common function called\n");
		} else 
			printf("test 4 failed\n");
	
		entry = "hInstance";
 		p = (int *) dlsym(hso,	entry);
	
		printf("test5: passed?: verify %d == 2\n",*p);
		
		memset(&d,0,sizeof(Dl_info));
		dladdr(p,&d);
		printf("test6: passed?: verify %s == hInstance\n",
			d.dli_sname);
		printf("file=%s dladdr=%p address=%p\n",
			d.dli_fname,d.dli_saddr,p);
		
		dlclose(hso);
	}
	return 0;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: dltest.c,v $
** Revision 1.3  1999/12/10 05:29:57  robf
** need glibc Dl_info structure, define USE_GNU
**
** Revision 1.2  1999/07/14 18:14:17  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
