/*--------------------------------------------------------------------------
**      Interlock.c                 Twin           From:  Twin/scheduler
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	scheduler/Interlock.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/scheduler/Interlock.c,v 1.3 1999/12/29 04:18:42 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Log.h"

/* (WIN32) Interlocked Arithmetic ****************************************** */

/*
 * WARNING: These functions are provided as functional, but not thread-safe!
 */

/* (WIN32) Synchronization ************************************************* */

DWORD	WINAPI
WaitForSingleObject(HANDLE hObject, DWORD dwTimeOut)
{
	APISTR((LF_API, "WaitForSingleObject:"
		" (API) hObject %x dwTimeOut %ld\n",
		hObject, dwTimeOut));
	return (WAIT_OBJECT_0);
}

DWORD	WINAPI
WaitForSingleObjectEx(HANDLE hObject, DWORD dwTimeOut, BOOL bAlertable)
{
	APISTR((LF_API, "WaitForSingleObject:"
		" (API) hObject %x dwTimeOut %ld bAlertable %s\n",
		hObject, dwTimeOut, bAlertable ? "TRUE" : "FALSE"));
	return (WAIT_OBJECT_0);
}

DWORD	WINAPI
WaitForMultipleObjects(DWORD dwCount, CONST HANDLE *lpHandles,
	BOOL bWaitAll, DWORD dwMilliseconds)
{
	APISTR((LF_API, "WaitForMultipleObjects:"
		" (API) dwCount %ld lpHandles %p\n",
		dwCount, (void *) lpHandles));
	return (WAIT_OBJECT_0);
}

DWORD	WINAPI
WaitForMultipleObjectsEx(DWORD dwCount, CONST HANDLE *lpHandles,
	BOOL bWaitAll, DWORD dwMilliseconds, BOOL bAlertable)
{
	APISTR((LF_API, "WaitForMultipleObjectsEx:"
		" (API) dwCount %ld lpHandles %p\n",
		dwCount, (void *) lpHandles));
	return (WAIT_OBJECT_0);
}

/* (WIN32) Semaphore ******************************************************* */

HANDLE	WINAPI
CreateSemaphore(LPSECURITY_ATTRIBUTES lpSemaphoreAttributes,
	LONG lInitialCount,
	LONG lMaximumCount,
	LPCSTR lpName)
{
	APISTR((LF_API, "CreateSemaphore: (API) %s\n", lpName));
	return ((HANDLE) 0);
}

HANDLE	WINAPI
OpenSemaphore(DWORD dwDesiredAccess,
	BOOL bInheritHandle,
	LPCSTR lpName)
{
	APISTR((LF_API, "OpenSemaphore: (API) %s\n", lpName));
	return ((HANDLE) 0);
}

BOOL	WINAPI
ReleaseSemaphore(HANDLE hSemaphore,
	LONG lpReleaseCount, LPLONG lpPreviousCount)
{
	APISTR((LF_API, "ReleaseSemaphore: (API) hSemaphore %x\n",
		hSemaphore));
	return (TRUE);
}

/* (WIN32) Event *********************************************************** */

HANDLE	WINAPI
CreateEvent(LPSECURITY_ATTRIBUTES lpEventAttributes,
	BOOL bManualReset,
	BOOL bInitialState,
	LPCSTR lpName)
{
	APISTR((LF_API, "CreateEvent: (API) %s\n", lpName));
	return ((HANDLE) 0);
}

HANDLE	WINAPI
CreateEventW(LPSECURITY_ATTRIBUTES lpEventAttributes,
	BOOL bManualReset,
	BOOL bInitialState,
	LPCWSTR lpName)
{
	APISTR((LF_API, "CreateEventW: (API) %s\n", lpName));
	return ((HANDLE) 0);
}

BOOL	WINAPI
ResetEvent(HANDLE hEvent)
{
	APISTR((LF_API, "ResetEvent: (API) hEvent %x\n", hEvent));
	return (TRUE);
}

BOOL	WINAPI
SetEvent(HANDLE hEvent)
{
	APISTR((LF_API, "SetEvent: (API) hEvent %x\n", hEvent));
	return (TRUE);
}

BOOL	WINAPI
PulseEvent(HANDLE hEvent)
{
	APISTR((LF_API, "PulseEvent: (API) hEvent %x\n", hEvent));
	return (TRUE);
}

/* (WIN32) Message Synchronization ***************************************** */

DWORD	WINAPI
MsgWaitForMultipleObjects(DWORD dwCount,
	LPHANDLE lpHandles,
	BOOL bWaitAll,
	DWORD dwMilliseconds,
	DWORD dwWakeMask)
{
	APISTR((LF_API, "MsgWaitForMultipleObjects: (API)\n"));
	return (WAIT_OBJECT_0);
}

/*------------------------- < Full Revision History > ----------------------
** $Log: Interlock.c,v $
** Revision 1.3  1999/12/29 04:18:42  jwhite
** Revised to support reading of DIALOGEX binary file.
** Added code for DrawTextW and CreateEventW.
**
** Revision 1.2  1999/07/13 20:17:37  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/
