#!/usr/bin/perl -w
#
#--------------------------------------------------------------------------
#      headercrunch.pl                   Twine           
#              
#------------------------- < License Information > ------------------------
#
#  
#      This file is released under terms of an LGPL.  Readers of 
#  this file may modify or use this source code so long as they make all
#  modifications publically available.  Please refer to TWINE-LICENSE 
#  for further details.
#  
#
#      The original Copyright is in 1999, by CodeWeavers, Inc.
#
#      
#------------------------ < File Content Description > --------------------
#
#  Module:	twine/mfc/4.2/headercrunch.pl
#	
#  Description:
#     This script generates two files, twine_headers.txt and win32_headers.txt
#     based on the header include paths of Twine's afx.h, and the original
#     Win32 afx.h.
#
#  Author:      John R. Sheets <jsheets@codeweavers.com>     
#    
#------------------------- < Revision Information > -----------------------
# $Header: /twinecvs/src/tools/headercrunch.pl,v 1.1 1999/09/17 17:03:28 jsheets Exp $
#
#--------------------------------------------------------------------------

#--------------------------------------------------------------------------
#                I N I T I A L I Z A T I O N 
#--------------------------------------------------------------------------

## Initialize variables


## WARNING!!  You must customize the following two directory paths
##  to point to the proper source directories on your system!

## Path to Twine source tree (top directory)
$twinedir = "/home/jsheets/cvs/twine";

## Path to Microsoft win32 source tree (for the header files)
$win32dir = "/home/jsheets/stuff/win32";


## Internally-used paths -- don't touch!
$mfcdir = "$twinedir/mfc/4.2";
$twine_include = "-I$twinedir/include -I$twinedir/include/twine -I$twinedir/windows";
$win32_include = "-I$win32dir/include";

$CC = "g++";

$CFLAGS = "-I$mfcdir/include -I/usr/X11R6/include  -DDEBUG  -DTWINE -D_REENTRANT  -fwritable-strings -fPIC -I../include -DDEBUG -g -D_X86_ -D_M_IX86 -DX386 -D_WIN32 -Dlinux -DTWIN32 -DTWIN_WINSOCK -D_AFX_PORTABLE -D_AFX_NO_DEBUG_CRT -D_AFX_NO_OCX_SUPPORT -D_AFX_NO_DAO_SUPPORT -D_AFX_NO_DB_SUPPORT -D_AFX_NO_RICHEDIT_SUPPORT -DTWINE_NO_CMONIKER -DNOAPIPROP -D_AFX_OLD_EXCEPTIONS -D__null=0";

#--------------------------------------------------------------------------
#                S U B R O U T I N E S
#--------------------------------------------------------------------------

sub parseHeaderList
{
    my ($gccDump) = @_;

    $newtext = "";
    $branch = "| ";
    $node = "+--";
    
    # Iterate through each line of $gccDump
    @headers = split(/\n/, $gccDump);
    foreach $line (@headers)
    {
	## Skip over non-header output from compiler
	if ($line =~ m/:/)
	{
	    next;
	}

	## Split line into an array of single chars
	@chars = split('', $line);

	## Count how many leading (well, total) spaces
	$spaces = grep(/ /, @chars);

	## Append tree formatting structures
	foreach $char (@chars)
	{
	    $spaces--;
	    if ($spaces > 0)
	    {
		$newtext .= $branch;
	    }
	    elsif ($spaces == 0)
	    {
		$newtext .= $node;
	    }
	    else
	    {
		$newtext .= $char;
	    }
	}

	$newtext .= "\n";
    }

    return($newtext);
}

#--------------------------------------------------------------------------
#                M A I N   R O U T I N E 
#--------------------------------------------------------------------------

## Create temporary test file

open(TESTFILE, "> test.c");
print TESTFILE "#include <afx.h>\n";
close(TESTFILE);

## Comment out the print statements below to turn off the dump
##  to stdout.  I'll put in command line arguments later...

## Run against Twine headers, post-process it, and write to file
$command = "$CC -c test.c -H $twine_include $CFLAGS";
$tmp = `$command 2>&1`;
$output = parseHeaderList($tmp);
print "Twine Command: \n$command\n\nTwine Results: \n$output";
open(FILE, "> twine_headers.txt");
print FILE $output;
close(FILE);

## Run against Win32 headers, post-process it, and write to file
$command = "$CC -c test.c -H $win32_include $CFLAGS";
$tmp = `$command 2>&1`;
$output = parseHeaderList($tmp);
print "Win32 Command: \n$command\n\nWin32 Results: \n$output";
open(FILE, "> win32_headers.txt");
print FILE $output;
close(FILE);


## Clean up our mess
system("rm test.c test.o");


#/*------------------------- < Full Revision History > ----------------------
#** $Log: headercrunch.pl,v $
#** Revision 1.1  1999/09/17 17:03:28  jsheets
#** Added new script to analyze header include paths
#**
#** Revision 1.1  1999/03/29 19:36:33  jwhite
#** Added
#**
#**
#**
#**-------------------------------------------------------------------------*/


