/*--------------------------------------------------------------------------
**      buffer.h                 Twin           From:  Twin/tools/rc
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc. 
**------------------------ < File Content Description > --------------------
**
**  Module:	tools/rc/buffer.h
**
**  Description: binarybuffer and other support structures for rc
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/tools/rc/buffer.h,v 1.3 1999/07/13 20:17:41 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/


/* I can't write this in C++... :,(
 *
 * Many of the structures in this file have corresponding functions that
 * go with them.  Generally, they take a pointer to the associated
 * structure (occassionally known as 'this', but you'll never "C" that
 * in here).  We try to track ownership through the yacc script...
 */

/* Some general things tend to happen quite frequently with many of these:
 *
 * foo_alloc: allocate a new foo.  Returns a foop (pointer to foo).
 * foo_free: free a foo.  Takes a foop, returns void.
 * foo_dup: make a copy of a foo.  The new foo is independent of the old one.
 *            Takes a foop, returns a different foop.
 * foo_cat: puts two foos together.  Takes two foops, returns one independent
 *            foop.
 */

typedef char *strp;

strp str_alloc(unsigned size);
void str_free(strp s);
strp str_dup(const char *s);

typedef struct binarybuffer
{
  unsigned len;
  unsigned alloc;
  void *buffer;
} *buffp;

/* Things to work with binary buffers */
#define ALLOCSIZE 32
buffp buff_alloc(unsigned size);
void buff_free(buffp b);
void *buff_ind(buffp b); /* Releases binarybuffer struct but leaves buffer intact */
buffp buff_unind(void *p, unsigned size); /* Makes buffp from arbitrary memory */
buffp buff_realloc(buffp b, unsigned size);

buffp buff_append_byte(buffp b, char c);
buffp buff_cat(buffp b1, buffp b2);
buffp buff_dup(buffp b1);
buffp buff_load(char *filename, int lookHere);

#define BUFF_STRUCT(b, v, t) (b) = buff_alloc(sizeof(t)); (b)->len = sizeof(t); (v) = (t*)(b)->buffer;

typedef struct tagRESOURCE
{
  const char *type;
  const char *name;
  buffp b;
  unsigned flags;
} RESOURCE, *resp;

resp res_new(void);
void res_free(resp);

struct maskvals
{
  int and;
  int or;
  long extendedFlags;
};

struct xyvals
{
  int x;
  int y;
};

struct dimvals
{
  struct xyvals corner;
  struct xyvals extent;
};

enum endianvals{little, big};

struct binflagvals
{
  int align;
  enum endianvals endian;
};

extern struct binflagvals defbinflags;

enum userrestype {shortint, longint, string, binary};

struct userresbuffer
{
  struct userresbuffer *next;
  enum userrestype t;
  union userresidata
  {
    unsigned long i;
    strp s;
    buffp b;
  } b;
};

/*------------------------- < Full Revision History > ----------------------
** $Log: buffer.h,v $
** Revision 1.3  1999/07/13 20:17:41  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/