/*--------------------------------------------------------------------------
**      LoadPE.c                 Twin           From:  Twin/win32
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	win32/LoadPE.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/win32/LoadPE.c,v 1.16 2000/01/05 22:02:35 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/

//#define VERBOSE 1
//#define SEVERE  1
//#define logpe LF_CONSOLE
#define logpe LF_LOGPE

#include <stdio.h>

#include "windows.h"

#include "peexe.h"

#include "Segment.h"

#include "Log.h"
#include "ctype.h"

#include "setjmp.h"
#include "LoadPE.h"
#include "BinTypes.h"
#include "Resources.h"
#include "Module.h"
#include "compat.h"
#include "Kernel.h"
#include "osdll.h"
#include "Debug.h"

extern char *GetTwinFilename();
extern TWINRT DefaultTwinRT32;

DWORD WinStubAPI(char *,char *,int);

DWORD PEGetProcAddress(MODULEINFO *, int ,char *);
void  FileErrorMB(int,char *);

static void DEBUG_LoadEntryPoints32( MODULEINFO *);
typedef struct
{
    struct datatype * type;
    DWORD seg;  /* 0xffffffff means current default segment (cs or ds) */
    DWORD off;
} DBG_ADDR;

void DEBUG_AddSymbol( char *, DBG_ADDR *, void * , int );

#define SYM_FUNC	 0x0
#define SYM_DATA	 0x1
#define SYM_WIN32	 0x2
/*
 * Load a PE file into memory
 * read headers first...
 * get file ImageBase
 * get file SizeOfImage
 *
 * VirtualAlloc(ImageBase,SizeofImage,...)
 * read section headers
 * for each section read section...
 */

static IMAGE_NT_HEADERS NTHeader[256];
static int NTHeaderCount;

#if DEBUG
void DumpHeader(PIMAGE_FILE_HEADER );
void DumpOptionalHeader(PIMAGE_OPTIONAL_HEADER );
#endif
void DumpSectionTable(void *,PIMAGE_SECTION_HEADER , unsigned );

int LoadImportsSection(char *,void * , PIMAGE_NT_HEADERS);
void LoadExportsTable(MODULEINFO *,PIMAGE_NT_HEADERS);
void LoadTlsTable(MODULEINFO *modinfo, PIMAGE_NT_HEADERS pNTHeader);

int ExecEntryPoint(MODULEINFO *);

int ExecPE(MODULEINFO *,LPUSERDLL,char *,WORD);

extern HMODULE    GetModuleFromInstance(HINSTANCE);
extern HINSTANCE  TWIN_GetInstanceFromTask(HTASK);

static void 
do_relocations(MODULEINFO *modinfo, PIMAGE_NT_HEADERS pNTHeader)
{
    	int 	delta  = modinfo->lpNTBaseAddress -
		(void *) (pNTHeader->OptionalHeader.ImageBase);

    	int	hdelta = (delta >> 16) & 0xFFFF;
    	int	ldelta = delta & 0xFFFF;
        IMAGE_DATA_DIRECTORY    dir;
	PIMAGE_BASE_RELOCATION  r;

	/* Nothing to do */
	if(delta == 0)
		return;

	dir = pNTHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC];
	
	r = MAKEPTR((PIMAGE_BASE_RELOCATION),
		modinfo->lpNTBaseAddress,
		dir.VirtualAddress);

	APISTR((logpe,"do_relocations: Actual Address=%p Desired=%p delta=%x\n",
    		modinfo->lpNTBaseAddress,
		pNTHeader->OptionalHeader.ImageBase,delta));

	APISTR((logpe,"BASE RELOCATIONS %x\n"));

	while(r->VirtualAddress) {
		char *page = (char*) RVA(modinfo->lpNTBaseAddress,
			r->VirtualAddress);
		int count = (r->SizeOfBlock - 8)/2;
		int i;

		APISTR((logpe,"%x RVA,   %x SizeOfBlock count=%d\n",
			r->VirtualAddress,r->SizeOfBlock,count));

		/* patching in reverse order */
		for(i=0;i<count;i++)
		{
			int offset = r->TypeOffset[i] & 0xFFF;
			int type = r->TypeOffset[i] >> 12;
			short *patch = (short *)(page+offset);

			switch(type)
			{
			case IMAGE_REL_BASED_ABSOLUTE: 
			        APISTR((logpe,"ABSOLUTE %p\n", patch));
				break;
			case IMAGE_REL_BASED_HIGH:
			        APISTR((logpe,"patching HI %p %x=%x\n", 
					patch,*patch, *patch + hdelta));
				*(short*)(page+offset) += hdelta;
				break;
			case IMAGE_REL_BASED_LOW:
			        APISTR((logpe,"patching LOW %p %x=%x\n", 
					patch,*patch, *patch + ldelta));
				*(short*)(page+offset) += ldelta;
				break;
			case IMAGE_REL_BASED_HIGHLOW:
			        APISTR((logpe,"%x HIGHLOW %p %x=%x\n", 
					offset,
					(int*)(page+offset),
					*(int*)(page+offset),
					*(int*)(page+offset) + delta));
#if 1
				*(int*)(page+offset) += delta;
#else
				{ int h=*(unsigned short*)(page+offset);
				  int l=r->TypeOffset[++i];
				  *(unsigned int*)(page + offset) = (h<<16) + l + delta;
				}
#endif
				break;
			case IMAGE_REL_BASED_HIGHADJ:
				APISTR((logpe,"Don't know what to do with IMAGE_REL_BASED_HIGHADJ\n"));
				break;
			case IMAGE_REL_BASED_MIPS_JMPADDR:
				APISTR((logpe,"Is this a MIPS machine ???\n"));
				break;
			default:
				APISTR((logpe,"Unknown fixup type\n"));
				break;
			}
		}
		r = (IMAGE_BASE_RELOCATION*)((char*)r + r->SizeOfBlock);
	}
}

HINSTANCE
adjustment(MODULEINFO *modinfo)
{
        LPTASKINFO lpTaskInfo;
	HINSTANCE hInst;
	HTASK     hTask;
	PIMAGE_NT_HEADERS pNTHeader;
	PIMAGE_FILE_HEADER pImageFileHeader;

	// we need an hInstance
	hInst = GlobalAlloc(GMEM_MOVEABLE,0);
	modinfo->hInst = hInst;
	modinfo->nRefCnt++;

	pNTHeader = modinfo->lpNTHeader;
	pImageFileHeader = &pNTHeader->FileHeader;

	hTask = GetCurrentTask();

        if ( pImageFileHeader->Characteristics & IMAGE_FILE_DLL) {
	} else {
		// bind the hModule to the hInst 
		// hModule comes from our modinfo
		lpTaskInfo = GETTASKINFO(hTask);
    		lpTaskInfo->hInst = hInst;
		ActivateTask(hTask);
	}

	CreateDataInstance(hInst,(HMODULE)modinfo->ObjHead.hObj,hTask);

	return hInst;
}

int
ExecPE(MODULEINFO *modinfo,LPUSERDLL lpUserDLL,char *lpCmdline,WORD flags)
{
	static void *BaseAddress;
	IMAGE_DOS_HEADER DosHeader;
 	PIMAGE_SECTION_HEADER pSectionHeaders;
	PIMAGE_NT_HEADERS pNTHeader;
	int i,len;
	int ret;
	HFILE hFile;
        DWORD reloc = 0,tlsdata = 0;
	
	/* read the dos image header first */
        hFile = lpUserDLL->hfExeFile;
    	_llseek(hFile,0,0);
        ret = _lread(hFile,&DosHeader,sizeof(IMAGE_DOS_HEADER));

	if(DosHeader.e_magic == IMAGE_DOS_SIGNATURE)	{

		/* now read in the nt header */
    		_llseek(hFile,DosHeader.e_lfanew,0);

		pNTHeader = &NTHeader[NTHeaderCount++];
		ret = _lread(hFile,pNTHeader, sizeof(IMAGE_NT_HEADERS));

		/* yes, it is a win32 header */
		if (pNTHeader->Signature != IMAGE_NT_SIGNATURE) {
			_lclose(hFile);
    			MessageBox((HWND)NULL,
				"No PE signature",
				"LoadPE error",MB_ICONSTOP);
			return 0;
		}
		BaseAddress = (void *) pNTHeader->OptionalHeader.ImageBase;
		BaseAddress = VirtualAlloc(
			BaseAddress,
			pNTHeader->OptionalHeader.SizeOfImage,
			MEM_COMMIT,
			PAGE_EXECUTE_READWRITE);

		if(BaseAddress == 0) {
        		IMAGE_DATA_DIRECTORY    dir;

        		/* We need to perform base relocations */
			dir = pNTHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC];
        		if (dir.Size)
            			reloc = dir.VirtualAddress;
        		else {
            			LOGSTR(( LF_CONSOLE,
                   "Need to relocate %s, but no relocation records present.\n",
			modinfo->lpFileName));
				return 0;
        		}

			BaseAddress = VirtualAlloc( 0,
				pNTHeader->OptionalHeader.SizeOfImage,
				MEM_COMMIT,
				PAGE_EXECUTE_READWRITE);
			
		} 

		if(BaseAddress == 0) {
			_lclose(hFile);
			LOGSTR((LF_CONSOLE, "cannot relocate binary\n"));
    			MessageBox((HWND)NULL,
				"Cannot relocate binary",
				"LoadPE error",MB_ICONSTOP);
			return 0;
		}

		logstr(LF_CONSOLE,"LoadModule32:   %s %8.8p (%p)\n",
			modinfo->lpFileName,BaseAddress,
			pNTHeader->OptionalHeader.ImageBase);

		modinfo->lpNTHeader = pNTHeader;
		modinfo->lpNTBaseAddress = BaseAddress;

		memcpy(BaseAddress,&DosHeader,sizeof(IMAGE_DOS_HEADER));

		memcpy(BaseAddress + DosHeader.e_lfanew,
			pNTHeader,sizeof(IMAGE_NT_HEADERS));


		pNTHeader = BaseAddress + DosHeader.e_lfanew;

#if SEVERE
		/* show the NT header */
		if(logpe)
		  DumpHeader(&pNTHeader->FileHeader);

		/* show the Optional header */
		if(logpe)
		  DumpOptionalHeader((PIMAGE_OPTIONAL_HEADER) 
			&pNTHeader->OptionalHeader);
#endif
		/************************************************/
		/* HACK store our modinfo in the OptionalHeader */
		/************************************************/
		pNTHeader->OptionalHeader.Reserved1 = (DWORD) modinfo;

		pSectionHeaders = (PIMAGE_SECTION_HEADER)
			((void *)BaseAddress + 		// Base Address
			DosHeader.e_lfanew +		// NT Header start
			sizeof(IMAGE_NT_HEADERS));	// NT Header

		/* now read the section headers */
		ret = _lread( hFile, pSectionHeaders, 
			sizeof(IMAGE_SECTION_HEADER)*
			pNTHeader->FileHeader.NumberOfSections);

		for(i=0; i < pNTHeader->FileHeader.NumberOfSections; i++) {
			void *LoadAddress;

			LoadAddress = 
			    RVA(BaseAddress,pSectionHeaders->VirtualAddress);

		        APISTR((logpe,"Section %d addr=%p name=%s\n",
				i,LoadAddress,pSectionHeaders->Name));

			/* load only non-BSS segments */
			if(!(pSectionHeaders->Characteristics &
				IMAGE_SCN_CNT_UNINITIALIZED_DATA)) {
			    _llseek(hFile,pSectionHeaders->PointerToRawData,SEEK_SET);
			    len = _lread(hFile,(char*) LoadAddress, 
				pSectionHeaders->SizeOfRawData);
	
			    if( len != pSectionHeaders->SizeOfRawData)
			    {
				logstr(LF_ERROR,"Failed to load section %x %x\n", i,len);
				exit(0);
			    }
			}
	
			/* thread local data... */
			if(strcmp(pSectionHeaders->Name,".rdata") == 0 ) {
				tlsdata++;
			}

			pSectionHeaders++;
		}

		_lclose(hFile);
	
		modinfo->dwWin32flag = 1;
    		modinfo->lptwinrt    = &DefaultTwinRT32;

		if(reloc) {
			do_relocations(modinfo,pNTHeader);
		}

		if(tlsdata) {
			APISTR((logpe,"has tls data\n"));
			LoadTlsTable(modinfo,pNTHeader);
		}

		adjustment(modinfo);

		/* we are dependent on other modules, go get and load those */
		if(LoadImportsSection(modinfo->lpFileName,BaseAddress,pNTHeader) == 0){
	    		FileErrorMB(ILL_FINAL,modinfo->lpFileName);
			return 0;
	        }

		//LoadExportsTable(modinfo,pNTHeader);

		ret = strlen(modinfo->lpFileName) + 2; 
		if(lpCmdline) {
			ret += strlen(lpCmdline);
		}

		modinfo->lpCmdline = WinMalloc(ret);

		strcpy(modinfo->lpCmdline,modinfo->lpFileName);
		if(lpCmdline) {
			strcat(modinfo->lpCmdline," ");
			strcat(modinfo->lpCmdline,lpCmdline);
		}
		DEBUG_LoadEntryPoints32( modinfo);

		ret = ExecEntryPoint( modinfo);

		return modinfo->hInst;
	}
	return 0;
}

typedef struct
{
        WORD    flag;
        LPSTR   name;
} WORD_FLAG_DESCRIPTIONS;

typedef struct
{
        DWORD   flag;
        LPSTR   name;
} DWORD_FLAG_DESCRIPTIONS;
#define IMAGE_FILE_DLL                  0x2000

/* Bitfield values and names for the IMAGE_FILE_HEADER flags */
WORD_FLAG_DESCRIPTIONS ImageFileHeaderCharacteristics[] =
{
{ IMAGE_FILE_RELOCS_STRIPPED, "RELOCS_STRIPPED" },
{ IMAGE_FILE_EXECUTABLE_IMAGE, "EXECUTABLE_IMAGE" },
{ IMAGE_FILE_LINE_NUMS_STRIPPED, "LINE_NUMS_STRIPPED" },
{ IMAGE_FILE_LOCAL_SYMS_STRIPPED, "LOCAL_SYMS_STRIPPED" },
{ IMAGE_FILE_MINIMAL_OBJECT, "MINIMAL_OBJECT" },
{ IMAGE_FILE_UPDATE_OBJECT, "UPDATE_OBJECT" },
{ IMAGE_FILE_BYTES_REVERSED_LO, "BYTES_REVERSED_LO" },
{ IMAGE_FILE_32BIT_MACHINE, "32BIT_MACHINE" },
{ IMAGE_FILE_PATCH, "PATCH" },
{ IMAGE_FILE_SYSTEM, "SYSTEM" },
{ IMAGE_FILE_DLL, "DLL" },
{ IMAGE_FILE_BYTES_REVERSED_HI, "BYTES_REVERSED_HI" },
{ IMAGE_FILE_DEBUG_STRIPPED,"DEBUGGING INFO STRIPPED" }
};

#define NUMBER_IMAGE_HEADER_FLAGS (sizeof(ImageFileHeaderCharacteristics) / sizeof(WORD_FLAG_DESCRIPTIONS))

void DumpHeader(PIMAGE_FILE_HEADER pImageFileHeader)
{
        UINT headerFieldWidth = 30;
        UINT i;
        char *szMachine;

        logstr(logpe,"File Header\n");

        switch( pImageFileHeader->Machine )
        {
                case IMAGE_FILE_MACHINE_I386:   szMachine = "i386"; break;
                case IMAGE_FILE_MACHINE_I860:   szMachine = "i860"; break;
                case IMAGE_FILE_MACHINE_R3000:  szMachine = "R3000"; break;
                case IMAGE_FILE_MACHINE_R4000:  szMachine = "R4000"; break;
                case IMAGE_FILE_MACHINE_ALPHA:  szMachine = "alpha"; break;
                default:    szMachine = "unknown"; break;
        }

        logstr(logpe,"  %-*s%04X (%s)\n", headerFieldWidth, "Machine:",
                                pImageFileHeader->Machine, szMachine);
        logstr(logpe,"  %-*s%04X\n", headerFieldWidth, "Number of Sections:",
                                pImageFileHeader->NumberOfSections);
        logstr(logpe,"  %-*s%08X\n", headerFieldWidth, "TimeDateStamp:",
                                pImageFileHeader->TimeDateStamp);
        logstr(logpe,"  %-*s%08X\n", headerFieldWidth, "PointerToSymbolTable:",
                                pImageFileHeader->PointerToSymbolTable);
        logstr(logpe,"  %-*s%08X\n", headerFieldWidth, "NumberOfSymbols:",
                                pImageFileHeader->NumberOfSymbols);
        logstr(logpe,"  %-*s%04X\n", headerFieldWidth, "SizeOfOptionalHeader:",
                                pImageFileHeader->SizeOfOptionalHeader);

        logstr(logpe,"  %-*s%04X\n", headerFieldWidth, "Characteristics:",
                                pImageFileHeader->Characteristics);

        for ( i=0; i < NUMBER_IMAGE_HEADER_FLAGS; i++ )
        {
                if ( pImageFileHeader->Characteristics &
                         ImageFileHeaderCharacteristics[i].flag )
                        logstr(logpe,"    %s\n", ImageFileHeaderCharacteristics[i].name );
        }
	logstr(logpe,"\n");
}

/* Bitfield values and names for the DllCharacteritics flags */
WORD_FLAG_DESCRIPTIONS DllCharacteristics[] =
{
{ IMAGE_LIBRARY_PROCESS_INIT, "PROCESS_INIT" },
{ IMAGE_LIBRARY_PROCESS_TERM, "PROCESS_TERM" },
{ IMAGE_LIBRARY_THREAD_INIT, "THREAD_INIT" },
{ IMAGE_LIBRARY_THREAD_TERM, "THREAD_TERM" },
};
#define NUMBER_DLL_CHARACTERISTICS (sizeof(DllCharacteristics) / sizeof(WORD_FLAG_DESCRIPTIONS))

/* Bitfield values and names for the LoaderFlags flags */
DWORD_FLAG_DESCRIPTIONS LoaderFlags[] =
{
{ IMAGE_LOADER_FLAGS_BREAK_ON_LOAD, "BREAK_ON_LOAD" },
{ IMAGE_LOADER_FLAGS_DEBUG_ON_LOAD, "DEBUG_ON_LOAD" }
};
#define NUMBER_LOADER_FLAGS (sizeof(LoaderFlags) / sizeof(DWORD_FLAG_DESCRIPTIONS))

/* Names of the data directory elements that are defined */
char *ImageDirectoryNames[] = {
        "EXPORT", "IMPORT", "RESOURCE", "EXCEPTION", "SECURITY", "BASERELOC",
        "DEBUG", "COPYRIGHT", "GLOBALPTR", "TLS", "LOAD_CONFIG" };

#define NUMBER_IMAGE_DIRECTORY_ENTRYS (sizeof(ImageDirectoryNames)/sizeof(char *))

/*
 * Dump the IMAGE_OPTIONAL_HEADER from a PE file
 */
void DumpOptionalHeader(PIMAGE_OPTIONAL_HEADER optionalHeader)
{
        UINT width = 30;
        char *s;
        UINT i;

        logstr(logpe,"Optional Header\n");

        logstr(logpe,"  %-*s%04X\n", width, "Magic", 
		optionalHeader->Magic);
        logstr(logpe,"  %-*s%u.%02u\n", width, "linker version",
                optionalHeader->MajorLinkerVersion,
                optionalHeader->MinorLinkerVersion);

        logstr(logpe,"  %-*s%X\n", width, "size of code", optionalHeader->SizeOfCode);
        logstr(logpe,"  %-*s%X\n", width, "size of initialized data",
                optionalHeader->SizeOfInitializedData);
        logstr(logpe,"  %-*s%X\n", width, "size of uninitialized data",
                optionalHeader->SizeOfUninitializedData);
        logstr(logpe,"  %-*s%X\n", width, "entrypoint RVA",
                optionalHeader->AddressOfEntryPoint);
        logstr(logpe,"  %-*s%X\n", width, "base of code", 
		optionalHeader->BaseOfCode);
        logstr(logpe,"  %-*s%X\n", width, "base of data", 
		optionalHeader->BaseOfData);
        logstr(logpe,"  %-*s%X\n", width, "image base", 
		optionalHeader->ImageBase);

        logstr(logpe,"  %-*s%X\n", width, "section align",
                optionalHeader->SectionAlignment);
        logstr(logpe,"  %-*s%X\n", width, "file align", 
		optionalHeader->FileAlignment);
        logstr(logpe,"  %-*s%u.%02u\n", width, "required OS version",
                optionalHeader->MajorOperatingSystemVersion,
                optionalHeader->MinorOperatingSystemVersion);
        logstr(logpe,"  %-*s%u.%02u\n", width, "image version",
                optionalHeader->MajorImageVersion,
                optionalHeader->MinorImageVersion);
        logstr(logpe,"  %-*s%u.%02u\n", width, "subsystem version",
                optionalHeader->MajorSubsystemVersion,
                optionalHeader->MinorSubsystemVersion);
        logstr(logpe,"  %-*s%X\n", width, "Reserved1", 
		optionalHeader->Reserved1);


        logstr(logpe,"  %-*s%X\n", width, "size of image", 
		optionalHeader->SizeOfImage);
        logstr(logpe,"  %-*s%X\n", width, "size of headers",
                optionalHeader->SizeOfHeaders);

        switch( optionalHeader->Subsystem )
        {
                case IMAGE_SUBSYSTEM_NATIVE: s = "Native"; break;
                case IMAGE_SUBSYSTEM_WINDOWS_GUI: s = "Windows GUI"; break;
                case IMAGE_SUBSYSTEM_WINDOWS_CUI: s = "Windows character"; break;
                case IMAGE_SUBSYSTEM_OS2_CUI: s = "OS/2 character"; break;
                case IMAGE_SUBSYSTEM_POSIX_CUI: s = "Posix character"; break;
                default: s = "unknown";
        }
        logstr(logpe,"  %-*s%04X (%s)\n", width, "Subsystem",
                        optionalHeader->Subsystem, s);

        logstr(logpe,"  %-*s%04X\n", width, "DLL flags",
                        optionalHeader->DllCharacteristics);
        for ( i=0; i < NUMBER_DLL_CHARACTERISTICS; i++ )
        {
                if ( optionalHeader->DllCharacteristics &
                         DllCharacteristics[i].flag )
                        logstr(logpe,"  %s\n", 
				DllCharacteristics[i].name );
        }
        if ( optionalHeader->DllCharacteristics )
                logstr(logpe,"\n");

        logstr(logpe,"  %-*s%X\n", width, "stack reserve size",
                optionalHeader->SizeOfStackReserve);
        logstr(logpe,"  %-*s%X\n", width, "stack commit size",
                optionalHeader->SizeOfStackCommit);
        logstr(logpe,"  %-*s%X\n", width, "heap reserve size",
                optionalHeader->SizeOfHeapReserve);
        logstr(logpe,"  %-*s%X\n", width, "heap commit size",
                optionalHeader->SizeOfHeapCommit);

        logstr(logpe,"  %-*s%08X\n", width, "loader flags",
                optionalHeader->LoaderFlags);

        for ( i=0; i < NUMBER_LOADER_FLAGS; i++ )
        {
                if ( optionalHeader->LoaderFlags &
                         LoaderFlags[i].flag )
                        logstr(logpe,"  %s", LoaderFlags[i].name );
        }

        if ( optionalHeader->LoaderFlags )
                logstr(logpe,"\n");

        logstr(logpe,"  %-*s%X\n", width, "RVAs & sizes",
                optionalHeader->NumberOfRvaAndSizes);

        logstr(logpe,"Data Directory\n");
        for ( i=0; i < optionalHeader->NumberOfRvaAndSizes; i++)
        {
                logstr(logpe,"  %-12s rva: %08X  size: %08X\n",
                        (i >= NUMBER_IMAGE_DIRECTORY_ENTRYS)
                                ? "unused" : ImageDirectoryNames[i],
                        optionalHeader->DataDirectory[i].VirtualAddress,
                        optionalHeader->DataDirectory[i].Size);
        }
}

/* Bitfield values and names for the IMAGE_SECTION_HEADER flags */
DWORD_FLAG_DESCRIPTIONS SectionCharacteristics[] = 
{
{ IMAGE_SCN_CNT_CODE, "CODE" },
{ IMAGE_SCN_CNT_INITIALIZED_DATA, "INITIALIZED_DATA" },
{ IMAGE_SCN_CNT_UNINITIALIZED_DATA, "UNINITIALIZED_DATA" },
{ IMAGE_SCN_LNK_INFO, "LNK_INFO" },
{ IMAGE_SCN_LNK_OVERLAY, "LNK_OVERLAY" },
{ IMAGE_SCN_LNK_REMOVE, "LNK_REMOVE" },
{ IMAGE_SCN_LNK_COMDAT, "LNK_COMDAT" },
{ IMAGE_SCN_MEM_DISCARDABLE, "MEM_DISCARDABLE" },
{ IMAGE_SCN_MEM_NOT_CACHED, "MEM_NOT_CACHED" },
{ IMAGE_SCN_MEM_NOT_PAGED, "MEM_NOT_PAGED" },
{ IMAGE_SCN_MEM_SHARED, "MEM_SHARED" },
{ IMAGE_SCN_MEM_EXECUTE, "MEM_EXECUTE" },
{ IMAGE_SCN_MEM_READ, "MEM_READ" },
{ IMAGE_SCN_MEM_WRITE, "MEM_WRITE" },
};

#define NUMBER_SECTION_CHARACTERISTICS  (sizeof(SectionCharacteristics) / sizeof(DWORD_FLAG_DESCRIPTIONS))

/*
** Dump the section table from a PE file or an OBJ
*/

void 
DumpSectionTable(void *LoadAddress,PIMAGE_SECTION_HEADER section, unsigned cSections)
{
	unsigned j;

	logstr(LF_CONSOLE,"Section Table %x %d\n",LoadAddress,cSections);
	
	logstr(LF_CONSOLE,"  %02X %-8.8s  %s: %08X  VirtAddr:  %08X\n",
			cSections, section->Name, "VirtSize",
			section->Misc.VirtualSize, section->VirtualAddress);
	logstr(LF_CONSOLE,"    raw data offs:   %08X  raw data size: %08X\n",
			section->PointerToRawData, section->SizeOfRawData );
	logstr(LF_CONSOLE,"    relocation offs: %08X  relocations:   %08X\n",
			section->PointerToRelocations, section->NumberOfRelocations);
	logstr(LF_CONSOLE,"    line # offs:     %08X  line #'s:      %08X\n",
			section->PointerToLinenumbers, section->NumberOfLinenumbers );
	logstr(LF_CONSOLE,"    characteristics: %08X\n", section->Characteristics);

	for ( j=0; j < NUMBER_SECTION_CHARACTERISTICS; j++ )
	{
		if ( section->Characteristics & 
			SectionCharacteristics[j].flag )
			logstr(LF_CONSOLE,"      %s\n", 
				SectionCharacteristics[j].name );
	}
}

/* */
/* Dump the imports table (the .idata section) of a PE file */
/* */
int LoadImportsSection(char *BaseName,void * BaseAddress, PIMAGE_NT_HEADERS pNTHeader)
{
	PIMAGE_IMPORT_DESCRIPTOR importDesc;
	PIMAGE_THUNK_DATA thunk;
	PIMAGE_IMPORT_BY_NAME pOrdinalName;
        IMAGE_DATA_DIRECTORY    dir;
	char *libname;
	DWORD faddr;
	DWORD *ip;
	char *ModuleName = BaseName;
		
	dir=pNTHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT];
	importDesc = MAKEPTR((PIMAGE_IMPORT_DESCRIPTOR),
		BaseAddress,
		dir.VirtualAddress);

 	logstr(logpe,"  DLL Imports Table: %s\n", ModuleName);

	while ( 1 )
	{
		HINSTANCE     hInst;
		HMODULE   hModule;
		MODULEINFO *modinfo;

		/* See if we've reached an empty IMAGE_IMPORT_DESCRIPTOR */
		if ( (importDesc->TimeDateStamp==0 ) && (importDesc->Name==0) )
			break;

		libname = (PBYTE)(importDesc->Name) + (int)BaseAddress;

#if VERBOSE
		logstr(logpe,"  Hint/Name Table: %08X\n", 
			importDesc->u.Characteristics);
 		logstr(logpe,"  TimeDateStamp:   %08X\n", 
			importDesc->TimeDateStamp);
 		logstr(logpe,"  ForwarderChain:  %08X\n", 
			importDesc->ForwarderChain);
 		logstr(logpe,"  First thunk RVA: %08X\n", 
			importDesc->FirstThunk);
#endif
	
		logstr(logpe,"  Library: %s\n",libname); 

		/* these are the equivalents... */

		hInst = LoadLibrary(libname);

	        if (hInst == 0) {
		   char buffer[256];
		   strncpy(buffer,libname,200);
		   strcat(buffer," File not found.");

    		   MessageBox((HWND)NULL,buffer,"LoadPE error",MB_OK);
		   logstr(logpe,"LoadLibrary failed: %s\n",libname); 
		   return 0;
		}

		hModule = GetModuleFromInstance(hInst);

		if(hModule == 0) {
		   logstr(logpe,"LoadLibrary failed: %s\n",libname); 
		   return 0;
		}

		modinfo = GETMODULEINFO(hModule);

		if(modinfo == 0) {
		   logstr(logpe,"LoadLibrary failed: %s\n",libname); 
		   return 0;
		}

		logstr(logpe,"RESOLVE exports from %s\n",libname); 

		ip    = MAKEPTR((DWORD *),BaseAddress,importDesc->FirstThunk);
		thunk = MAKEPTR((PIMAGE_THUNK_DATA),BaseAddress,importDesc->u.Characteristics);

		if(importDesc->u.Characteristics == 0)
			thunk = (PIMAGE_THUNK_DATA) ip;

		while (1) {
			if (thunk->u1.AddressOfData == 0)
			   break;

			pOrdinalName = thunk->u1.AddressOfData;

			thunk++;	

			if((int) pOrdinalName & 0x80000000) {
			     int ordinal = (int)pOrdinalName & 0x7fffffff;

			     faddr = PEGetProcAddress(modinfo,ordinal,0);

			     logstr(logpe,
				"    import ORDINAL %p %s\t@%p %s\n",
				(void *) ordinal,libname,ip, 
				faddr?"":"TRAP NO ELEMENT");

			      if (faddr == 0) {
				faddr = (DWORD) WinStubAPI(libname,
						"ORDINAL",
						ordinal);
			      }
			     *ip = faddr;

			} else {
			     PIMAGE_IMPORT_BY_NAME pimage;
			     pimage = (PIMAGE_IMPORT_BY_NAME) 
				     ((PBYTE) pOrdinalName + (int)BaseAddress);

			     faddr = PEGetProcAddress(modinfo,
					pimage->Hint,pimage->Name);

			     logstr(logpe,
				"    import %-24s %s\t@%p %4d %s addr=%p\n",
				pimage->Name,libname,ip, 
				pimage->Hint,faddr?"":"TRAP",faddr);

			      if (faddr == 0) {
				faddr = (DWORD) WinStubAPI(libname,
						pimage->Name,
						pimage->Hint);
			      }
			     *ip = faddr;
			}
			ip++;
		}

		importDesc++;	/* advance to next IMAGE_IMPORT_DESCRIPTOR */
	}
	return 1;
}

void
LoadTlsTable(MODULEINFO *modinfo, 
	     PIMAGE_NT_HEADERS pNTHeader)
{
	PIMAGE_TLS_DIRECTORY    pdir;
    	DWORD datasize,blocksize;
        IMAGE_DATA_DIRECTORY    dir;
	int index;
	char *mem;
   
	/* get the image_data_directory for exported symbols */
	dir = pNTHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_TLS];

        /*---------------------------------------------------------
        **  JPW 3/18/99 - Exit if there doesn't appear to be any...
        **-------------------------------------------------------*/
        if (! dir.VirtualAddress)
            return;

	/* convert to pointer */
	pdir = MAKEPTR((PIMAGE_TLS_DIRECTORY),
		modinfo->lpNTBaseAddress,
		dir.VirtualAddress);

	datasize= pdir->EndAddressOfRawData-
		  pdir->StartAddressOfRawData;
       	blocksize    = datasize + pdir->SizeOfZeroFill;

        index = TlsAlloc();

	*pdir->AddressOfIndex=index;

	logstr(logpe,"tls datasize=%d zero=%d\n",
		datasize,blocksize);

        mem= WinMalloc(blocksize);
	memset(mem,0,blocksize);

        memcpy(mem,(LPVOID)pdir->StartAddressOfRawData,datasize);

	//thdb = ...
        //thdb->tls_array[index] = mem;

	// we are in the right context...
	TlsSetValue(index,mem);
}

void 
LoadExportsTable(MODULEINFO *modinfo, PIMAGE_NT_HEADERS pNTHeader)
{
	PIMAGE_EXPORT_DIRECTORY exportDir;
	DWORD i;
	PDWORD functions;
	PWORD ordinals;
	PSTR *names;
	LPSTR fname;
        IMAGE_DATA_DIRECTORY    dir;
        char *ModuleName = modinfo->lpFileName;

	/* get the image_data_directory for exported symbols */
	dir = pNTHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT];

	/* convert to pointer */
	exportDir = MAKEPTR((PIMAGE_EXPORT_DIRECTORY),
		modinfo->lpNTBaseAddress,
		dir.VirtualAddress);

	/* any functions at all? */
	if (exportDir->NumberOfFunctions == 0)
	   return ;

	logstr(logpe,"Exports of %s\n",ModuleName);

	/* create array of function addresses */
	functions = MAKEPTR((PDWORD),modinfo->lpNTBaseAddress,
		exportDir->AddressOfFunctions);

	/* create array of function ordinals */
	ordinals =  MAKEPTR((PWORD),modinfo->lpNTBaseAddress,
		exportDir->AddressOfNameOrdinals);

	/* create array of RVA's to function names */
	names =      MAKEPTR((PSTR *),modinfo->lpNTBaseAddress,
		exportDir->AddressOfNames);
	
	/* resort to straight line linear search */
	/* should/could we ignore comparing strings, until we */
	/* are in range? */
	for ( i=0; i < exportDir->NumberOfNames; i++ )
	{
		/* get the pointer to the name */
		fname = MAKEPTR((LPSTR),modinfo->lpNTBaseAddress,*names);

	        logstr(logpe,"export %x %s %p\n",
			*ordinals,fname,functions);

		/* bump all the pointers */
		functions++;	/* function address */
		ordinals++;	/* function ordinal */
		names++;	/* function name */
	}

	/* the name is not in the module... */
	return ;
}

extern WORD AssignSelector(LPBYTE , WORD , BYTE , DWORD );
extern void debuggerbreak();

void
invoke_binary32(int * fp, void *lpNTBaseAddress)
{
	FARPROC entry;
	char * dobreak;

	entry = (FARPROC) fp;

	APISTR((LF_CONSOLE,"invoke_binary(%p)\n",entry));
	dobreak = (char *) getenv("WIN_INVOKEBREAK");
	if(dobreak)
		debuggerbreak();

	entry(lpNTBaseAddress, 0);
}

int
ExecEntryPoint(MODULEINFO *modinfo)
{
	void *BaseAddress; 
	PIMAGE_NT_HEADERS pNTHeader;
	int *fp;
	PIMAGE_FILE_HEADER pImageFileHeader;
	FARPROC	    entry;
	char *lpszModuleName = modinfo->lpFileName;
	THDB *pThread;

	BaseAddress = modinfo->lpNTBaseAddress;
	pNTHeader   = modinfo->lpNTHeader;

	fp = MAKEPTR((int *), BaseAddress, 
			pNTHeader->OptionalHeader.AddressOfEntryPoint);

	if(fp == 0)
		return modinfo->hInst;

	APISTR((LOGFLAG(CALL,BIN),"invoke_binary32: %s entrypoint=%p\n", 
		lpszModuleName,fp));

	pImageFileHeader = &pNTHeader->FileHeader;

	modinfo->exit_code = 0;
	pThread = osdll_thread_gettdb();

        if ( pImageFileHeader->Characteristics & IMAGE_FILE_DLL) {
		if(setjmp(modinfo->env) == 0) {

			entry = (FARPROC) fp;
			entry(modinfo->hInst,DLL_PROCESS_ATTACH,0);
			longjmp(modinfo->env,1);
		} 
	} else 
	{
		if(setjmp(modinfo->env) == 0) {

			invoke_binary32(fp, modinfo->lpNTBaseAddress);
			longjmp(modinfo->env,1);
		} 
	}

	APISTR((LOGFLAG(RET,BIN),"invoke_binary32: %s entrypoint=%p returns %d\n",
		lpszModuleName,fp,modinfo->exit_code));

	return modinfo->hInst;
}

long
LibMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow)
{
	APISTR((logpe,"LibMain(HINSTANCE %x, LPSTR %p, int %d)\n",
		hInst,hPrevInst,lpCmdLine,nCmdShow));

	return 0;
}

void DEBUG_LoadEntryPoints(void)
{
	APISTR((logpe,"LoadEntryPoints\n"));
}

/***********************************************************************
 *           DEBUG_LoadEntryPoints32
 *
 * Load the entry points of a Win32 module into the hash table.
 */
static void DEBUG_LoadEntryPoints32( MODULEINFO *modinfo)
{
    DBG_ADDR addr;
    char buffer[256];
    int i, j;
    IMAGE_EXPORT_DIRECTORY *exports;
    IMAGE_DATA_DIRECTORY dir;
    WORD *ordinals;
    void **functions;
    const char **names;
    void *BaseAddress; 
    PIMAGE_NT_HEADERS pNTHeader;

    BaseAddress = modinfo->lpNTBaseAddress;
    pNTHeader   = modinfo->lpNTHeader;

    addr.seg = 0;
    addr.type = NULL;

    /* Add start of DLL */

    addr.off = (DWORD) BaseAddress;
    DEBUG_AddSymbol( modinfo->lpModuleName, &addr, NULL, SYM_WIN32 | SYM_FUNC );

    /* Add entry point */
    sprintf( buffer, "%s.EntryPoint", modinfo->lpModuleName);
    addr.off = (DWORD) MAKEPTR((int *), BaseAddress, 
			pNTHeader->OptionalHeader.AddressOfEntryPoint);

    DEBUG_AddSymbol( buffer, &addr, NULL, SYM_WIN32 | SYM_FUNC );

    	/* Add exported functions */

	/* get the image_data_directory for exported symbols */
	dir = pNTHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT];
        if (!dir.Size) return;

	/* convert to pointer */
	exports = MAKEPTR((PIMAGE_EXPORT_DIRECTORY),
		modinfo->lpNTBaseAddress,
		dir.VirtualAddress);

	/* any functions at all? */
	if (exports->NumberOfFunctions == 0)
	   return ;

	/* create array of function ordinals */
	ordinals =  MAKEPTR((PWORD),modinfo->lpNTBaseAddress,
		exports->AddressOfNameOrdinals);

	/* create array of RVA's to function names */
	names =      (const char **) MAKEPTR((PSTR *),modinfo->lpNTBaseAddress,
		exports->AddressOfNames);

	/* create array of function addresses */
	functions = (void **) MAKEPTR((PDWORD),modinfo->lpNTBaseAddress,
		exports->AddressOfFunctions);

    for (i = 0; i < exports->NumberOfNames; i++)
    {
        if (!names[i]) continue;
        sprintf( buffer, "%s.%s", modinfo->lpModuleName, 
		(char *)RVA(BaseAddress,(int)names[i]) );
        addr.off = (DWORD) RVA( BaseAddress,(int)functions[ordinals[i]] );
        DEBUG_AddSymbol( buffer, &addr, NULL, SYM_WIN32 | SYM_FUNC );
    }

    for (i = 0; i < exports->NumberOfFunctions; i++)
    {
        if (!functions[i]) continue;
        /* Check if we already added it with a name */
        for (j = 0; j < exports->NumberOfNames; j++)
            if ((ordinals[j] == i) && names[j]) break;
        if (j < exports->NumberOfNames) continue;
        sprintf( buffer, "%s.%ld",modinfo->lpModuleName , i + exports->Base );
        addr.off = (DWORD)RVA( BaseAddress,(int)functions[i] );
        DEBUG_AddSymbol( buffer, &addr, NULL, SYM_WIN32 | SYM_FUNC );
    }

#if 0
    // add debug symbol information...
    dir = &PE_HEADER(hModule)->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_DEBUG];
    if (dir->Size)
        DEBUG_RegisterDebugInfo(hModule, name, dir->VirtualAddress, dir->Size);
#endif
}

void
DEBUG_AddSymbol( char *buffer, DBG_ADDR *addr, void * null, int flag)
{
	DBG_INFO dbg;

	dbg.symbol = buffer;
	dbg.address = (void *)addr->off;
	dbg.flag   = flag;

	TWIN_DebuggerProc(TWIN_DBGADDS,0,&dbg);
}
/*------------------------- < Full Revision History > ----------------------
** $Log: LoadPE.c,v $
** Revision 1.16  2000/01/05 22:02:35  robf
** logging
**
** Revision 1.15  1999/12/23 23:13:35  jwhite
** Add an additional NULl parameter to the invocation of the binary.
**
** Revision 1.14  1999/12/19 21:20:43  jwhite
** When invoking a VC6 binary, it appears to be necessary to supply it
** with the base address of the NT exectuable.
**
** Revision 1.13  1999/12/13 18:05:07  robf
** code cleanup
**
** Revision 1.12  1999/12/08 07:04:30  robf
** fix globalalloc
**
** Revision 1.11  1999/11/30 04:09:11  robf
** new pe loader
**
** Revision 1.8  1999/07/13 20:17:44  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/
