/*--------------------------------------------------------------------------
**      Debug.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/Debug.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/Debug.c,v 1.3 1999/11/29 20:35:22 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>
#include <signal.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Debug.h"

#ifdef X386
void TWIN_RegisterDebugSignal(int);
#endif

void
TWIN_DebuggerProc(int opcode, int wparam, void *lparam)
{
	static void (*debugger)(int,int,void *);

	switch(opcode) {
	/* callback from debugger shared library */
	/* set debugger function */
	case TWIN_DBGINIT:
		debugger = lparam;
		return;

	/* routine to load the debugger shared library */
	/* loads it, and if it successfully called back */
	/* calls its initialization point */
	case TWIN_DBGLOAD:
#ifdef X386
		if(debugger == 0) 
        		LoadLibrary("debug");
		if(debugger)  {
			TWIN_RegisterDebugSignal(SIGINT);
			TWIN_RegisterDebugSignal(SIGTRAP);
		}
#endif

		break;

	case TWIN_DBGADDS:
	case TWIN_DBGADDR:
	case TWIN_DBGCALL:
		/* fall through, and call debugger */
		break;

	default:
		FatalAppExit(0,"Unknown DebuggerProc opcode\n");
	}

	if(debugger) 
		debugger(opcode,wparam,lparam);
}

/*------------------------- < Full Revision History > ----------------------
** $Log: Debug.c,v $
** Revision 1.3  1999/11/29 20:35:22  robf
** added debugger add symbol
**
** Revision 1.2  1999/07/09 21:55:39  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
