/*--------------------------------------------------------------------------
**      ExecLinux.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/ExecLinux.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/Execlinux.c,v 1.3 1999/07/09 21:55:40 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>
#include <signal.h>
#include <setjmp.h>
#include <errno.h>
#include <syscall.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "ldt.h"
#include "windows.h"
#include "DPMI.h"
#include "kerndef.h"		/* BINADDR */
#include "BinTypes.h"		/* ENV structure */
#include "Log.h"
#include "dos.h"
#include "Exec.h"
#include "Debug.h"

#define	STACK_DWORDS	0x4000

struct lsigaction
{
    void (*lsa_handler)();
    unsigned long lsa_mask;
    unsigned long lsa_flags;
    void (*lsa_restorer)();
};

static struct lsigaction sact;
static unsigned long TWIN_Stack[STACK_DWORDS];

#define sys_modify_ldt(a, b) syscall(123,1,(void *)a,(int)b)
#define sys_read_ldt(a, b) syscall(123,2,(void *)a,(int)b)

#define SIGACTION wine_sigaction

/* Similar to the sigaction function in libc, except it leaves alone the
   restorer field, which is used to specify the signal stack address */
static __inline__ int wine_sigaction( int sig, struct lsigaction *new,
                                      struct lsigaction *old )
{
    __asm__ __volatile__( "pushl %%ebx\n\t"
                          "movl %2,%%ebx\n\t"
                          "int $0x80\n\t"
                          "popl %%ebx"
                          : "=a" (sig)
                          : "0" (SYS_sigaction),
                            "g" (sig),
                            "c" (new),
                            "d" (old) );
    return 0;
}

int sysi86(int func, void *data)
{
    struct modify_ldt_ldt_s ldt_info;
    struct ssd *req;
    unsigned short wFlags;

    if (func == SI86DSCR)
    {
	req = (struct ssd *) data;
	wFlags = (unsigned short) req->acc1;
	wFlags |= ((unsigned short) req->acc2 << 8);

	ldt_info.entry_number = (unsigned int)(req->sel >> 3);
	ldt_info.base_addr = req->bo;

	if (req->ls == 0) 
	{
	    ldt_info.limit = 0;
	    ldt_info.limit_in_pages = 0;
	}
	else if ((req->ls - 1) > 0xFFFFF || (wFlags & DF_PAGES)) 
	{
	    ldt_info.limit = (req->ls - 1) >> 12;
	    ldt_info.limit_in_pages = 1;
	}
	else 
	{
	    ldt_info.limit = req->ls - 1;
	    ldt_info.limit_in_pages = 0;
	}

	ldt_info.seg_32bit = (wFlags & DF_32)?1:0;

	if (wFlags & DF_CODE) 
	    ldt_info.contents = MODIFY_LDT_CONTENTS_CODE;
	else if (wFlags & DF_EXPANDDOWN)
	    ldt_info.contents = MODIFY_LDT_CONTENTS_STACK;
	else
	    ldt_info.contents = MODIFY_LDT_CONTENTS_DATA;

	ldt_info.read_exec_only = (wFlags & DF_DWRITEABLE) ? 0 : 1;
	ldt_info.seg_not_present = (wFlags & DF_PRESENT) ? 0 : 1;
	return sys_modify_ldt(&ldt_info,sizeof(ldt_info));
    }
    else
	return -1;
}

static void
TWIN_GPFSigHandler(int sig)
{
    struct TWIN_i386_context_s  *sc;
    
    sc = (struct TWIN_i386_context_s *) ((long *) &sig + 1);
    TWIN_GPFHandler(sig, sc);
    TWIN_ValidateSegmentRegisters(sc);
}

static void
TWIN_DebugSigHandler (int sig)
{
    struct TWIN_i386_context_s  *sc;

    sc = (struct TWIN_i386_context_s *) ((long *) &sig + 1);

    if (sig == SIGTRAP)
	sc->eip--;
    
    DEBUG_TASK(sig, sc);
    TWIN_ValidateSegmentRegisters(sc);
}

int
TWIN_RegisterGPFSignal(int sig)
{
	int rc;
    sact.lsa_handler  = TWIN_GPFSigHandler;
    sact.lsa_flags    = SA_RESTART|SA_NOMASK;
    sact.lsa_restorer = (void (*)()) &TWIN_Stack[STACK_DWORDS-1];
    sact.lsa_mask     = 0;
    rc =  SIGACTION(sig, &sact, NULL);
    return rc;
}

int
TWIN_RegisterDebugSignal(int sig)
{
    int rc;
    sact.lsa_handler = TWIN_DebugSigHandler;
    sact.lsa_flags = 0;
    sact.lsa_restorer = (void (*)()) &TWIN_Stack[STACK_DWORDS-1];
    rc =  SIGACTION(sig, &sact, NULL);
    return rc;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: Execlinux.c,v $
** Revision 1.3  1999/07/09 21:55:40  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
