/*--------------------------------------------------------------------------
**      Kernel.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/Kernel.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/Kernel.c,v 1.14 2000/01/21 20:25:49 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "WinDefs.h"
#include "kerndef.h"
#include "Resources.h"
#include "Module.h"
#include "Kernel.h"
#include "Driver.h"
#include "Log.h"
#include "WinMalloc.h"
#include "Strings32.h"
#include "thread.h"
#include "osdll.h"

/******************************************************************************/

LPQUEUE AllocQueue(int);

/* external functions */
extern BOOL MenuMouseMove(LPMSG);
extern BOOL TWIN_ForwardButtonEvent(LPMSG);
extern BOOL HaveDisabledAncestor(HWND);
extern void debuggerbreak();

#ifdef TWIN_BIN32
extern HINSTANCE TWIN_InstanceFromModule32(void *);
#endif
extern HINSTANCE TWIN_GetInstanceFromTask( HTASK );

/*****************************************************************************/

LPQUEUE  SystemQueue;

/*****************/
/* API functions */
/*****************/

LONG WINAPI
SetSwapAreaSize(UINT uiSize)
{
    APISTR((LF_APISTUB,"SetSwapAreaSize(UINT=%x)\n", uiSize));
    return MAKELONG((uiSize)?uiSize:0x8000,0xc000);
}

BOOL WINAPI
DefineHandleTable(WORD wOffset)
{
    APISTR((LF_APISTUB,"DefineHandleTable(WORD=%x)\n", wOffset));
    return TRUE;
}

UINT WINAPI
RegisterWindowMessage(LPCSTR lpszMessage)
{
    UINT rc;
    APISTR((LF_APICALL,"RegisterWindowMessage(LPCSTR=%s)\n", lpszMessage));
    rc = GlobalAddAtom(lpszMessage);
    APISTR((LF_APIRET,"RegisterWindowMessage: returns UINT %x\n",rc));
    return rc;
}

DWORD
GetShortPathName(LPCTSTR inpath,LPTSTR outpath,DWORD len)
{
	char pathname[256];
	char *p;
	int  n;
        APISTR((LF_APICALL,"GetShortPathName(LPSTR=%p,LPSTR=%p,DWORD=%d)\n",
		inpath,outpath,len));

	strncpy(pathname,inpath,256);
	if((p = strrchr(pathname,'\\'))) {
		p++;
	} else
		p = pathname;

	if(len < strlen(p))
		n = len;
	else	n = strlen(p);
	strncpy(outpath,p,n);
	
        APISTR((LF_APIRET,"GetShortPathName returns int %d\n",n));
	return n;
}


#ifdef TWIN_BIN32

FARPROC
SetUnhandledExceptionFilter(FARPROC fp)
{
        APISTR((LF_API,"SetUnhandleExceptionFilter(FARPROC=%p)\n",fp));
	return (FARPROC) 0;
}


UINT WINAPI
RegisterWindowMessageA(LPCSTR lpszMessage)
{
    UINT rc;
    APISTR((LF_APICALL,"RegisterWindowMessageA(LPCSTR=%s)\n", lpszMessage));
    rc = GlobalAddAtom(lpszMessage);
    APISTR((LF_APIRET,"RegisterWindowMessageA: returns UINT %x\n",rc));
    return rc;
}

typedef struct _exception {
	struct _exception *prev;
	FARPROC fn;
} EXCEPTION_FRAME;
	
typedef struct __except_record {
	DWORD ExceptionCode;
	DWORD ExceptionFlags;
	struct __except_record *ExceptionRecord;
	void  *ExceptionAddress;
	DWORD nParams;
	UINT  ExceptionInfo[15];
} EXCEPTION_RECORD;

typedef enum _exception_disposition {
	ExceptionContinueExecution,
	ExceptionContinueSearch,
	ExceptionNestedException,
} EXCEPTION_DISPOSITION;

typedef EXCEPTION_DISPOSITION (*handler)(EXCEPTION_RECORD *,void *,void *,void *);

VOID 
RaiseException(
	DWORD dwExceptionCode,
	DWORD dwExceptionFlag,
	DWORD nNumberArgs,
	CONST DWORD *lpArguments)
{
    EXCEPTION_FRAME *seh;
    TEB * teb = NtCurrentTeb();
    EXCEPTION_DISPOSITION rc;
    EXCEPTION_RECORD r;
    int i;

    APISTR((LF_CONSOLE,"RaiseExeception(%x,%x,%x,%x)\n", 
	dwExceptionCode,
	dwExceptionFlag,
	nNumberArgs,
	lpArguments ));
    if(teb) {
	r.ExceptionCode = dwExceptionCode;
	r.ExceptionFlags = dwExceptionFlag;
	r.nParams = nNumberArgs;

	for(i=0;i<nNumberArgs;i++) 	
		r.ExceptionInfo[i] = lpArguments[i];

    	for(seh = teb->except;seh && seh != 0xffffffff;seh=seh->prev) {
		rc = seh->fn(&r,seh,0,0);
		switch(rc) {
		case ExceptionContinueExecution:
    			APISTR((LF_CONSOLE,"exceptioncontinue...\n")); 
			if(!(r.ExceptionFlags & 1))
				return;
    			APISTR((LF_CONSOLE,"exceptioncontinue not continuable...\n")); 
			debuggerbreak();

		case ExceptionContinueSearch:
			break;
		case ExceptionNestedException:
		default:
    			APISTR((LF_CONSOLE,"exception nested or default\n")); 
			debuggerbreak();
		}
	}	
    	APISTR((LF_CONSOLE,"reached end of exception chain\n")); 
	debuggerbreak();
    }

    debuggerbreak();
    APISTR((LF_CONSOLE,"RaiseException: returns void\n"));
    return;
}

void WINAPI
RtlUnwind(EXCEPTION_FRAME *pEndFrame,LPVOID unused,EXCEPTION_RECORD *p,DWORD eax)
{
    	int depth;
    	EXCEPTION_FRAME *seh;
    	EXCEPTION_DISPOSITION rc;
    	EXCEPTION_RECORD r;
    	TEB * teb = NtCurrentTeb();
	
    	APISTR((LF_CONSOLE,"RtlUnwind(%x,%x,%x,%x)\n", 
		pEndFrame,
		unused,
		p,
		eax ));
    	if(teb) {
		depth = 0;
		if(!p) {
			r.ExceptionCode =  0xc0000027;
			r.ExceptionFlags = 0;
			r.ExceptionAddress = RtlUnwind;
			r.ExceptionRecord  = 0;
			r.nParams = 0;
	
			p = &r;
		}

		p->ExceptionFlags |= 2;
		if(pEndFrame)
			p->ExceptionFlags |= 4;	
		seh = teb->except;

    		while(seh && seh != 0xffffffff) {
		rc = seh->fn(p,seh,0,0);
		switch(rc) {
		case ExceptionContinueExecution:
    			APISTR((LF_CONSOLE,"exceptioncontinue...\n")); 
			if(!(r.ExceptionFlags & 1))
				return;
    			APISTR((LF_CONSOLE,"exceptioncontinue not continuable...\n")); 
			debuggerbreak();

		case ExceptionContinueSearch:
			break;
		case ExceptionNestedException:
		default:
    			APISTR((LF_CONSOLE,"exception nested or default\n")); 
			debuggerbreak();
		}
	
		seh=seh->prev;
		teb->except = seh;
	}	

    	APISTR((LF_CONSOLE,"reached end of exception chain\n")); 
	debuggerbreak();
    }

    debuggerbreak();
    APISTR((LF_CONSOLE,"RaiseException: returns void\n"));
    return;
}

 

UINT WINAPI
RegisterWindowMessageW(LPCWSTR lpszMessage)
{
    UINT rc;
    int    len;
    char  *lpstr;
	
    len = lstrlenW(lpszMessage);

    lpstr = WinMalloc(len);
    WideCharToMultiByte(0,0,(LPCWSTR)lpszMessage,0,lpstr,len,0,0);
    APISTR((LF_APICALL,"RegisterWindowMessageW(LPCSTR=%s)\n", lpstr));
    rc = GlobalAddAtom(lpstr);
    APISTR((LF_APIRET,"RegisterWindowMessageW returns UINT %x\n", rc));
    WinFree(lpstr);
    return rc;
}
#endif

FARPROC WINAPI
MakeProcInstance(FARPROC lpproc, HINSTANCE hInstance)
{
    APISTR((LF_APICALL,"MakeProcInstance(FARPROC=%x,HINSTANCE=%x)\n", 
	lpproc,hInstance));
    APISTR((LF_APIRET,"MakeProcInstance: returns FARPROC %x\n",lpproc));
    return lpproc;	
}

void WINAPI
FreeProcInstance(FARPROC lpproc)
{
    APISTR((LF_APICALL,"FreeProcInstance(FARPROC=%x)\n", lpproc));
    APISTR((LF_APIRET,"FreeProcInstance: returns void\n"));
    return;
}

#include <sys/types.h>
#include <sys/wait.h>


VOID WINAPI
Sleep(DWORD sleeptime)
{
    APISTR((LF_APICALL, "Sleep(DWORD=%d)\n",sleeptime));

    osdll_time_sleep(sleeptime);

    APISTR((LF_APIRET, "Sleep: returns void\n"));
    return;
}

BOOL
CreateDataInstance(HINSTANCE hInst, HMODULE hModule, HTASK hTask)
{
    MEMORYINFO *lpMemory;

#ifdef TWIN_BIN32
    if (HIWORD(hInst)) {
	hInst = TWIN_InstanceFromModule32((void *) hInst);
    }
#endif
    if (!(lpMemory = GETHANDLEINFO(hInst)))
	return FALSE;

    lpMemory->hModule = hModule;
    lpMemory->hTask = hTask;

    RELEASEHANDLEINFO(lpMemory);
    return TRUE;
}


HMODULE
GetModuleFromInstance(HINSTANCE hInstance)
{
    MEMORYINFO *lpMemory;
    HANDLE hResult;

    if (hInstance == 0)
    {
        HTASK t;
	TASKINFO *taskinfo;
	HINSTANCE hInst;

        t = GetCurrentTask();
	taskinfo = GETTASKINFO(t);
	hInst = taskinfo->hInst;
	RELEASETASKINFO(taskinfo);
	if(hInst == 0) {
		return 0;
	}
        return GetModuleFromInstance(hInst); 
    }

#ifdef TWIN_BIN32
    if (HIWORD(hInstance)) {
	hInstance = TWIN_InstanceFromModule32((void *) hInstance);
    }
#endif
    if (!(lpMemory = GETHANDLEINFO(hInstance))) {
        MODULEINFO *lpModule = GETMODULEINFO(hInstance);
        BOOL bGotModule =  lpModule ? TRUE : FALSE;
	if(lpModule)
		RELEASEMODULEINFO(lpModule);

	if (!bGotModule)
	    return (HMODULE)0;
	else {
	    return (HMODULE)hInstance;
	}
    }
    hResult = lpMemory->hModule;
    RELEASEHANDLEINFO(lpMemory);
    return hResult;
}

HTASK
GetTaskFromInstance(HINSTANCE hInstance)
{
    LPMEMORYINFO lpMemory;
    HANDLE hTask;

    if (hInstance == 0)
	return GetCurrentTask();

#ifdef TWIN_BIN32
    if (HIWORD(hInstance)) {
	hInstance = TWIN_InstanceFromModule32((void *) hInstance);
    }
#endif
    if (!(lpMemory = GETHANDLEINFO(hInstance)))
	return (HTASK)0;

    hTask = lpMemory->hTask;
    RELEASEHANDLEINFO(lpMemory);
    return hTask;
}

HINSTANCE
GetInstanceFromModule(HMODULE hModule)
{
    LPMODULEINFO lpModuleInfo;
    HANDLE hInst;
    if (hModule == 0)
	return (HINSTANCE)0;

#ifdef TWIN_BIN32
    if (HIWORD(hModule)) {
	return TWIN_InstanceFromModule32((void *)hModule);
    }
#endif

    if (!(lpModuleInfo = GETMODULEINFO(hModule))) {
        LPHANDLEINFO lpHandleInfo = GETHANDLEINFO(hModule);
	BOOL bHandle = lpHandleInfo ? TRUE : FALSE;
	RELEASEHANDLEINFO(lpHandleInfo);
	if (!bHandle) {
	    return (HINSTANCE)NULL;
	}
	else
	    return (HINSTANCE)hModule;
    }
    
    hInst = lpModuleInfo->hInst;
    RELEASEMODULEINFO(lpModuleInfo);
    return hInst;
}


BOOL WINAPI
GetInputState(void)
{
	LPQUEUE    lpQueue;
	lpQueue = SystemQueue;	

	return (lpQueue->lpQueueNext)?TRUE:FALSE;
}

DWORD WINAPI
GetQueueStatus(UINT fuFlags)
{
    APISTR((LF_APISTUB,"GetQueueStatus(UINT=%x)\n",fuFlags));
    return 0L;
}

/* we should use the HardwareEnabled flag when adding mouse or
 * key messages to the system queue. Currently it is ignored.
 */

static BOOL SystemQueueEnabled = TRUE;

BOOL WINAPI
EnableHardwareInput(BOOL fEnableInput)
{
	BOOL	last;
	last    	   = SystemQueueEnabled;
	SystemQueueEnabled = fEnableInput;
	return last;
}

/*
 * QueueAddMsg:
 *	message == 0	insert message at front 
 *	message == -1	append to message queue
 *	message == msg  append after message
 */

LPQUEUE
QueueGetPtr(DWORD ThreadId,int sys)
{
        HANDLE     hqueue = 0;
	THDB      *tdb;
	PDB       *pdb;
	LPQUEUE   lpQueue = 0;
	
	if(ThreadId == 0) {
		if(sys ) {
			return SystemQueue;
		}
		ThreadId = GetCurrentThreadId();
	}

        tdb = (THDB *) THDB_TO_THREAD_ID(ThreadId); 
	if(tdb == 0)
	  	return 0;

	pdb = (PDB *) tdb->process;
	hqueue = tdb->teb.queue;	

    	if(hqueue) {
    		lpQueue = (LPQUEUE) pdb->handletable[hqueue];
	}

	return lpQueue;	
}

BOOL 
QueueAddMsg(DWORD ThreadId, WORD Insert, LPMSG lpMsg)
{
    LPQUEUE    lpQueue = 0;
    LPQUEUEMSG lpQueueMsg,lpQueuePtr;

    lpQueue = QueueGetPtr(ThreadId,1);

    if(lpQueue == 0) {
	return FALSE;
    }

    /* see if any free messages */
    if ((lpQueueMsg = lpQueue->lpQueueFree) == 0) {
	return FALSE;
    }

    /* copy the new message in */
    memcpy((LPSTR)&(lpQueueMsg->Message), (LPSTR)lpMsg, sizeof(MSG));
    if (lpQueueMsg->Message.message == WM_CONVERT)
	lpQueueMsg->wmc = *((WMCONVERT *)lpQueueMsg->Message.lParam);

    /* take message off the free list */
    lpQueue->lpQueueFree = lpQueueMsg->lpNextMsg;

    /* if no messages yet, then insert... */
    if(lpQueue->lpQueueNext == 0)
	Insert = 0;

    /* insert msg */
    if(Insert == 0) {
	/* set the next pointer of this message to the first message */
	lpQueueMsg->lpNextMsg = lpQueue->lpQueueNext;

	/* set the message queue to point to the new message */
	lpQueue->lpQueueNext = lpQueueMsg;
    } else {
	/* append msg to queue */
	/* find the message before this message */
	for(lpQueuePtr = lpQueue->lpQueueNext;
            lpQueuePtr->lpNextMsg;
	    lpQueuePtr = lpQueuePtr->lpNextMsg);
		
	/* set the last pointer to this one, and terminate this one */
	lpQueuePtr->lpNextMsg = lpQueueMsg;
	lpQueueMsg->lpNextMsg = 0;
    }
    return(TRUE);
}

BOOL 
QueueGetMsg(LPQUEUE lpQueue, LPMSG lpMsg, HWND hWnd,
		UINT wMsgFilterMin, UINT wMsgFilterMax, UINT wFlags)
{

#define	POOLSIZE	16
#define	POOLMASK	0x000f

    LPQUEUEMSG lpQueueMsg, lpQueuePtr, lpQueueNext;
    MSG msg;
    int	   nmsgs;
    int	   rc;
    static WMCONVERT ConvPool[POOLSIZE];
    static int nPoolIndex = 0;
    extern BOOL IsMouseOrKeyboardMsg(UINT);
    UINT cFlags;

    if(lpQueue == 0) {
	    lpQueue = SystemQueue;
    }

    /* search for a message meeting our filters... */
    for(lpQueueMsg = lpQueue->lpQueueNext,nmsgs=0,rc=0;
	lpQueueMsg;
	lpQueueMsg = lpQueueNext) {

	cFlags = wFlags;

	lpQueueNext = (LPQUEUEMSG)(lpQueueMsg->lpNextMsg);

	msg = lpQueueMsg->Message;

	if(msg.hwnd && !IsWindow(msg.hwnd)) {
		/* we have an event for a non-existent window */
		/* don't even try to pass it on, instead flush */
		/* it from the queue, and get another... */
		cFlags |= PM_REMOVE;
	} else {

	if ((lpQueue == SystemQueue) &&
	    (IsMouseOrKeyboardMsg(msg.message) == WM_MOUSEFIRST))
	    if (!TWIN_ForwardButtonEvent(&msg))
		continue;

	if (wMsgFilterMin) {
	    if (msg.message < wMsgFilterMin)
		continue;
	}

	if (wMsgFilterMax) {
	     if (msg.message > wMsgFilterMax)
		continue;
	}

	if (hWnd == HWND_BROADCAST) {
		if (msg.hwnd)
			continue;

	} else if (hWnd) {
		if (msg.hwnd &&
		    msg.hwnd != hWnd) {
	            if (!IsChild(hWnd, msg.hwnd))
			continue;
		}
	}
	}

	if(msg.hwnd && !IsWindow(msg.hwnd))
	    rc++;

	else {
	    if (msg.hwnd &&
		GetWindowTask(msg.hwnd) != GetCurrentTask())
	    {
		ReadyTask(GetWindowTask(msg.hwnd));
		continue;
	    }
	    
	    if (msg.hwnd &&
			HaveDisabledAncestor(msg.hwnd) && 
			lpQueue == SystemQueue) {
		if ((rc == 0) &&
			(msg.message != WM_MOUSEMOVE) &&
			(msg.message != WM_NCMOUSEMOVE))
		    MessageBeep(0);
		rc++;
	    }
	    else {
		rc = 0;
		memcpy((LPSTR)lpMsg,(LPSTR)&(msg), sizeof(MSG));
		if (lpMsg->message == WM_CONVERT) {
		    memcpy((LPSTR)&ConvPool[nPoolIndex],
			(LPSTR)&(lpQueueMsg->wmc),
			sizeof(WMCONVERT));
		    lpMsg->lParam = (LPARAM)&ConvPool[nPoolIndex];
		    lpMsg->wParam |= HSWMC_ALLOC;
		    nPoolIndex = (nPoolIndex+1) & POOLMASK;
		}
	    }
	}

	/* are we removing it? */
	if (cFlags & PM_REMOVE) {
		/* is it the first one in the queue */
		if (lpQueueMsg == lpQueue->lpQueueNext)
		    lpQueue->lpQueueNext = lpQueueMsg->lpNextMsg;
		else {
		    /* find the message before this one */
		    for(lpQueuePtr = lpQueue->lpQueueNext;
			    lpQueuePtr->lpNextMsg != lpQueueMsg;
			    lpQueuePtr = lpQueuePtr->lpNextMsg);

		/* set the next message of our previous to our next msg */
		    lpQueuePtr->lpNextMsg = lpQueueMsg->lpNextMsg;
		}

		/* add this message to the free list */
		lpQueueMsg->lpNextMsg = lpQueue->lpQueueFree;
		lpQueue->lpQueueFree = lpQueueMsg;
	}
	if(rc == 0) return(TRUE);
    }
    return FALSE;
}

void 
QueueSetFlags(DWORD ThreadId, WORD wFlags)
{
	LPQUEUE lpqueue = QueueGetPtr(ThreadId,0);

	if(lpqueue)
    		lpqueue->wQueueFlags |= wFlags;
}

void
QueueClearFlags(DWORD ThreadId, WORD wFlags)
{
	LPQUEUE lpqueue = QueueGetPtr(ThreadId,0);

	if(lpqueue)
    		lpqueue->wQueueFlags &= ~wFlags;
}

DWORD WINAPI
GetAppCompatFlags(HANDLE hTask)
{
	return 0;	/* Task module was built to target Windows 3.1 */
}

/* (WIN32) Kernel Error **************************************************** */

static DWORD dwLastError = 0;

DWORD	WINAPI
GetLastError(VOID)
{
	DWORD rc;
	APISTR((LF_APICALL, "GetLastError()\n"));

	rc = dwLastError;

	APISTR((LF_APIRET, "GetLastError: returns DWORD %x\n",rc));

	return rc;
}

VOID	WINAPI
SetLastError(DWORD dwError)
{
	APISTR((LF_APICALL, "SetLastError(DWORD=%x)\n",dwError));
	dwLastError = dwError;
	APISTR((LF_APIRET, "SetLastError: returns void\n"));
}

VOID	WINAPI
SetLastErrorEx(DWORD dwError, DWORD dwType)
{
	APISTR((LF_APICALL, "SetLastErrorEx(DWORD=%x,DWORD=%x)\n",
		dwError,dwType));

	dwLastError = dwError;

	APISTR((LF_APIRET, "SetLastErrorEx: returns void\n"));
}

void
ExitProcess(int id)
{
	HINSTANCE	hInst;
	HMODULE      hModule;
	MODULEINFO   *modinfo;
	//jmp_buf env;

	APISTR((LF_API,"ExitProcess(%d)\n",id));

	hInst = TWIN_GetInstanceFromTask( GetCurrentTask());
	hModule = GetModuleFromInstance(hInst);
	modinfo = GETMODULEINFO(hModule);

	modinfo->exit_code = id;
	//env = modinfo->env;

	//RELEASEMODULEINFO(modinfo);
	/* this is not protected, but I'm assuming */
	/* that we are just exiting, so it should  */
	/* be alright */
	longjmp(modinfo->env,1);
}

int WINAPI
GetKeyboardType(int fnKeyboard)
{
    	return (int)DRVCALL_KEYBOARD(PKH_GETTYPE,fnKeyboard,0,0);
}

extern WORD GetSystemDefaultLCID();

/***********************************************************************
 *           GetKeyboardLayout                  (USER32.250)
 */
HKL WINAPI GetKeyboardLayout(DWORD dwLayout)
{
        HKL layout;
	APISTR((LF_APISTUB,"GetKeyboardLayout(%d)\n",dwLayout));
        layout = (0xcafe<<16)|GetSystemDefaultLCID();
        return layout;
}


/**********************************************************************
 *           ActivateKeyboardLayout      (USER32.1)
 */
HKL WINAPI ActivateKeyboardLayout(HKL hLayout, UINT flags)
{
    	APISTR((LF_APISTUB, "ActivateKeyboardLayout(%d, %d)\n", 
		hLayout, flags));

    	return 0;
}


/***********************************************************************
 *           GetKeyboardLayoutList              (USER32.251)
 * FIXME
 */
UINT WINAPI GetKeyboardLayoutList(int nBuff,HKL *layouts)
{
        APISTR((LF_APISTUB,"GetKeyboardLayoutList(%d,%p)\n",nBuff,layouts));
        if (layouts)
                layouts[0] = GetKeyboardLayout(0);
        return 1;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: Kernel.c,v $
** Revision 1.14  2000/01/21 20:25:49  robf
** first pass at raiseexception
**
** Revision 1.13  1999/12/21 08:34:18  robf
** only free module if we got one
**
** Revision 1.12  1999/12/13 16:47:48  jwhite
** Minor comment changes.
**
** Revision 1.11  1999/12/08 07:52:22  robf
** updates for binary emulation
**
** Revision 1.10  1999/11/29 18:19:44  robf
** logging apis
**
** Revision 1.9  1999/11/29 06:25:21  robf
** fixes for RaiseException()
**
** Revision 1.8  1999/10/25 16:53:23  jsheets
** Cleaned up lots of missing CALL/RET logging pairs
**
** Revision 1.7  1999/09/28 19:32:52  jsheets
** Added missing WINAPI's
**
** Revision 1.6  1999/07/09 21:55:41  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
