/*--------------------------------------------------------------------------
**      KrnTask.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/KrnTask.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/KrnTask.c,v 1.4 1999/11/29 18:29:06 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "kerndef.h"
#include "Resources.h"
#include "Module.h"
#include "Kernel.h"
#include "Driver.h"
#include "Log.h"
#include "WinConfig.h"
#include "KrnTask.h"
#include "LoadDLL.h"
#include "WindowCreate.h"
#include "Classes.h"
#include "Messages.h"
#include "Segment.h"
#include "DPMI.h"
#include "osdll.h"

ENV       *envp_global;

static  LPOBJHEAD lpTaskHead = (LPOBJHEAD)NULL;

extern LPQUEUE    SystemQueue;
extern BOOL TWIN_InDriverWait;

extern LPQUEUE AllocQueue(int);
extern void TWIN_ReadyAllTasks(void);

#define	DEFAULT_QUEUE_SIZE	8
#define TASK_STACK_SIZE		0x100000

HINSTANCE FindPreviousInstance(HMODULE hModule, HTASK hTask);
THDB * TWIN_AllocTHDB(HTASK,DWORD , LPTHREAD_START_ROUTINE ,LPVOID );

ENV       *
GetEnvpGlobal()
{
	return envp_global;
}

LPOBJHEAD 
TWIN_GetTaskHead()
{
	return lpTaskHead;
}

UINT
GetNumTasks(void)
{
    LPTASKINFO lpTask;
    int nTasks;
    
    nTasks = 0;

    for (lpTask = (LPTASKINFO) lpTaskHead; 
	 lpTask;
	 lpTask = (LPTASKINFO) lpTask->ObjHead.lpObjNext)
    {
	nTasks++;
    }

    return nTasks;
}

HTASK WINAPI
GetCurrentTask(void)
{
    THDB *pthread;
    PDB  *pdb;
    HTASK hTask = 0;

    APISTR((LF_APICALL,"GetCurrentTask()\n"));
    pthread = osdll_thread_gettdb();
    if(pthread) {
    	pdb = (PDB *) pthread->process;
    	hTask = pdb->hTask;
    }

    APISTR((LF_APIRET,"GetCurrentTask: returns HTASK %x\n",hTask));
    return hTask;
}

BOOL WINAPI
IsTask(HTASK hTask)
{
    LPTASKINFO lpti;
    BOOL bR;
    
    APISTR((LF_APICALL,"IsTask(HTASK=%x)\n",hTask));
    lpti = GETTASKINFO(hTask);
    bR = lpti ? TRUE : FALSE;
    if (bR)
      RELEASETASKINFO(lpti);
    APISTR((LF_APIRET,"IsTask: returns BOOL %d\n",bR));
    return bR;
}

BOOL
TWIN_IsLiveTask(HTASK hTask)
{
    LPTASKINFO lpTask = GETTASKINFO(hTask);

    BOOL bR = (lpTask && !(lpTask->wStateFlags & TASK_DELETE_PENDING))
       ? TRUE : FALSE;

    if (lpTask)
      RELEASETASKINFO(lpTask);
    
    return bR;
}

BOOL WINAPI
ActivateTask(HTASK hTask)
{
#ifdef	LATER
    /* Task activation stuff */
#endif
    return(TRUE);
}

void
InitializeNativeTask(LPSTR lpszCmdLine)
{
    LPTASKINFO lpTaskTemp;
    THDB *initthread;
    HTASK hTask;

    /* Create task structure */
    if (!(lpTaskTemp = CREATEHTASK(hTask))) {
	return;
    }

    if (SystemQueue == 0)
	SystemQueue = AllocQueue(20*DEFAULT_QUEUE_SIZE);


    initthread = TWIN_AllocTHDB(hTask,0,0,lpszCmdLine);
    osdll_thread_init(initthread);

    lpTaskTemp->wStateFlags |= TASK_READY;

    lpTaskTemp->ObjHead.lpObjNext = (LPOBJHEAD)lpTaskHead;

    lpTaskHead = (LPOBJHEAD)lpTaskTemp;

    RELEASETASKINFO(lpTaskTemp);
}

HTASK
CreateTask()
{
#if 0
    LPTASKINFO lpTaskTemp;
    HTASK hTask;

    /* Create task structure */
    if (!(lpTaskTemp = CREATEHTASK(hTask))) 
    {
	return 0;
    }

    // create a process?  and threads?
    // and queue for that thread...

    lpTaskTemp->hInst = 0;
    lpTaskTemp->wTDBSelector = 0;
    lpTaskTemp->lpCmdLine = NULL;
    lpTaskTemp->wStateFlags = 0;
    lpTaskTemp->GlobalEnvp = (ENV*)WinMalloc(sizeof(*lpTaskTemp->GlobalEnvp));
    memset((LPSTR)lpTaskTemp->GlobalEnvp, 0, sizeof(*lpTaskTemp->GlobalEnvp));

    lpTaskTemp->ObjHead.lpObjNext = (LPOBJHEAD)lpTaskHead;
    lpTaskHead = (LPOBJHEAD)lpTaskTemp;

    RELEASETASKINFO(hpTaskTemp);
    return hTask;
#endif
    APISTR((LF_CONSOLE,"CreateTask() failed\n"));
    return 0;
}

void
ExitTask(ENV *envp)
{

	while(InternalYield())
		;

	longjmp(envp->jump_buffer,1);

	/* will never reach here */
	exit(0);
}

BOOL
TWIN_FreeTaskWindow(HWND hwnd, LPARAM lParam)
{
    debuggerbreak();
    DestroyWindow(hwnd);
    return TRUE;
}

void
FreeTask(HTASK hTask)
{
    LPTASKINFO lpTaskFree;
    LPTASKINFO lpTaskPrev;
    HMODULE hModule;

    /* Never free the current task */
    if (hTask == GetCurrentTask())
    {
	TWIN_ReadyAllTasks();
	return;
    }
        
    lpTaskFree = GETTASKINFO(hTask);
    if (!lpTaskFree)
	return;

    /* Destroy all windows owned by this task... */
    EnumTaskWindows(hTask, (WNDENUMPROC)TWIN_FreeTaskWindow, 0);

    /* Free the PSP and TDB for this task */

    /* Free envp chain for this task */

    /* Free all segments used by this task */

    /* Is this the last instance of this module? */
    hModule = GetModuleFromInstance(lpTaskFree->hInst);
    if (!FindPreviousInstance(hModule, hTask))
    {
	/* Unregister all classes if this is last instance of module */
	TWIN_UnregisterModuleClasses(hModule);

	/* If the last instance of a module, then free the module */
    }

    if (lpTaskHead == (LPOBJHEAD) lpTaskFree)
	lpTaskHead = (LPOBJHEAD) lpTaskFree->ObjHead.lpObjNext;
    else
    {
	for (lpTaskPrev = (LPTASKINFO) lpTaskHead; 
	     lpTaskPrev && 
	     (LPTASKINFO) lpTaskPrev->ObjHead.lpObjNext != lpTaskFree; 
	     lpTaskPrev = (LPTASKINFO) lpTaskPrev->ObjHead.lpObjNext)
	{
	    /* searching */
	}

	if (lpTaskPrev)
	    lpTaskPrev->ObjHead.lpObjNext = lpTaskFree->ObjHead.lpObjNext;
    }

    RELEASETASKINFO(lpTaskFree);
    FREEHTASK(hTask);
}

void
ReadyTask(HTASK hTask)
{
    LPTASKINFO lpTask;
    
    if(hTask) {
	    lpTask = GETTASKINFO(hTask);
	    if (!lpTask)
		return;

	    lpTask->wStateFlags |= TASK_READY;
	    RELEASETASKINFO(lpTask);
    }
}    

void
TWIN_ReadyAllTasks()
{
    LPTASKINFO lpTask;
    LPTASKINFO lpNext;
    
    for (lpTask = (LPTASKINFO) lpTaskHead; 
	 lpTask;
	 lpTask = lpNext)
    {
	lpNext = (LPTASKINFO) lpTask->ObjHead.lpObjNext;

	lpTask->wStateFlags |= TASK_READY;
    }
}

void
DeleteTask(HTASK hTask)
{
    LPTASKINFO lpTask;

    if (!hTask)
	hTask = GetCurrentTask();

    lpTask = GETTASKINFO(hTask);
    if (!lpTask)
	return;

    lpTask->wStateFlags |= TASK_DELETE_PENDING;
    lpTask->wStateFlags &= ~TASK_READY;

    /* If we are deleting ourselves, then we don't ever want to return    */
    /* from this function call.  So, we just sit in a loop yielding the   */
    /* processor to other tasks.  Calling Yield() once should do it, but  */
    /* it is very important that even a bug in Yield() doesn't let us     */
    /* return.                                                            */
    if (hTask == GetCurrentTask())
    {
	while (TWIN_SendMessagePendingForTask(hTask))
	    ReplyMessage(0);
	
	while (1)
	{
	    InternalYield();
	    lpTask->wStateFlags |= TASK_DELETE_PENDING;
	    lpTask->wStateFlags &= ~TASK_READY;
	    if(GetNumTasks() <= 1)
		break;
	}
    }
    RELEASETASKINFO(lpTask);
}

HTASK
ScheduleTask()
{
    LPQUEUEMSG lpQueueMsg;
    LPTASKINFO lpTask;
    LPTASKINFO lpNext;
    LPTASKINFO lpReadyTask = NULL;
    BOOL bBeforeCurrent = TRUE;
    int iLiveTasks = 0;

    {
	/*
	 * Before scheduling, process all events on the driver message
	 * queue.  This will assure us that all events are in the standard
	 * message queues before we try to decide which tasks are ready to
	 * run.
	 */
	TWIN_InDriverWait = TRUE;
	DriverWaitEvent(TRUE);
	TWIN_InDriverWait = FALSE;

	/*
	 * Examine the system message queue and ready any tasks that have
	 * messages waiting in that queue.
	 */
	for (lpQueueMsg = SystemQueue->lpQueueNext;
	     lpQueueMsg;
	     lpQueueMsg = (LPQUEUEMSG) lpQueueMsg->lpNextMsg)
	{
	    HWND hwnd = lpQueueMsg->Message.hwnd;
	    
	    if (hwnd)
		ReadyTask(GetWindowTask(hwnd));
	}
    
	/* We have chosen a very simple scheduling algorithm.  It is  */
	/* a round-robin scheme with all tasks having equal priority. */
	/* We look at three things to determine whether a process is  */
	/* ready to run:					      */
	/*	1. Is the ready flag set?			      */
	/*	2. Do we need a paint or other "flagged" message?     */
	/*	3. Are there any messages in the queue?		      */
	   
	for (lpTask = (LPTASKINFO) lpTaskHead; 
	     lpTask;
	     lpTask = lpNext)
	{
	    lpNext = (LPTASKINFO) lpTask->ObjHead.lpObjNext;
	    
	    /* Since we are looking through the tasks anyway, let's   */
	    /* delete processes that are pending deletion.            */
	    if (lpTask->wStateFlags & TASK_DELETE_PENDING)
	    {
		FreeTask(lpTask->ObjHead.hObj);
		continue;
	    }

	    /* This task is still alive.  Update the count.           */
	    iLiveTasks++;

	    /* Apply the test to see if process is ready.             */
	    if ((!bBeforeCurrent || !lpReadyTask) &&
		((lpTask->wStateFlags & TASK_READY)))
	    {
		lpReadyTask = lpTask;
		if (!bBeforeCurrent)
		    break;
	    }

	    if (lpTask->ObjHead.hObj == GetCurrentTask())
		bBeforeCurrent = FALSE;
	}

	if (lpReadyTask)
	    return lpReadyTask->ObjHead.hObj;

	else if (!iLiveTasks) {
	    ExitWindows(0,0);
	}
    }
    
    return GetCurrentTask();
}

void WINAPI
Yield(void)
{
    ReadyTask(GetCurrentTask());
    InternalYield();
}

void    WINAPI 
DirectedYield(HTASK hTask)
{
#ifndef TWIN32
    ReadyTask(GetCurrentTask());
    InternalDirectedYield(hTask);
#endif
}

BOOL InternalYield()
{
    HTASK hTask;
    
    hTask = ScheduleTask();
    if (hTask == GetCurrentTask())
	return FALSE;
    
    InternalDirectedYield(hTask);
    return TRUE;
}    

void
InternalDirectedYield(HTASK hTask)
{
    LPTASKINFO lpNewTask;
    LPTASKINFO lpOldTask;

    if (hTask == GetCurrentTask())
	return;
    
    lpNewTask = GETTASKINFO(hTask);
    lpOldTask = GETTASKINFO(GetCurrentTask());
    if (!lpNewTask || !lpOldTask) {
        if (lpNewTask) {
	  RELEASETASKINFO(lpNewTask);
	}
	if (lpOldTask) {
	  RELEASETASKINFO(lpOldTask);
	}
	return;
    }

    /* clear the ready flag on the target task */
    lpNewTask->wStateFlags &= ~TASK_READY;

    /* save global variables that are task specific */
    lpOldTask->GlobalEnvp = envp_global;

    /* set global variables from saved versions in target task */
    envp_global = lpNewTask->GlobalEnvp;
    
    RELEASETASKINFO(lpNewTask);
    RELEASETASKINFO(lpOldTask);

    /* Are there any SendMessages to process? */
    TWIN_ReceiveMessage(FALSE);
}

HINSTANCE FindPreviousInstance(HMODULE hModule, HTASK hTask)
{
    LPTASKINFO lpTask;
    LPTASKINFO lpNext;
    HMODULE hTaskModule;
    
    for (lpTask = (LPTASKINFO) lpTaskHead; 
	 lpTask;
	 lpTask = lpNext)
    {
	lpNext = (LPTASKINFO) lpTask->ObjHead.lpObjNext;
	
	if (lpTask->ObjHead.hObj != hTask && lpTask->hInst)
	{
	    hTaskModule = GetModuleFromInstance(lpTask->hInst);
	    if (hTaskModule && hTaskModule == hModule)
		return lpTask->hInst;
	}
    }

    return 0;
}

HINSTANCE
TWIN_GetInstanceFromTask(HTASK hTask)
{
    LPTASKINFO lpTask;
    HINSTANCE hResult;

    if (hTask == 0)
	hTask = GetCurrentTask();

    if (NULL == (lpTask = GETTASKINFO(hTask)))
	return (HINSTANCE)0;
    else {
        hResult = lpTask->hInst;
	RELEASETASKINFO(lpTask);
	return hResult;
    }
}

void
TWIN_LocalExpandStackFix(UINT uSegment, LPBYTE lpSegment, LPBYTE lpOldSegment)
{
    ENV *lpEnv;
    LPTASKINFO lpTask;
    LPTASKINFO lpNext;
    
    for (lpTask = (LPTASKINFO) lpTaskHead; lpTask; lpTask = lpNext)
    {
	lpNext = (LPTASKINFO) lpTask->ObjHead.lpObjNext;
	
	if (lpTask->ObjHead.hObj != GetCurrentTask())
	    lpEnv = lpTask->GlobalEnvp;
	else
	    lpEnv = envp_global;
	
	for( ; lpEnv; lpEnv = (ENV *) lpEnv->prev_env) 
	{

	    if (lpEnv->reg.ss == uSegment) 
	    {
		lpEnv->reg.sp = 
		    (REGISTER) (lpSegment + (WORD) ((DWORD) lpEnv->reg.sp - 
						    (DWORD) lpOldSegment));
		lpEnv->reg.bp = 
		    (REGISTER) (lpSegment + (WORD) ((DWORD) lpEnv->reg.bp - 
						    (DWORD) lpOldSegment));
	    }
	}
    }
}

LPINITT
InitTask(void)
{
    static INITT InitStruct;
    LPTASKINFO lpTask;
    extern LPINITT WindowsInitTask(INITT *);
    
    lpTask = GETTASKINFO(GetCurrentTask());

    InitStruct.wStackBottom = lpTask->uiAutodataSize + lpTask->uiStackSize;
    InitStruct.wStackTop = lpTask->uiAutodataSize;
    InitStruct.wHeapSize = lpTask->uiHeapSize;
    RELEASETASKINFO(lpTask);
    return (LPINITT) WindowsInitTask(&InitStruct);
}

LPINITT
WindowsInitTask(LPINITT InitStruct)
{
    TASKINFO *lpTaskInfo;
    HANDLE hTask;

    hTask = GetCurrentTask();
    if((lpTaskInfo = (TASKINFO *)GETTASKINFO(hTask))) {
	InitStruct->lpCmdLine = lpTaskInfo->lpCmdLine;
	InitStruct->hInstance = lpTaskInfo->hInst;
    }
    InitStruct->hPrevInstance = 0;
    InitStruct->nCmdShow = SW_SHOWNORMAL;
    RELEASETASKINFO(lpTaskInfo);
    return InitStruct;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: KrnTask.c,v $
** Revision 1.4  1999/11/29 18:29:06  robf
** modify destroywindow call
**
** Revision 1.3  1999/07/09 21:55:41  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
