/*--------------------------------------------------------------------------
**      Strings32.c                 Twine           From:  Wine/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file is a direct copy of the equivalent file in the
**  Wine project (www.winehq.com).  The Wine license permits
**  copying of source code, so long as all copyright and credit
**  information is preserved.  Any and all copyrights originally
**  in the Wine source code file are preserved exactly as they were
**  when this copy was made from the Wine source tree.
**  (see the 'Original Copyright(s)', below).
**  An exacty copy of the Wine license is contained in WINE-LICENSE.
**  
**      For a more general description of the Wine license, and
**  for a full list of the authors who deserve credit for this
**  code, please see www.winehq.com, or WINE-AUTHORS.
**  
**      However, because this file is released with Twine, we
**  are obliged to also release it under the LGPL license.
**  In short, you may make a copy or republish this code so long as
**  all of your changes to source code are again made publicly available.
**  Further, absolutely no warranty of any kind is given or implied.
**  Refer to the LGPL description at www.fsf.org or the file LICENSE 
**  for further details.
**  
**  
**      ==========================================================
**      IF YOU MAKE CHANGES TO THIS CODE, PLEASE REFLECT
**      THOSE CHANGES IN THE ORIGINAL WINE CODE, AT WWW.WINEHQ.COM
**      ==========================================================
**
**      
**------------------------ < Original Copyright(s) > -----------------------
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/Strings32.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/Strings32.c,v 1.10 1999/12/20 21:47:57 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Log.h"
#include "strings.h"
#include "Strings32.h"
#include "ctype.h"
#include "Log.h"

char *
WinStrdupW(LPCWSTR lpwstr)
{
	int    len;
	char  *lpstr;
	
	len = lstrlenW(lpwstr);

	/* count is # shorts in original... */
	/* add room for terminator */
	lpstr = WinMalloc(len+1);
	if(lpstr)
		WideCharToMultiByte(0,0,(LPCWSTR)lpwstr,0,lpstr,len,0,0);
	
	return lpstr;
}
int 
MultiByteToWideChar(UINT page, DWORD flags, const char *src, int srclen,
                               LPWSTR dst, int dstlen)
{
    /* if srclen == -1, len is strlen of src */
    if (srclen == -1)
         srclen = lstrlen(src);

    /* if !destination, return size needed for copy */
    if (!dst)
         return srclen*2;

    /* shorthand copy, multibyte to wide char */
    lstrcpynAtoW(dst,src,srclen); 
    return srclen*2;
}

int WINAPI
WideCharToMultiByte(UINT codepage,DWORD dwFlags,LPCWSTR wide,int widelen,LPSTR multi,int multilen,LPCSTR defchar,LPBOOL fuseddefault)
{
	short *cp;
	char  *buffer = multi;
	int   width   = widelen;
	int   size    = 0;
	
	APISTR((LF_APICALL,"WideCharToMultiByte(UINT=%x,DWORD=%x,LPCWSTR=%p,int,LPSTR=%p,int,LPCSTR=%p,LPBOOL=%p)\n",
		codepage,dwFlags,wide,widelen,multi,multilen,defchar,fuseddefault));

	if(fuseddefault)
		*fuseddefault = 0;

	if(width == 0)
		width = lstrlenW((short *)wide);

	/* width is count...,not byte length  */
	for(cp = (short *) wide;width && *cp;width--) {
		if(*cp < 256) {
		       if(multi && multilen) {
				*buffer = (*cp & 0xff);
				multilen--;
			}
		} else {
		       if(multi && multilen) {
				*buffer = defchar?*defchar:'?';
				multilen--;
			}
		        if(fuseddefault)
				*fuseddefault = 1;
		}

		cp++;
		buffer++;
		size++;
	}

	if(multi && multilen)
		*buffer = 0;

	// this is wrong...should NOT be bigger than orginal size?
	// don't count more than we actually do...
	//size += 1;	

	APISTR((LF_APIRET,"WideCharToMultiByte(...) returns (%d) %d\n",
		widelen,size));
	return size;
}

LPWSTR 
lstrcatW( short * dst, short * src )
{
    short * p = dst;
    while (*p) p++;
    while ((*p++ = *src++));
    return dst;
}

int 
lstrcmpW( short * str1, short * str2 )
{
    if (str1 && !str2) {
	    while (*str1 && (*str1 == *str2)) { 
		str1++; str2++; 
	    }
	    return (int)(*str1 - *str2);
    }
    return 0;
}

int 
lstrcmpiW( short * str1, short * str2 )
{
    int res;

    if (str1 && str2) {
	    while (*str1) {
		if ((res = toupper(*str1) - toupper(*str2)) != 0) 
		   return res;

		str1++;
		str2++;
	    }
	    return toupper(*str1) - toupper(*str2);
    }
    return 0;
}

int 
lstrncmpiW( short * str1, short * str2, int n )
{
    int res;

    if (!n) 
	return 0;

    while ((--n > 0) && *str1) {
        if ((res = toupper(*str1) - toupper(*str2)) != 0) 
	   return res;

        str1++;
        str2++;
    }

    return toupper(*str1) - toupper(*str2);
}

LPWSTR 
lstrcpyW( short * dst, short * src )
{
    short * p = dst;

    while ((*p++ = *src++));

    return dst;
}

LPWSTR 
lstrcpynW( short * dst, short * src, int n )
{
    short * p = dst;

    for(;;) {
	if(n == 0)
		break;
	if(*src == 0) 
		break;
	*p = *src;
	p++;
	src++;
	n--;
    }
    *p = 0;

    return dst;
}

LPSTR
lstrcpyA(LPSTR a,LPCSTR b)
{
	return lstrcpy(a,b);
}
LPSTR
lstrcpynA(LPSTR a,LPCSTR b,int n)
{
	return lstrcpyn(a,b,n);
}
int
lstrcmpA(LPCSTR a,LPCSTR b)
{
	return lstrcmp(a,b);
}
int
lstrcmpiA(LPCSTR a,LPCSTR b)
{
	return lstrcmpi(a,b);
}
LPSTR
lstrcatA(LPSTR a,LPCSTR b)
{
	return lstrcat(a,b);
}
int
lstrlenA( LPCSTR str)
{
	return lstrlen(str);
}

/* return count of characters */
int 
lstrlenW( LPCWSTR str )
{
    int len = 0;

    if (!str) 
	return 0;

    while (*str++) 
	len++;

    return len;
}

#if 0
size_t WINAPI 
wcslen(LPCWSTR t)
{
	return lstrlenW(t);
}
#endif

LPWSTR lstrcpyAtoW( LPWSTR dst, LPCSTR src )
{
    LPWSTR p = dst;

    if(dst && src)
    	while ((*p++ = (WCHAR)(unsigned char)*src++));
    return dst;
}


LPSTR lstrcpyWtoA( LPSTR dst, LPCWSTR src )
{
    register LPSTR p = dst;
    if(dst && src) 
    	while ((*p++ = (CHAR)*src++));
    return dst;
}

LPWSTR lstrcpynAtoW( LPWSTR dst, LPCSTR src, int n )
{
    LPWSTR p = dst;
    if(dst && src && n) {
    	while ((n-- > 1) && *src) 
		*p++ = (WCHAR)(unsigned char)*src++;
    }
    if(p)
      *p = 0;
    return dst;
}

LPSTR lstrcpynWtoA( LPSTR dst, LPCWSTR src, int n )
{
    LPSTR p = dst;
    if(dst && src && n) {
	    while ((n-- > 1) && *src) 
		*p++ = (CHAR)*src++;
    }
    if(p)
      *p = 0;

    return dst;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: Strings32.c,v $
** Revision 1.10  1999/12/20 21:47:57  robf
** WideCharToMultibyte copied too many bytes, vc6 runtime checks overflow
**
** Revision 1.9  1999/11/29 22:07:33  robf
** logging for widechartomultibyte
**
** Revision 1.7  1999/07/09 21:55:44  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
