/*--------------------------------------------------------------------------
**      Time.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/Time.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/Time.c,v 1.2 1999/07/09 21:55:45 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Resources.h"
#include "Module.h"
#include "osdll.h"

/* FileTimeToSystemTime: Convert a FILETIME to a SYSTEMTIME */
BOOL WINAPI
FileTimeToSystemTime(LPFILETIME lpft, LPSYSTEMTIME lpst)
{
    return osdll_time_ftos(lpft,lpst);
}

/* SystemTimetoFileTime: Convert a SYSTEMTIME to a FILETIME */
BOOL WINAPI
SystemTimeToFileTime(LPSYSTEMTIME lpst, LPFILETIME lpft)
{
    return osdll_time_stof(lpst,lpft);
}
  
BOOL WINAPI
FileTimeToLocalFileTime(const FILETIME *lpft, LPFILETIME lplft)
{
    return osdll_time_ftolf(lpft,lplft);
}

BOOL WINAPI
LocalFileTimeToFileTime(LPFILETIME lplft, LPFILETIME lpft)
{
    return osdll_time_lftof(lplft,lpft);
}

void WINAPI
GetSystemTime(LPSYSTEMTIME lpst)
{
    osdll_time_systime(0,lpst);
}

void WINAPI
GetLocalTime(LPSYSTEMTIME lpst)
{
    osdll_time_localtime(0,lpst);
}

BOOL WINAPI
SetSystemTime(LPSYSTEMTIME lpst)
{
    return osdll_time_systime(1,lpst);
}

BOOL WINAPI
SetLocalTime(LPSYSTEMTIME lpst)
{
    return osdll_time_localtime(1,lpst);
}

BOOL WINAPI
SetTimeZoneInformation(LPTIME_ZONE_INFORMATION lptzi)
{
    return osdll_time_timezone(1,lptzi);
}

#ifdef TWIN_BIN32

#define TIME_ZONE_ID_UNKNOWN    0
#define TIME_ZONE_ID_STANDARD   1
#define TIME_ZONE_ID_DAYLIGHT   2

BOOL WINAPI
GetTimeZoneInformation(LPTIME_ZONE_INFORMATION tzinfo)
{
    time_t gmt, lt;

    gmt = time(NULL);
    lt = mktime(localtime(&gmt));

    tzinfo->Bias = (gmt - lt) / 60;
    tzinfo->StandardBias = 0;
    tzinfo->DaylightBias = -60;

    return TIME_ZONE_ID_UNKNOWN;
}
#endif

BOOL WINAPI
DosDateTimeToFileTime(WORD wDOSDate, WORD wDOSTime, LPFILETIME lpft)
{
    return osdll_time_dostofile(wDOSDate,wDOSTime,lpft);
}

BOOL WINAPI
FileTimeToDosDateTime(LPFILETIME lpft, LPWORD lpwDOSDate, LPWORD lpwDOSTime)
{
    return osdll_time_filetodos(lpft,lpwDOSDate,lpwDOSTime);
}


LONG WINAPI
CompareFileTime(LPFILETIME lpft1, LPFILETIME lpft2)
{
  if (lpft1->dwHighDateTime > lpft2->dwHighDateTime)
    return 1;
  if (lpft1->dwHighDateTime < lpft2->dwHighDateTime)
    return -1;
  if (lpft1->dwLowDateTime > lpft2->dwLowDateTime)
    return 1;
  if (lpft1->dwLowDateTime < lpft2->dwLowDateTime)
    return -1;
  return 0;
}

/* We don't support system time adjustment, but the user can always turn it off... */
BOOL WINAPI
SetSystemTimeAdjustment(DWORD dwTimeAdjustment, BOOL bTimeAdjustmentDisabled)
{
  if (bTimeAdjustmentDisabled)
    return TRUE;
  return FALSE;
}

BOOL WINAPI
GetSystemTimeAdjustment(DWORD *lpTimeAdjustment, DWORD *lpTimeIncrement,
			LPBOOL lpTimeAdjustmentDisabled)
{
  *lpTimeAdjustmentDisabled = TRUE;
  return TRUE;
}

void WINAPI
GetSystemTimeAsFileTime(LPFILETIME lpft)
{
  SYSTEMTIME st;

  GetSystemTime(&st);
  SystemTimeToFileTime(&st, lpft);
}

BOOL WINAPI
SystemTimeToTzSpecificLocalTime(LPTIME_ZONE_INFORMATION lpTimeZoneInformation,
				     LPSYSTEMTIME lpUCT, LPSYSTEMTIME lpst)
{
    return osdll_time_settime(lpTimeZoneInformation, lpUCT, lpst);
}

/*------------------------- < Full Revision History > ----------------------
** $Log: Time.c,v $
** Revision 1.2  1999/07/09 21:55:45  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
