/*--------------------------------------------------------------------------
**      UsrCursor.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/UsrCursor.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/UsrCursor.c,v 1.6 1999/12/09 12:00:50 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Log.h"
#include "WinDefs.h"
#include "GdiObjects.h"
#include "Module.h"
#include "Resources.h"
#include "Driver.h"

typedef struct tagINTERNALCURINFO {
    HCURSOR hCursor;   /* hCursor of windows that have no cursor */
			/* or temporary cursor to be overridden   */
    HWND   hWnd;	/* top-level frame associated with cursor */
    LPVOID lpDrvData;   /* cursor drv data	*/
    int    nCount;	/* display count;	*/
#ifdef LATER
			/* if (nCount < 0), cursor is not visible */
#endif
} INTERNALCURINFO;

void TWIN_SetCursorWindow(HWND);
HWND TWIN_GetCursorWindow(void);

static INTERNALCURINFO InternalCursor;
static HCURSOR hNullCursor;

static BOOL TWIN_DefineCursorWindow();
    
HCURSOR WINAPI
SetCursor(HCURSOR hCursor)
{
    HWND hWnd;
    HCURSOR hLast;
    LPVOID lpWinData;
    LPCURSORINFO lpCursorInfo;
    LRESULT ret;

    APISTR((LF_APICALL,"SetCursor(HCURSOR=%x)\n",hCursor));

    hLast = InternalCursor.hCursor;
    if (!TWIN_DefineCursorWindow() && (hCursor == hLast)) {
        APISTR((LF_APIRET,"SetCursor: returns HCURSOR %x\n",hCursor));
	return hCursor;
    }

    if (hCursor == 0) {
	ret = DRVCALL_CURSORS(PCH_HIDECURSOR,0L,0L,0L);
	if (ret) {
	    InternalCursor.hCursor = 0;
	    InternalCursor.lpDrvData = 0;
	    InternalCursor.nCount = -1;
            APISTR((LF_APIRET,"SetCursor: returns HCURSOR %x\n",hLast));
	    return hLast;
	}
	else  	/* driver couldn't do it */
	    if (!hNullCursor)
		hNullCursor = LoadCursor(0,"NULLCURSOR");
   }

   if (!(lpCursorInfo = (hCursor)?GETCURSORINFO(hCursor):
				GETCURSORINFO(hNullCursor))) {
        APISTR((LF_APIRET,"SetCursor: returns HCURSOR %x\n",0));
	return 0;
    }

    InternalCursor.hCursor = hCursor;
    InternalCursor.lpDrvData = lpCursorInfo->lpCurData;
    InternalCursor.nCount = 0;
  
    if((hWnd = InternalCursor.hWnd) && IsWindow(hWnd))
	lpWinData = (LPVOID)WIN_GETDRVDATA(InternalCursor.hWnd);
    else
	lpWinData = NULL;

    DRVCALL_CURSORS(PCH_SETCURSOR,InternalCursor.lpDrvData,lpWinData,0L);
    if (hLast == 0)
	DRVCALL_CURSORS(PCH_SHOWCURSOR,0L,0L,0L);

    RELEASECURSORINFO(lpCursorInfo);

    APISTR((LF_APIRET,"SetCursor: returns HCURSOR %x\n",hLast));
    return hLast;
}

int WINAPI
ShowCursor(BOOL fShow)
{
    HWND   hWnd;
    LPVOID lpWinData;
    LPCURSORINFO lpCursorInfo;

    APISTR((LF_APICALL,"ShowCursor: %x\n",fShow));

    if (fShow) {
	if (++(InternalCursor.nCount) == 0)
	    if (!(DRVCALL_CURSORS(PCH_SHOWCURSOR,0L,0L,0L)) &&
				InternalCursor.hCursor) {
		TWIN_DefineCursorWindow();
		hWnd = InternalCursor.hWnd;
		if(IsWindow(hWnd))
			lpWinData = (LPVOID)WIN_GETDRVDATA(hWnd);
		else	
			lpWinData = NULL;
		DRVCALL_CURSORS(PCH_SETCURSOR,InternalCursor.lpDrvData,
					lpWinData,0L);
	    }
    }
    else {
	if (--(InternalCursor.nCount) == -1) {
	    if (!DRVCALL_CURSORS(PCH_HIDECURSOR,0L,0L,0L)) {
		if (!hNullCursor)
		    hNullCursor = LoadCursor(0,"NULLCURSOR");
		lpCursorInfo = GETCURSORINFO(hNullCursor);
		TWIN_DefineCursorWindow();
		hWnd = InternalCursor.hWnd;
		if(IsWindow(hWnd))
			lpWinData = (LPVOID)WIN_GETDRVDATA(hWnd);
		else	
			lpWinData = NULL;
		DRVCALL_CURSORS(PCH_SETCURSOR,lpCursorInfo->lpCurData,
					lpWinData,0L);
		RELEASECURSORINFO(lpCursorInfo);
	    }
	}
    }
	    
    return InternalCursor.nCount;
}

HCURSOR WINAPI
GetCursor(void)
{
    APISTR((LF_APICALL,"GetCursor: \n"));
    APISTR((LF_APIRET, "GetCursor() returning %x\n", InternalCursor.hCursor));
    return InternalCursor.hCursor;
}

#ifdef	TWIN32
BOOL	WINAPI
#else
void	WINAPI
#endif
GetCursorPos(POINT FAR *lppt)
{
    APISTR((LF_APICALL, "GetCursorPos(POINT FAR*=%p)\n", lppt));
    DRVCALL_CURSORS(PCH_GETCURSORPOS,0,0,(LPVOID)lppt);
    APISTR((LF_APIRET,"GetCursorPos: returning %x,%x\n",lppt->x,lppt->y));
#ifdef	TWIN32
	return (TRUE);
#endif
}

#ifdef	TWIN32
BOOL	WINAPI
#else
void	WINAPI
#endif
SetCursorPos(int x, int y)
{
    POINT pt;

    APISTR((LF_APICALL,"SetCursorPos: setting to %x,%x\n",x,y));
    pt.x = x;
    pt.y = y;
    DRVCALL_CURSORS(PCH_SETCURSORPOS,0,0,(LPVOID)&pt);

#ifdef	TWIN32
	APISTR((LF_APIRET, "SetCursorPos() returning TRUE\n"));
	return (TRUE);
#endif
}

void  WINAPI
ClipCursor(const RECT FAR* lprc)
{
    if (lprc)
	APISTR((LF_APICALL,"ClipCursor: setting to %x,%x,%x,%x\n",
		lprc->left,lprc->top,lprc->right,lprc->bottom));
    else
	APISTR((LF_APICALL,"ClipCursor: setting to NULL\n"));

    DRVCALL_CURSORS(PCH_SETCLIPCURSOR,0,0,(LPVOID)lprc);
}

void  WINAPI
GetClipCursor(LPRECT lprc)
{
    APISTR((LF_APICALL,"GetClipCursor(LPRECT=%p)\n",lprc));

    if (!lprc) {
         APISTR((LF_APIFAIL,"GetClipCursor: returns void\n"));
	return;
    }

    DRVCALL_CURSORS(PCH_GETCLIPCURSOR,0,0,(LPVOID)lprc);

    APISTR((LF_APIRET,"GetClipCursor: returning void\n"));
}

#ifdef TWIN32
HCURSOR WINAPI
CopyCursor(HCURSOR hCur)
{
    APISTR((LF_APISTUB,"CopyCursor(HCURSOR=%x)\n", hCur));

    return (HCURSOR)NULL;
}

#else
HCURSOR WINAPI
CopyCursor(HINSTANCE hInst, HCURSOR hCur)
{
    APISTR((LF_APISTUB,"CopyCursor(HINSTANCE=%x,HCURSOR=%x)\n",
	hInst,hCur));

    return (HCURSOR)NULL;
}
#endif

static BOOL
TWIN_DefineCursorWindow()
{
    HWND hWnd,hWndFrame;

    hWnd = GetActiveWindow();
    /* Because someone didn't fixup Active window correctly */
    /* when a window is destroyed, e.g., mdidemo ljw 5/9/96 */
    if (!IsWindow(hWnd)) {
	hWnd = 0;
    }
    else {
	hWnd = GetTopLevelAncestor(hWnd);
	if(IsWindow(hWnd)) {
		if ((hWndFrame = GetWindowFrame(hWnd)))
	    		hWnd = hWndFrame;
		else
			hWnd = 0;
	}
    }
    if (hWnd && InternalCursor.hWnd == hWnd) {
	return FALSE;
    }
    else {
	InternalCursor.hWnd = hWnd;
	return TRUE;
    }
}

void
TWIN_SetCursorWindow(HWND hWnd)
{
    if (IsWindow(hWnd))
	InternalCursor.hWnd = hWnd;
    else
	InternalCursor.hWnd = (HWND)0;
}

HWND
TWIN_GetCursorWindow(void)
{
    return InternalCursor.hWnd;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: UsrCursor.c,v $
** Revision 1.6  1999/12/09 12:00:50  robf
**
**
** bug fixes, mostly with error logging
**
** Revision 1.5  1999/12/09 10:50:11  robf
** validate hwnd
**
** Revision 1.4  1999/11/29 22:13:27  robf
** modified cursor ownership/fallback
**
** Revision 1.3  1999/09/29 18:17:26  jsheets
** Straightened out some logging messages
**
** Revision 1.2  1999/07/09 21:55:45  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/

