/*--------------------------------------------------------------------------
**      WinMain.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/WinMain.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/WinMain.c,v 1.5 1999/09/28 20:46:30 jsheets Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*
ChangeLog
	Apr 20 1998
	Added code to pop the first word off lpCmdLine passed to WinMain,
	so applications see correct arguments. Fixes the bug that causes
	notepad.exe and write.exe to try to load xwin as the filename to
	load.	
 */

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "windowsx.h"
#include "commdlg.h"
#include "Log.h"
#include "kerndef.h"
#include "Resources.h"
#include "Module.h"
#include "WinMain.h"

/* This is to determine whether this object file has been linked in */
BOOL bIsXWin = TRUE;

extern LPSTR strpbrkr(LPCSTR,LPCSTR);
extern HANDLE TWIN_LoadFile(WORD,LPSTR,LPARAM);
extern short WINAPI WSysGetExe(HWND, LPSTR, int);
BOOL FAR PASCAL CenterPopup( HWND, HWND);

extern int __argc;
extern char **__argv;

int FAR PASCAL   RunDlgFn( HWND, WORD, WPARAM, LPARAM );

HINSTANCE	hInstance;
char szText[256];

int
GetRunFile()
{
   FARPROC        lpProc;          
   int rc;

   lpProc = MakeProcInstance( (FARPROC)RunDlgFn, hInstance );
   rc = DialogBox( hInstance, MAKEINTRESOURCE(IDD_DIALOG), 0, lpProc );
   FreeProcInstance( lpProc );
   return rc;
}

int WINAPI
WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmdLine, int nCmdShow)
{
    int    rc;
    LPSTR  lpszFileName=0;

    hInstance = hInst;
    APISTR((LF_APICALL,"WinMain(HINSTANCE=%x,HINSTANCE=%x,LPSTR=%s,int=%x)\n",
	hInstance,hPrev,lpCmdLine,nCmdShow));

    if (__argc > 1) {
    	    char  *p;

	    lpszFileName = __argv[1];
	    /* skip over the command.exe string */
	    p = strchr(lpCmdLine,' ');

	    lpCmdLine = 0;

	    if (p) {
		p++;
	        p = strchr(p,' ');
		if (p) { 
		   p++;
		   lpCmdLine = p;
	        }
	    }
	
    } else {
	    szText[0] = 0;
	    lpszFileName = szText;
	    lpCmdLine = 0;

	    if (GetRunFile() != IDOK )
		return ( 0 );
    }

    rc = TWIN_LoadFile(ILL_BINARY|ILL_APPL|ILL_FILENAME|ILL_NOFORK,
					lpCmdLine,(LPARAM)lpszFileName);
    APISTR((LF_APIRET,"WinMain: returns int %d\n",rc));
    return 0;
}

int FAR PASCAL 
RunDlgFn( HWND hDlg, WORD wMsg, WPARAM wParam, LPARAM lParam )
{
   BOOL         bResult; 
   HWND		hListBox;
 
   bResult = TRUE;
   
   switch( wMsg ) {
   case WM_INITDIALOG : /* initialize dialog box */
       {
	CenterPopup(hDlg,0);
	hListBox = GetDlgItem(hDlg,IDC_COMBO);
        SendMessage(hListBox,CB_INSERTSTRING,-1,(LONG)(LPSTR)"winver");
        SendMessage(hListBox,CB_INSERTSTRING,-1,(LONG)(LPSTR)"notepad");
        SendMessage(hListBox,CB_INSERTSTRING,-1,(LONG)(LPSTR)"calc");

	SendMessage(hListBox,CB_SELECTSTRING,-1,(LONG)(LPSTR)"winver");
      }
      break;

   case WM_COMMAND : /* window command */
      
      /* process sub-message */
      switch( wParam ) {
         case IDOK : 
         	EndDialog( hDlg, IDOK );
        	GetDlgItemText( hDlg, IDC_COMBO, szText, sizeof(szText) );
		return IDOK;

      	 case IDCANCEL :
         	EndDialog( hDlg, IDCANCEL );
		return IDCANCEL;

	 case IDBROWSE :
		if( WSysGetExe(hDlg, szText, 80) == IDOK) {
			hListBox = GetDlgItem(hDlg,IDC_COMBO);
        		SendMessage(hListBox,CB_INSERTSTRING,-1,(LONG)szText);
			SendMessage(hListBox,CB_SELECTSTRING,-1,(LONG)szText);
		}
	        break; 
         default : /* something else */
         	break;
      }
      
      break;
   default :
      bResult = FALSE;
      break;
   }  
   
   /* return result */
   return( bResult );
}

/*
 * CenterPopup( hWnd, hParentWnd ) : BOOL
 *
 *    hWnd              window handle
 *    hParentWnd        parent window handle
 *
 * This routine centers the popup window in the screen or display
 * using the window handles provided.  The window is centered over
 * the parent if the parent window is valid.  Special provision
 * is made for the case when the popup would be centered outside
 * the screen - in this case it is positioned at the appropriate
 * border.
 *
 */

BOOL FAR PASCAL CenterPopup( 
      HWND     hWnd,
      HWND     hParentWnd
   )
{
   /* local variables */
   int      xPopup;   
   int      yPopup;   
   int      cxPopup;  
   int      cyPopup;  
   int      cxScreen; 
   int      cyScreen; 
   int      cxParent; 
   int      cyParent; 
   RECT     rcWindow; 

   /* retrieve main display dimensions */
   cxScreen = GetSystemMetrics( SM_CXSCREEN );
   cyScreen = GetSystemMetrics( SM_CYSCREEN );

   /* retrieve popup rectangle  */
   GetWindowRect( hWnd, (LPRECT)&rcWindow );

   /* calculate popup extents */
   cxPopup = rcWindow.right - rcWindow.left;
   cyPopup = rcWindow.bottom - rcWindow.top;

   /* calculate bounding rectangle */
   if ( hParentWnd ) {

      /* retrieve parent rectangle */
      GetWindowRect( hParentWnd, (LPRECT)&rcWindow );

      /* calculate parent extents */
      cxParent = rcWindow.right - rcWindow.left;
      cyParent = rcWindow.bottom - rcWindow.top;

      /* center within parent window */
      xPopup = rcWindow.left + ((cxParent - cxPopup)/2);
      yPopup = rcWindow.top + ((cyParent - cyPopup)/2);

      /* adjust popup x-location for screen size */
      if ( xPopup+cxPopup > cxScreen )
         xPopup = cxScreen - cxPopup;

      /* adjust popup y-location for screen size */
      if ( yPopup+cyPopup > cyScreen )
         yPopup = cyScreen - cyPopup;

   } else {

      /* center within entire screen */
      xPopup = (cxScreen - cxPopup) / 2;
      yPopup = (cyScreen - cyPopup) / 2;

   }

   /* move window to new location & display */
   MoveWindow(
      hWnd,
      ( xPopup > 0 ) ? xPopup : 0,
      ( yPopup > 0 ) ? yPopup : 0,
      cxPopup,
      cyPopup,
      TRUE
   );

   /* normal return */
   return( TRUE );

}

/*------------------------- < Full Revision History > ----------------------
** $Log: WinMain.c,v $
** Revision 1.5  1999/09/28 20:46:30  jsheets
** Added missing WINAPI's and straightened up some logging statements
**
** Revision 1.4  1999/07/09 21:55:45  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
